/****************************************************************/
// Atelier.cpp: implementation of the CAtelier class.
//
// La classe CAtelier regroupe l'ensemble des initialisations, crations
// d'objets et fonctions principales. Le code contenu dans cette classe
// est typiquement le code que l'on aurait plac dans la classe CDoc
// du projet. Ce choix est justifie par le fait :
// Il suffit d'instancier un objet de la classe CAtelier pour avoir accs
// aux fonctions principales, ainsi si le projet devait servir dans une
// autre application (MDI,SDI ou autre) la procdure serait extrmement
// simplifie.
//
/****************************************************************/


/********************/
// INCLUSION
/********************/

#include "stdafx.h"
#include "MyApp.h"
#include "Atelier.h"
#include "ListeZones.h"
#include "MyAppDoc.h"
#include "constante.h"
#include "Bloc.h"
#include "DialogParamTAO.h"
#include "ComposanteConnexe.h"
#include <direct.h>
#include "MainFrm.h"


#define BINARISATION_DEBUT	150
#define BINARISATION_FIN	186
#define BINARISATION_PAS	6

/********************/
// PRECOMPILATION
/********************/

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

/********************/
// CONSTRUCTEUR
/********************/

CAtelier::CAtelier(CDocument *p)
{

	pDoc = (CMyAppDoc *)p;

	ImageOrigine = new CImage();
	ImageCourante = new CImage();
	ImageTemporaire = new CImage();
	ImageBinarisee = new CImage();
	liste_de_zones = new CListeZones(pDoc);
	

	
	nom_image="";

}


/********************/
// DESTRUCTEUR
/********************/

CAtelier::~CAtelier()
{

	delete ImageOrigine;
	delete ImageCourante;
	delete ImageTemporaire;
	delete ImageBinarisee;

	delete liste_de_zones;

}


/********************/
// METHODES
/********************/


/********************************************************************/
/* Nom mthode : getImageOrigine                                     
/*                                                                  
/* Fonction : retourne l'image d'origine
/*                                                       
/*                                                           
/********************************************************************/

CImage * CAtelier::getImageOrigine()
{

	return ImageOrigine;

}


/********************************************************************/
/* Nom mthode : getImageCourante                                     
/*                                                                  
/* Fonction : retourne l'image courante
/*                                                       
/*                                                           
/********************************************************************/

CImage * CAtelier::getImageCourante()
{

	return ImageCourante;

}


/********************************************************************/
/* Nom mthode : getImageTemporaire                                     
/*                                                                  
/* Fonction : retourne l'image temporaire
/*                                                       
/*                                                           
/********************************************************************/

CImage * CAtelier::getImageTemporaire()
{

	return ImageTemporaire;

}


/********************************************************************/
/* Nom mthode : getImageBinarisee                                     
/*                                                                  
/* Fonction : retourne l'image temporaire
/*                                                       
/*                                                           
/********************************************************************/

CImage * CAtelier::getImageBinarisee()
{

	return ImageBinarisee;

}


/********************************************************************/
/* Nom mthode : Sauvegarder                                     
/*                                                                  
/* Fonction : sauvegarde l'image courante dans le format spcifi par type
/*                                                       
/*                                                           
/********************************************************************/

void CAtelier::Sauvegarder(CString type,CString nom)
{

	ImageCourante->Sauvegarder(type,nom);

}


/********************************************************************/
/* Nom mthode : Charger                                     
/*                                                                  
/* Fonction : image d'origine est initialise avec l'image prcise dans path
/*            image courante est initialise avec l'image prcise dans path
/*            image courante convertie en niveaux de gris                                           
/*                                                           
/********************************************************************/

void CAtelier::Charger(CString path)
{
	CMyAppApp* pAppTemp = ((CMyAppApp*)AfxGetApp());
	nom_image = path;

	CString ext, nom_seul, nom_img_cour, path_Courante;
	ext = path.Right(path.GetLength() - path.ReverseFind('.') - 1);
	nom_seul = path.Left(path.ReverseFind('.')).Right(path.Left(path.ReverseFind('.')).GetLength() - path.ReverseFind('\\') -1);
	nom_img_cour.Format("%s_COUR.%s",nom_seul,ext);
	path_Courante.Format("%s\\imgCourantesTmp\\%s",pAppTemp->projet->getParamProj()->getM_repTravail(),nom_img_cour);

	if (!ImageOrigine->Charger(path)) return;

	if (PathFileExists(path))
	{
		ImageCourante->Charger(path);
		ImageCourante->ConvertirNdg();
	}
	

}

void CAtelier::DeCharger()
{
	CMyAppApp* pAppTemp = ((CMyAppApp*)AfxGetApp());
	CString path;

	path = nom_image;

	CString ext, nom_seul, nom_img_cour, nom_img_temp, nom_img_bin, pathGeneral, path_CouranteSsImg, path_TemporaireSsImg, path_BinariseeSsImg, path_Courante, path_Temporaire, path_Binarisee;
	ext = path.Right(path.GetLength() - path.ReverseFind('.') - 1);
	nom_seul = path.Left(path.ReverseFind('.')).Right(path.Left(path.ReverseFind('.')).GetLength() - path.ReverseFind('\\') -1);

	nom_img_cour.Format("%s_COUR.%s",nom_seul,ext);
	nom_img_temp.Format("%s_TEMP.%s",nom_seul,ext);
	nom_img_bin.Format("%s_BIN.%s",nom_seul,ext);

	pathGeneral.Format("%s\\imgTmp",pAppTemp->projet->getParamProj()->getM_repTravail());
	path_CouranteSsImg.Format("%s\\Courantes",pathGeneral);
	path_TemporaireSsImg.Format("%s\\Temporaires",pathGeneral);
	path_BinariseeSsImg.Format("%s\\Binarisees",pathGeneral);

	path_Courante.Format("%s\\imgTmp\\Courantes\\%s",pAppTemp->projet->getParamProj()->getM_repTravail(),nom_img_cour);
	path_Temporaire.Format("%s\\imgTmp\\Temporaires\\%s",pAppTemp->projet->getParamProj()->getM_repTravail(),nom_img_temp);
	path_Binarisee.Format("%s\\imgTmp\\Binarisees\\%s",pAppTemp->projet->getParamProj()->getM_repTravail(),nom_img_bin);

	if (!PathFileExists(pathGeneral))
	{
		_mkdir(pathGeneral);
	}

	if (!PathFileExists(path_CouranteSsImg)) _mkdir(path_CouranteSsImg);
	if (!PathFileExists(path_TemporaireSsImg)) _mkdir(path_TemporaireSsImg);
	if (!PathFileExists(path_BinariseeSsImg)) _mkdir(path_BinariseeSsImg);

	//sauvegarder les images
	ImageCourante->Sauvegarder(ext,path_Courante);
	if (ImageTemporaire->m_pImage != NULL) ImageTemporaire->Sauvegarder(ext,path_Temporaire);
	if (ImageBinarisee->m_pImage != NULL) ImageBinarisee->Sauvegarder(ext,path_Binarisee);

	//supprimer les copies d'images en mmoire
	ImageOrigine->DeCharger(path);
	ImageCourante->DeCharger(path_Courante);
	if (ImageTemporaire->m_pImage != NULL) ImageTemporaire->DeCharger(path_Temporaire);
	if (ImageBinarisee->m_pImage != NULL) ImageBinarisee->DeCharger(path_Binarisee);


}


void CAtelier::ReCharger()
{
	//recopie les images dans la mmoire
	ImageOrigine->ReCharger();
	ImageCourante->ReCharger();
	ImageCourante->ConvertirNdg();
	ImageTemporaire->ReCharger();
	if (ImageTemporaire->getNbBitsParPixel() != -1)
	{
		ImageTemporaire->ConvertirNdg();
	}
	ImageBinarisee->ReCharger();
	if (ImageBinarisee->getNbBitsParPixel() != -1)
	{
		ImageBinarisee->ConvertirNdg();
	}
}


/********************************************************************/
/* Nom mthode : RetablirOrigine                                     
/*                                                                  
/* Fonction : copie l'image d'origine dans l'image courante
/*                                                       
/*                                                           
/********************************************************************/

void CAtelier::RetablirOrigine()
{

	ImageCourante->Copier(ImageOrigine);

}


/********************************************************************/
/* Nom mthode : Afficher_Image_Courante                                     
/*                                                                  
/* Fonction : affiche l'image courante sur l'cran en fonction de la
/*            valeur de zoom zoomcoeff
/*
/* ecran : contexte de priphrique reprsentant la zone de l'cran
/*         o il est possible de dessiner
/* zoomcoeff : valeur du facteur de zoom                                          
/*                                                           
/********************************************************************/

void CAtelier::Afficher_Image_Courante(CDC *ecran, double zoomcoeff)
{

	ImageCourante->Afficher(ecran,zoomcoeff);

}


/********************************************************************/
/* Nom mthode : Afficher_Image_Origine                                     
/*                                                                  
/* Fonction : affiche l'image d'origine sur l'cran en fonction de la
/*            valeur de zoom zoomcoeff
/*
/* ecran : contexte de priphrique reprsentant la zone de l'cran
/*         o il est possible de dessiner
/* zoomcoeff : valeur du facteur de zoom                                          
/*                                                           
/********************************************************************/

void CAtelier::Afficher_Image_Origine(CDC *ecran, double zoomcoeff)
{

	ImageOrigine->Afficher(ecran,zoomcoeff);

}


/********************************************************************/
/* Nom mthode : DetectionContours                                     
/*                                                                  
/* Fonction : dtection des contours
/*            extraction des composantes connexes
/*            tude de taille et labelisation primitive BRUIT/TEXTE/IMAGE
/*
/* PDX : largeur max d'une petite CC
/* PDY : hauteur max d'une petite CC
/* GDX : largeur min d'une grande CC
/* GDY : hauteur min d'une grande CC
/*                                                       
/*                                                           
/********************************************************************/

void CAtelier::DetectionContours(int PDX, int PDY, int GDX, int GDY)
{

	ImageCourante->EffacerBords();
	liste_de_zones->DetectionContours(ImageCourante);
	liste_de_zones->EtiquetterZone(PDX,PDY,GDX,GDY);

}


/********************************************************************/
/* Nom mthode : Affiche_CC                                     
/*                                                                  
/* Fonction : affiche le rectangle englobant de chaque composante 
/*            connexe dtecte
/*			  Utilise la fonction Affiche_CC de la classe CListeZones
/*
/* ecran : contexte de priphrique sur lequel afficher les rectangles
/* zoomcoeff : facteur de zoom utilis
/* k : numro de la CC  afficher, si -1 tout afficher  
/*
/*                                                           
/********************************************************************/

void CAtelier::Affiche_CC(CDC *ecran, double zoomcoeff, int k)
{

	liste_de_zones->Affiche_CC(ecran,zoomcoeff,k);

}


/********************************************************************/
/* Nom mthode : Affiche_Zone                                     
/*                                                                  
/* Fonction : affiche le rectangle englobant de chaque zone 
/*            connexe dtecte
/*
/* ecran : contexte de priphrique sur lequel afficher les rectangles
/* zoomcoeff : facteur de zoom utilis
/*                                                      
/*                                                           
/********************************************************************/

void CAtelier::Affiche_Zone(CDC *ecran, double zoomcoeff)
{

	liste_de_zones->Affiche_Zone(ecran,zoomcoeff);

}


/********************************************************************/
/* Nom mthode : Plage                                     
/*                                                                  
/* Fonction : application de l'algorithme des plages -> carte de niveaux de 
/*            gris correspondant au nombre de pixels blancs que l'on a t 
/*            capable d'aligner horizontalement et verticalement
/*                                                           
/********************************************************************/

void CAtelier::Plage()
{	

	int x,y,largeur,hauteur;
	
	largeur=ImageCourante->getLargeur();
	hauteur=ImageCourante->getHauteur();

	//CProgressBar pg("cration des plages...",hauteur);
	((CMainFrame*)AfxGetMainWnd())->DemarrerProgressBar("Cration des plages...",hauteur);
	CWaitCursor wc;
	bool encore;
	int compteur;
	
	int **tableau;

	tableau=new int*[hauteur];

	for(y=0;y<hauteur;y++)
	{

		tableau[y]=new int[largeur];

	}
	
	for(y=0;y<hauteur;y++)
	{

		for(x=0;x<largeur;x++)
		{

			tableau[y][x] = 0;

		}

	}

	for(y=0;y<hauteur;y++){

		for(x=0;x<largeur;x++)
		{

			if(ImageCourante->getPixel(x,y)==BLANC)
			{

				int ibis=x;
				encore = true;
				compteur = 0;

				while (ibis<largeur-1 && encore)
				{

					ibis++;

					if(ImageCourante->getPixel(ibis,y)==BLANC) compteur++;

					else encore = false; 

				}

				int a;

				for (a=x;a<=ibis;a++)
				{

					tableau[y][a] = compteur;

				}

				x = ibis;

			}
			else
			{

				tableau[y][x]=0;

			}

		}

		//pg.m_progress.StepIt();
		((CMainFrame*)AfxGetMainWnd())->IncrementerProgressBar();

	}

	for (x=0;x<largeur;x++){

		for (y=0;y<hauteur;y++)
		{

			if(ImageCourante->getPixel(x,y)==BLANC)
			{

				int ibis=y;
				encore = true;
				compteur = 0;

				while (ibis<hauteur-1 && encore)
				{

					ibis++;

					if(ImageCourante->getPixel(x,ibis)==BLANC) compteur++;

					else encore = false; 

				}

				int a;

				for (a=y;a<=ibis;a++)
				{

					//if (compteur > tableau[a][x]) tableau[a][x] = compteur;
					tableau[a][x] += compteur;

				}

				y = ibis;

			}
			else
			{

				tableau[y][x]=0;

			}

		}
		//non pg.m_progress.StepIt();
	}
	
	//determination de la valeur max

	int max=0;

	for(y=0;y<hauteur;y++)
	{

		for(x=0;x<largeur;x++)
		{

			if (tableau[y][x] > max) max = tableau[y][x];

		}
	}

	//normalisation des valeurs
	for(y=0;y<hauteur;y++)
	{

		for(x=0;x<largeur;x++)
		{

			ImageCourante->setPixel(x,y,255-(int)((double)tableau[y][x] / (double)max * 255.0));

		}
	}
	
	for(y=0;y<hauteur;y++)
	{

		delete tableau[y];

	}

	delete tableau;

}



/********************************************************************/
/* Nom mthode : PlageHorizontale                                     
/*                                                                  
/* Fonction : application de l'algorithme des plages -> carte de niveaux de
/*            gris correspondant au nombre de pixels blancs que l'on a t 
/*            capable d'aligner horizontalement
/*                                                           
/********************************************************************/

void CAtelier::PlageHorizontale()
{	

	int x,y,largeur,hauteur;
	
	largeur=ImageCourante->getLargeur();
	hauteur=ImageCourante->getHauteur();

	//CProgressBar pg("cration des plages...",largeur);
	((CMainFrame*)AfxGetMainWnd())->DemarrerProgressBar("Cration des plages...",largeur);
	CWaitCursor wc;
	bool encore;
	int compteur;
	
	int **tableau;

	tableau=new int*[hauteur];

	for(y=0;y<hauteur;y++)
	{

		tableau[y]=new int[largeur];

	}
	
	for(y=0;y<hauteur;y++)
	{

		for(x=0;x<largeur;x++)
		{

			tableau[y][x] = 0;

		}

	}

	
	for (x=0;x<largeur;x++){

		for (y=0;y<hauteur;y++)
		{

			if(ImageCourante->getPixel(x,y)==BLANC)
			{
				int ibis=y;
				encore = true;
				compteur = 0;

				while (ibis<hauteur-1 && encore)
				{
					ibis++;

					if(ImageCourante->getPixel(x,ibis)==BLANC) compteur++;

					else encore = false; 

				}

				int a;

				for (a=y;a<=ibis;a++)
				{
					//if (compteur > tableau[a][x]) tableau[a][x] = compteur;
					tableau[a][x] += compteur;
				}

				y = ibis;

			}
			else
			{

				tableau[y][x]=0;

			}
		}

		//pg.m_progress.StepIt();
		((CMainFrame*)AfxGetMainWnd())->IncrementerProgressBar();

	}
	
	//determination de la valeur max
	int max=0;

	for(y=0;y<hauteur;y++)
	{
		for(x=0;x<largeur;x++)
		{
			if (tableau[y][x] > max) max = tableau[y][x];
		}
	}


	//normalisation des valeurs
	for(y=0;y<hauteur;y++)
	{
		for(x=0;x<largeur;x++)
		{
			ImageCourante->setPixel(x,y,255-(int)((double)tableau[y][x] / (double)max * 255.0));
		}
	}
	

	for(y=0;y<hauteur;y++)
	{
		delete tableau[y];
	}

	delete tableau;

}


/********************************************************************/
/* Nom mthode : PlageVerticale                                     
/*                                                                  
/* Fonction : application de l'algorithme des plages -> carte de niveaux de 
/*            gris correspondant au nombre de pixels blancs que l'on a t 
/*            capable d'aligner verticalement
/*                                                           
/********************************************************************/

void CAtelier::PlageVerticale()
{	

	int x,y,largeur,hauteur;
	
	largeur=ImageCourante->getLargeur();
	hauteur=ImageCourante->getHauteur();

	//CProgressBar pg("Cration des plages...",hauteur);
	((CMainFrame*)AfxGetMainWnd())->DemarrerProgressBar("Cration des plages...",hauteur);
	CWaitCursor wc;
	bool encore;
	int compteur;
	
	int **tableau;

	tableau=new int*[hauteur];

	for(y=0;y<hauteur;y++)
	{

		tableau[y]=new int[largeur];

	}
	
	for(y=0;y<hauteur;y++)
	{

		for(x=0;x<largeur;x++)
		{
			tableau[y][x] = 0;
		}

	}

	for(y=0;y<hauteur;y++){

		for(x=0;x<largeur;x++)
		{
			if(ImageCourante->getPixel(x,y)==BLANC)
			{
				int ibis=x;
				encore = true;
				compteur = 0;

				while (ibis<largeur-1 && encore)
				{
					ibis++;

					if(ImageCourante->getPixel(ibis,y)==BLANC) compteur++;

					else encore = false; 

				}

				int a;

				for (a=x;a<=ibis;a++)
				{
					tableau[y][a] = compteur;
				}

				x = ibis;

			}
			else
			{

				tableau[y][x]=0;

			}

		}

		//pg.m_progress.StepIt();
		((CMainFrame*)AfxGetMainWnd())->IncrementerProgressBar();

	}

	//determination de la valeur max
	int max=0;

	for(y=0;y<hauteur;y++)
	{
		for(x=0;x<largeur;x++)
		{
			if (tableau[y][x] > max) max = tableau[y][x];
		}
	}


	//normalisation des valeurs
	for(y=0;y<hauteur;y++)
	{
		for(x=0;x<largeur;x++)
		{
			ImageCourante->setPixel(x,y,255-(int)((double)tableau[y][x] / (double)max * 255.0));
		}
	}
	

	for(y=0;y<hauteur;y++)
	{
		delete tableau[y];
	}

	delete tableau;

}


/********************************************************************/
/* Nom mthode : Fusion                                     
/*                                                                  
/* Fonction : application de l'algorithme de fusion dans le sens horizontal 
/*            puis vertical
/*
/* seuil_h	:	seuil de fusion horizontale 
/* type_h	:	type des zones concernees par la fusion horizontale
/* seuil_v	:	seuil de fusion verticale
/* type_v	:	type des zones concernees par la fusion verticale
/*                                                           
/********************************************************************/

void CAtelier::Fusion(int seuil_h, CString type_h,int seuil_v, CString type_v)
{

	FusionHorizontale(seuil_h,type_h);
	FusionVerticale(seuil_v,type_v);
	
}


/********************************************************************/
/* Nom mthode : FusionHorizontale                                     
/*                                                                  
/* Fonction : application de l'algorithme de fusion dans le sens horizontal
/*            
/*
/* seuil	:	seuil de fusion horizontale
/* type	:	type des zones concernees par la fusion horizontale
/*                                                           
/********************************************************************/

void CAtelier::FusionHorizontale(int seuil, CString type)
{

	int oh;

	do
	{
		oh=liste_de_zones->Ppv_horizontal(ImageCourante,seuil,type);	
		
	}

	while (oh != 0 );

}



/********************************************************************/
/* Nom mthode : FusionVerticale                                     
/*                                                                  
/* Fonction : application de l'algorithme de fusion dans le sens Vertical
/*            
/*
/* seuil	:	seuil de fusion Verticale
/* type	:	type des zones concernees par la fusion Verticale
/*                                                           
/********************************************************************/

void CAtelier::FusionVerticale(int seuil, CString type)
{

	int oh;

	do
	{
		oh=liste_de_zones->Ppv_vertical(ImageCourante,seuil,type);
	}

	while (oh != 0 );

}



/********************************************************************/
/* Nom mthode : FusionHorizontaleCDG                                     
/*                                                                  
/* Fonction : application de l'algorithme de fusion dans le sens horizontal
/*            sur les zones de type texte en utilisant la distance entre les
/*            centre de gravit de chaque zone (experimental)
/*
/* seuil	:	seuil de fusion horizontale
/*                                                           
/********************************************************************/

void CAtelier::FusionHorizontaleCDG(int seuil)
{

	int oh;

	do
	{
		oh=liste_de_zones->Ppv_horizontal_CDG(ImageCourante,seuil);	
		
	}

	while (oh != 0 );
	
}


/********************************************************************/
/* Nom mthode : FusionVerticaleCDG                                     
/*                                                                  
/* Fonction : application de l'algorithme de fusion dans le sens vertical
/*            sur les zones de type texte en utilisant la distance entre les
/*            centre de gravit de chaque zone (experimental)
/*
/* seuil	:	seuil de fusion vertical
/*                                                           
/********************************************************************/

void CAtelier::FusionVerticaleCDG(int seuil)
{

	int oh;

	do
	{
		oh=liste_de_zones->Ppv_vertical_CDG(ImageCourante,seuil);
	}

	while (oh != 0 );

}


/********************************************************************/
/* Nom mthode : Sauvegarder_Images                                     
/*                                                                  
/* Fonction : sauvegarde de l'image courante, origine ou temporaire selon
/*            ce qui est spcifi
/*                                                           
/********************************************************************/

void CAtelier::Sauvegarder_Images(int mappage, int delta, int i/*=0*/)
{
	//si i=0, on utilise l'image d'origine pour decouper les images
	//si i=1 l'image courante
	//si i=2 l'image temporaire

	//extraction du rapport de mappage
	//mappage spcifie la nouvelle largeur de la page
	double rapport = (double)mappage / ImageCourante->getLargeur();
	

	//recup pointeur sur la bonne image  dcouper
	CImage *im = NULL;

	if (i==0) im = getImageOrigine();

	if (i==1) im = getImageCourante();

	if (i==2) im = getImageTemporaire();


	//sauvegarde de chaque zone identifie
	liste_de_zones->DecoupageImages(im, rapport, delta);
}




/********************************************************************/
/* Nom mthode : SegmentationImageTexte                                     
/*                                                                  
/* Fonction : permet la segementation entre les images et le texte
/*            
/*                                                           
/********************************************************************/

void CAtelier::SegmentationImageTexte()
{
	//s'enregistre dans le scnarioTrait car cette fonction n'est appele que lors des traitements
	//si la case traiter le texte est cocher (dans parametres)

	//copie de l'image courante dans l'image temporaire
	CRegleSR *r1 = new CRegleSR(true);
	((CMyAppApp*)AfxGetApp())->projet->getLaboTrait()->getScenarioTrait()->Ajouter(r1);
	r1->Executer(pDoc);

	//coloriage des zones de type TEXTE en blanc
	CRegleSupType *r2=new CRegleSupType("TEXTE",BLANC,false);
	((CMyAppApp*)AfxGetApp())->projet->getLaboTrait()->getScenarioTrait()->Ajouter(r2);
	r2->Executer(pDoc);

	//coloriage des zones de type TEXTEI en blanc
	CRegleSupType *r3=new CRegleSupType("TEXTEI",BLANC,false);
	((CMyAppApp*)AfxGetApp())->projet->getLaboTrait()->getScenarioTrait()->Ajouter(r3);
	r3->Executer(pDoc);

	//application de l'algo des plages
	CReglePlage *r4=new CReglePlage();
	((CMyAppApp*)AfxGetApp())->projet->getLaboTrait()->getScenarioTrait()->Ajouter(r4);
	r4->Executer(pDoc);

	//binarisation de l'image
	CRegleBin *r5=new CRegleBin(210);
	((CMyAppApp*)AfxGetApp())->projet->getLaboTrait()->getScenarioTrait()->Ajouter(r5);
	r5->Executer(pDoc);

	//cration du type BRUIT
	CRegleType *r6=new CRegleType("BRUIT",RGB(255,255,0));
	((CMyAppApp*)AfxGetApp())->projet->getLaboTrait()->getScenarioTrait()->Ajouter(r6);
	r6->Executer(pDoc);

	//calcul de la densite
	CRegleCC *r7 = new CRegleCC("TEXTEI","BRUIT","TEXTE",-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,0.1,1,-1,-1);
	((CMyAppApp*)AfxGetApp())->projet->getLaboTrait()->getScenarioTrait()->Ajouter(r7);
	r7->Executer(pDoc);
	
	//collage de l'image temporaire dans l'image courante
	CRegleSR *r8 = new CRegleSR(false);
	((CMyAppApp*)AfxGetApp())->projet->getLaboTrait()->getScenarioTrait()->Ajouter(r8);
	r8->Executer(pDoc);

	//elimination des zones de type bruit sans altration de l'image
	CRegleSupType *r9=new CRegleSupType("BRUIT",INCOLOR, true);
	((CMyAppApp*)AfxGetApp())->projet->getLaboTrait()->getScenarioTrait()->Ajouter(r9);
	r9->Executer(pDoc);
}








/********************************************************************/
/* Nom mthode : GenererDicoEtXML                                     
/*                                                                  
/* Fonction : gnre le dictionnaire des formes et les fichiers XML qui
/*            permettront d'effectuer par la suite la TAO
/*            
/* type_zone : le nom de la zone sur laquelle on veut appliquer
/*             les algos permettant la gnration des fichiers
/* seuil : le seuil  appliquer lors de la comparaison des CC
/*         permettant leur classification
/*                                                          
/********************************************************************/

// ################ this function generates the dictionary of forms and XML files 
// ################ will be made by following the TAO
// ################ type_zone is the name of the area on which you want to apply
// ################ the algos for the generation of file is below
// ################ threshold is to apply when coparing CC and allow their classification



void CAtelier::GenererDicoEtXML (CString type_zone, double seuil)
{

	// ######### update attribute of the single instance of the class
	// mise  jour des attributs de l'unique instance de la classe CListeCC
	((CMyAppApp*)(AfxGetApp()))->projet->getLaboProj()->liste_permanente->set_type_zone(type_zone);
	((CMyAppApp*)(AfxGetApp()))->projet->getLaboProj()->liste_permanente->set_seuil(seuil);

	//CProgressBar pg("Cration du dico et des fichiers XML...",liste_de_zones->liste->GetCount());
	((CMainFrame*)AfxGetMainWnd())->DemarrerProgressBar("Cration du dico et des fichiers XML...",liste_de_zones->liste->GetCount());
	CWaitCursor wc;


	POSITION p1,p2;

	for (p1 = liste_de_zones->liste->GetHeadPosition();p1 != NULL;)
	{
		CZone *z = (CZone *)(liste_de_zones->liste->GetNext(p1));

		if (z->getClasse()->getNomClasse() == type_zone)
		{

			CBloc *bloc = new CBloc(z->get_liste_elements());

			/***************/
			/* PFE 2006 Deb
			/***************/

			// Decomposition du bloc d'lment en ligne
			// On test avant si la dcomposition n'a pas dj tait effectue
			// This function is responsible for line segmentation and detection, class detection, character segmentation and detection

			if(bloc->getDecomposition_En_Ligne_Effectue() == false)
			{
				bloc->Decomposer_en_lignes(); 
			}

			/***************/
			/* Rachit Agrawal 2009
			/***************/
			// This function is responsible for accent merging
			bloc->Rachit1();
			// This function is responsible for word segmentation
			bloc->Rachit2(pDoc);
			/***************/
			/* Rachit Agrawal 2009
			/***************/


			/***************/
			/* PFE 2006 Fin
			/***************/
			/*
			bloc->Ordonner_lignes();

			for (p2 = bloc->get_liste_lignes()->GetHeadPosition(); p2 != NULL;)
			{

				CLigne *ligne = (CLigne *)(bloc->get_liste_lignes()->GetNext(p2));

				// ########## these all functions are defined in ligne.cpp

				ligne->Supprimer_accentuation();  // ########## this function is used to remove the accent
				ligne->Calculer_espacement_moyen();// ################ this function is used to calculate average spacing
				ligne->Decomposer_en_mots();// ################ decomposing words

			}
		
			// ########## This function is responsible for creation of XML files
			*/
			bloc->Creer_structure_XML(z->getRect(), pDoc, type_zone);
	
			delete bloc;

		}

		//pg.m_progress.StepIt();
		((CMainFrame*)AfxGetMainWnd())->IncrementerProgressBar();

	}
}







/*************************************************************************************************/
/* PFE 2006
/*************************************************************************************************/



/********************************************************************/
/* Nom mthode : AnalyseFineDesCC
/*                                                                  
/* Fonction : permet d'amliorer la detection des composantes connexes
/*            en faisant varier le seuil de binarisation pour la zone
/*            dfini par une CC et son voisinage (8 voisins). On compte
/*            le nombre de CC trouv pur chaque seuil et on dcide  la 
/*            fin de fusionner certain voisin avec la CC ou non
/*            
/********************************************************************/

	
/* improves the detection of connected components
/* By varying the threshold for the binarization
/* Defined by a CC and its neighborhood (8 neighbors). There are
/* The number of pure CC found on each level and decide to
/* End of merging with the neighboring CC or not
*/

void CAtelier::AnalyseFineDesZonesTextes(CString type)
{

	POSITION p1,p2;
	int ymoy=0;
	int lmoy=0;
	int hmoy=0;
	int x1=0, x2=0, y1=0, y2=0;
	double ecartL=0;
	double ecartH=0;
	double ecartCdg=0;
	int numBloc=0;
	int numLigne=0;
	int ordonneeMax=0;
	int ordonneeMin=0;
	int picMaxYHaut=0;
	int picMaxYBas=0;
	int hauteurImageCourante=0;
	int nbElementLigne=0;
	int longueur_zone_virtuelle = 15;

	//CProgressBar pg("Sauvegarde en cours...",liste_de_zones->liste->GetCount());
	((CMainFrame*)AfxGetMainWnd())->DemarrerProgressBar("Sauvegarde en cours...",liste_de_zones->liste->GetCount());
	CWaitCursor wc;

	for (p1 = liste_de_zones->liste->GetHeadPosition();p1 != NULL;) // pour chaque zone
	{
		CZone *z = (CZone *)(liste_de_zones->liste->GetNext(p1)); 

		if (z->getClasse()->getNomClasse() == type) // On prend que les zones de TEXTE
		{
			// Rcupration dans un bloc de la liste des elements de la zone
			CBloc *bloc = new CBloc(z->get_liste_elements());

			// Decomposition du bloc d'lment en ligne
			// On test avant si la dcomposition n'a pas dj tait effectue

		// ############ this function is defined in bloc.cpp and is used to decompose the lines

			if(bloc->getDecomposition_En_Ligne_Effectue() == false)
			{
				bloc->Decomposer_en_lignes(); 
			}
			// ############ this function is defined to make the lines in order

			bloc->Ordonner_lignes(); // on ordonne les lignes dans le sens de lecture
			
// ############### this function is used to set up path to the directory

			// mise en place des chemins jusqu'au rpertoires des blocs
			numBloc++;	// compteur de bloc
			numLigne=0;
			
// ############ recovery of the file name being processed

			// rcupration du nom du fichier en cours de traitement
			CString nom_fichier_encours=((CMyAppDoc*)pDoc)->m_nom; 

// ############### create the directory with the name of the image processed in the directory result of the application

			// cration du rpertoire portant le nom de l'image trait dans le rpertoire rsultat de l'appli
			CString chemin_non_image; 
			chemin_non_image.Format ("%s\\Image_%s", ((CMyAppApp*)(AfxGetApp()))->projet->getParamProj()->getM_repResultats(), nom_fichier_encours);
			_mkdir (chemin_non_image);

// ########### creation of the directory of blocks

			// cration des rpertoires blocs situ dans le rpertoire prcdement cr
			CString chemin_bloc;
			chemin_bloc.Format ("%s\\Bloc%d", chemin_non_image, numBloc);
			_mkdir (chemin_bloc);
			// fin chemin bloc


			for (p2 = bloc->get_liste_lignes()->GetHeadPosition(); p2 != NULL;) // pour chaque ligne
			{
				numLigne++; // compteur de ligne
				CLigne *l = (CLigne *)(bloc->get_liste_lignes()->GetNext(p2)); 
				nbElementLigne = l->get_liste_elements()->GetCount();

				if( nbElementLigne >= longueur_zone_virtuelle )
				{
// ########## get the height of the image 

					// rcupration de la hauteur de l'image courante
					hauteurImageCourante = getImageCourante()->getHauteur();

// ########## setting up the path for directory lines 

					// mise en place du chemin jusqu'au rpertoire des lignes
					CString chemin_ligne;
					chemin_ligne.Format ("%s\\Ligne%d", chemin_bloc, numLigne);
					_mkdir (chemin_ligne);
					// fin chemin ligne
					// rcupre pointeur sur l'image d'origine
					CImage *image = NULL;
					image = getImageOrigine(); 
					// lancement de la detection des anomalies
					l->DetectionAnomalie2(image, pDoc, chemin_ligne, numBloc, numLigne, longueur_zone_virtuelle, hauteurImageCourante);
				}
				else
				{
					// rcupration de la hauteur de l'image courante
					hauteurImageCourante = getImageCourante()->getHauteur();
					// mise en place du chemin jusqu'au rpertoire des lignes
					CString chemin_ligne;
					chemin_ligne.Format ("%s\\Ligne%d", chemin_bloc, numLigne);
					_mkdir (chemin_ligne);

					// #############  end line path
					// ############## get pointer to original image

					// fin chemin ligne
					// rcupre pointeur sur l'image d'origine
					CImage *image = NULL;
					image = getImageOrigine(); 
					// lancement de la detection des anomalies

// ############## this function is used to classify connected coponents in different classes

					l->DetectionAnomalie(image, pDoc, chemin_ligne, numBloc, numLigne, hauteurImageCourante);
				}

			}
			delete bloc;
		}
		//pg.m_progress.StepIt();

// ############" evertime for loop goes progress bar is incremented
		((CMainFrame*)AfxGetMainWnd())->IncrementerProgressBar();
	}
}



/********************************************************************/
/* Nom mthode : Sauvegarder_Images_lignes
/*                                                                  
/* Fonction : permet de sauvegarder les diffrentes lignes dans le
/*            repertoire resultat
/*
/********************************************************************/
void CAtelier::Sauvegarder_Images_lignes(int mappage, int delta, int imageChoisie)
{

	POSITION p1,p2;
	int numLigne = 0;
	int numBloc = 0;
	int largeurcourante;
	int hauteurcourante;
	int x1,x2,y1,y2;
	CRect *rectangle;
	FILE *f,*fmap,*fmapbis;

	// Rcupration du nom du fichier en cours de traitement
	CString nom_fichier_encours = ((CMyAppDoc*)pDoc)->m_nom;
	// Rcupration de l'extension du fichier en cours de traitement
	CString extension_fichier_encours = ((CMyAppDoc*)pDoc)->m_extension; 
	// Rcupration du repertoire des resultats de l'appli
	CString repertoire_resultat_appli = ((CMyAppApp*)(AfxGetApp()))->projet->getParamProj()->getM_repResultats();

	double rapport = (double)mappage / ImageCourante->getLargeur();
	

	// Rcupre un pointeur sur l'image  utiliser

	CImage *image = NULL;

	if (imageChoisie == 0) image = getImageOrigine();

	if (imageChoisie == 1) image = getImageCourante();

	if (imageChoisie == 2) image = getImageTemporaire();

	// Fin rcuperation pointeur sur l'image  utiliser

	// Cration des fichiers de coordonnes
	CString chemin_coord;
	chemin_coord.Format("%s\\coord",((CMyAppApp*)(AfxGetApp()))->projet->getParamProj()->getM_repResultats());
	_mkdir (chemin_coord);

	CString coord;
	coord.Format("%s\\%s_Lignes_xy.txt",chemin_coord,nom_fichier_encours);

	CString coordmap;
	coordmap.Format("%s\\%s_Lignes_xy_map.txt",chemin_coord,nom_fichier_encours);

	CString coordmapbis;
	coordmapbis.Format("%s\\Lignes_xy_mapbis.txt",chemin_coord,nom_fichier_encours);


	// Fin cration des fichiers de coordonne

	f = fopen (coord, "w");
	fmap = fopen(coordmap,"w");
	fmapbis = fopen(coordmapbis,"a");

	// Fin cration des fichiers de coordonnes

	// Recopie de l'image courante
	CImage *image_courante = new CImage(); // cration d'une nouvelle CImage qui va contenir l'image originale
	image_courante->Copier(image); // recopie de l'image courante


	//CProgressBar pg("Sauvegarde en cours...",liste_de_zones->liste->GetCount());
	((CMainFrame*)AfxGetMainWnd())->DemarrerProgressBar("Sauvegarde en cours...",liste_de_zones->liste->GetCount());
	CWaitCursor wc;


	// Pour chaque zone
	for (p1 = liste_de_zones->liste->GetHeadPosition();p1 != NULL;) 
	{
		numBloc++;
		
		CZone *z = (CZone *)(liste_de_zones->liste->GetNext(p1)); 
		
		// Cration du rpertoire portant le nom de l'image trait dans le rpertoire rsultat de l'appli
		CString chemin_enregistrement_image; 
		chemin_enregistrement_image.Format ("%s\\%s", ((CMyAppApp*)(AfxGetApp()))->projet->getParamProj()->getM_repResultats(), z->getClasse()->getNomClasse());
		_mkdir (chemin_enregistrement_image);

		if (z->getClasse()->getNomClasse() == "TEXTE") // On prend que les zones de TEXTE
		{

			// Rcupration dans un bloc de la liste des elements de la zone
			CBloc *bloc = new CBloc(z->get_liste_elements());

			// Decomposition du bloc d'lment en ligne
			// On test avant si la dcomposition n'a pas dj tait effectue
			if(bloc->getDecomposition_En_Ligne_Effectue() == false)
			{
				bloc->Decomposer_en_lignes(); 
			}

			// On ordonne les lignes dans le sens de lecture
			bloc->Ordonner_lignes(); 
			
			numLigne = 0;

			// Pour chaque ligne
			for (p2 = bloc->get_liste_lignes()->GetHeadPosition(); p2 != NULL;) 
			{
				numLigne++;

				CLigne *l = (CLigne *)(bloc->get_liste_lignes()->GetNext(p2)); 
				rectangle = l->getRect();
				// rcupre la coord x en haut  gauche
				x1 = rectangle->TopLeft().x;
				// rcupre la coord x en bas  droite
				x2 = rectangle->BottomRight().x;	
				// rcupre la coord y en haut  gauche 
				y1 = rectangle->TopLeft().y;
				// rcupre la coord y en bas  droite
				y2 = rectangle->BottomRight().y;	
				// rcupre la largeur de l'lment
				largeurcourante = l->getRect()->Width();
				// rcupre la hauteur de l'lment
				hauteurcourante = l->getRect()->Height();


				// Cration de l'imagette
				CImage *image_tmp = new CImage(); // cration CImage temporaire qui contiendra la sortie
				image_tmp->setTailleLxH(largeurcourante,hauteurcourante); // mise  la bonne taille
				PLFilterCrop filtre(x1,y1,x2,y2); // application d'un filtre

				try
				{
					filtre.Apply(image_courante->getPointeur(),image_tmp->getPointeur()); // recopie dans image_tmp la partie qui nous interesse de image_courante
				}
				catch(PLTextException e)
				{
					CString message;
					message.Format("Filter application error");
					AfxMessageBox(message);
					continue;
				}

				// Cration du repertoire et du nom de fichier en sortie
				CString chemin_nom_image_sortie;
				chemin_nom_image_sortie.Empty(); // vide le chemin de sortie
				chemin_nom_image_sortie.Format ("%s\\%s_Bloc_%d_Ligne_%d.%s", chemin_enregistrement_image, nom_fichier_encours, numBloc, numLigne, extension_fichier_encours); // chemin complet de sortie

				try
				{
					image_tmp->Sauvegarder(extension_fichier_encours,chemin_nom_image_sortie); // sauvegarde image dans le repertoire
					delete image_tmp;
				}
				catch(PLTextException e)
				{
					CString message;
					message.Format("Error while saving: %s",LPCTSTR(e),(LPCTSTR)e.GetCode());
					AfxMessageBox(message);
				}	
				// Fin cration imagette
				
				CString chemin_complet_fichier;
				chemin_complet_fichier.Empty();
				chemin_complet_fichier.Format("%s\\%s\\%s_%s_%02d.%s", repertoire_resultat_appli, z->getClasse()->getNomClasse(), nom_fichier_encours, z->getClasse()->getNomClasse(), numBloc, extension_fichier_encours );
				
				fprintf(f,"%s %d Ligne %d : %d %d %d %d \n", z->getClasse()->getNomClasse(), numBloc, numLigne, x1, y1, largeurcourante, hauteurcourante);
				fprintf(fmap,"%s %d Ligne %d : %d\t%d\t%d\t%d\t%s\n", z->getClasse()->getNomClasse(), numBloc, numLigne, (int)floor((double)(x1)*rapport), (int)floor((double)(y1)*rapport), (int)floor(largeurcourante*rapport), (int)floor(hauteurcourante*rapport),chemin_complet_fichier);
				fprintf(fmapbis,"%s_%s_%02d.%s %s %d,%d,%d,%d\n", nom_fichier_encours, z->getClasse()->getNomClasse(), numBloc, extension_fichier_encours, z->getClasse()->getNomClasse(), (int)floor((x1)*rapport),(int)floor((y1)*rapport),(int)floor((x2)*rapport),(int)floor((y2)*rapport));
			}

			delete bloc;

		}
		
		//pg.m_progress.StepIt();
		((CMainFrame*)AfxGetMainWnd())->IncrementerProgressBar();

	}

	delete image_courante;
	
	fclose (f);
	fclose (fmap);
	fclose (fmapbis);
}