/****************************************************************/
// Atelier.h: interface for the CAtelier class.
//
// La classe CAtelier regroupe l'ensemble des initialisations, crations
// d'objets et fonctions principales. Le code contenu dans cette classe
// est typiquement le code que l'on aurait plac dans la classe CDoc
// du projet. Ce choix est justifie par le fait :
// Il suffit d'instancier un objet de la classe CAtelier pour avoir accs
// aux fonctions principales, ainsi si le projet devait servir dans une
// autre application (MDI,SDI ou autre) la procdure serait extrmement
// simplifie.
//
/****************************************************************/

#if !defined(AFX_ATELIER_H__DCA52C2D_DBAD_469D_95A9_E6FAD253F3CB__INCLUDED_)
#define AFX_ATELIER_H__DCA52C2D_DBAD_469D_95A9_E6FAD253F3CB__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


#include "zone.h"
#include "image.h"
#include "Ligne.h"
#include "ListeZones.h"
#include "ListeCC.h"
#include "Scenario.h"


/*********************/
// CLASSE
/*********************/

class CAtelier  
{

private:

	CRect *rect;

	CString nom_image; //nom complet du fichier  traiter

	CImage *ImageOrigine; //image originale
	CImage *ImageCourante; //image sur laquelle on travaille
	CImage *ImageTemporaire; //image pour le copier/coller
	CImage *ImageBinarisee; //image obtenue aprs excution de la binarisation
	CDocument *pDoc;//pointeur sur le document


public:

	
	CListeZones *liste_de_zones;
	

	
	void SegmentationImageTexte();
	void Sauvegarder_Images(int mappage, int delta, int i=0);

	void FusionVerticale(int seuil, CString type);
	void FusionHorizontale(int seuil, CString type);
	void FusionVerticaleCDG(int seuil);
	void FusionHorizontaleCDG(int seuil);
	void Fusion(int seuil_h, CString type_h,int seuil_v, CString type_v);

	void Plage();
	void PlageHorizontale();
	void PlageVerticale();
	void Affiche_CC(CDC *ecran, double zoomcoeff, int k);
	void Affiche_Zone(CDC *ecran, double zoomcoeff);

	void GenererDicoEtXML (CString type_zone, double seuil);

	void DetectionContours(int PDX, int PDY, int GDX, int GDY);
	void Afficher_Image_Origine(CDC *ecran, double zoomcoeff);
	void Afficher_Image_Courante(CDC *ecran, double zoomcoeff);
	
	void RetablirOrigine();
	void Charger(CString path);
	void DeCharger();
	void ReCharger();
	void Sauvegarder(CString type,CString nom);

	/***************/
	/* PFE 2006 Deb
	/***************/

	void AnalyseFineDesZonesTextes(CString type);
	void Sauvegarder_Images_lignes(int mappage, int delta, int imageChoisie);

	/***************/
	/* PFE 2006 Fin
	/***************/

	CImage * getImageTemporaire();
	CImage * getImageOrigine();
	CImage * getImageCourante();
	CImage * getImageBinarisee();

	//CString getNomImage(){return nom_image;}

	//void VoisinageElement();

	CAtelier(CDocument *pDoc);
	virtual ~CAtelier();

};


#endif // !defined(AFX_ATELIER_H__DCA52C2D_DBAD_469D_95A9_E6FAD253F3CB__INCLUDED_)
