// Image.cpp: implementation of the CImage class.
//
// Regroupe toutes les actions qu'il est possible d'effectuer sur une image
//
/************************************************************************/



/***********************/
//  INCLUSION
/***********************/

#include "stdafx.h"
#include "MyApp.h"
#include "Image.h"
#include "constante.h"
#include <malloc.h>
#include "MainFrm.h"
#include <fstream>
#include <iostream>
using namespace std;

/***********************/
//  PRECOMPILATION
/***********************/

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif



/***********************/
//  DEFINITION
/***********************/


#define BLANC 255
#define NOIR 0
//#define MAX 1


/***********************/
//  CONSTRUCTEURS
/***********************/

/***************************************************************/ 
// constructeur de la classe CImage
/***************************************************************/

CImage::CImage()
{

	m_pImage = new PLWinBmp;
	binarisee=false;
	ndgrisee=false;

	bpp = -1;
}


/***************************************************************/ 
// constructeur de la classe CImage
/***************************************************************/

CImage::CImage(int largeur, int hauteur, int bpp)
{

	m_pImage = new PLWinBmp;
	m_pImage->Create(largeur,hauteur,bpp,true,true);
	binarisee=false;
	ndgrisee=false;
	this->largeur = largeur;
	this->hauteur = hauteur;
	this->bpp = bpp;
}



/***********************/
//  DESTRUCTEUR
/***********************/

/***************************************************************/ 
// destructeur de la classe CImage
/***************************************************************/

CImage::~CImage()
{

	if(m_pImage) delete m_pImage;
	m_pImage=NULL;
}


/***********************/
//  METHODES
/***********************/

/********************************************************************/
/* Nom mthode : getTaille
/*                                                                  
/* Fonction : renvoie la taille de l'image    
/*  
/* Sortie : couple de valeurs de type CSize reprsentant la hauteur
/*          et la largeur de l'image
/*                       
/********************************************************************/

CSize CImage::getTaille()
{

	return m_pImage->GetSize();

}


/********************************************************************/
/* Nom mthode : Charger
/*                                                                  
/* Fonction : mettre  disposition une image au format BMP  
/*            en fonction de n'importe quel type d'image
/*
/* path->nom complet de l'image  convertir
/*
/* Exception : la conversion au format BMP n'est pas possible
/*             le format de fichier n'est pas pris en charge
/*                       
/********************************************************************/

bool CImage::Charger(CString path)
{
	//chargement de l'image
	try
	{
		PLAnyPicDecoder m_Decoder;
		m_Decoder.MakeBmpFromFile(path,m_pImage);
		largeur = m_pImage->GetWidth();
		hauteur = m_pImage->GetHeight();
		bpp = m_pImage->GetBitsPerPixel();
		nomCompletImg = path;
		return true;

	}
	catch(PLTextException e)
	{

		AfxMessageBox("Format unvalid !");
		return false;

	}

}



void CImage::DeCharger(CString path)
{
	//dechargement de l'image
	if(m_pImage) delete m_pImage;
	m_pImage = new PLWinBmp;
	m_pImage = NULL;

	nomCompletImg = path;
}



void CImage::ReCharger()
{

	//rechargement de l'image
	if(m_pImage) delete m_pImage;
	m_pImage = new PLWinBmp;

	try
	{
		PLAnyPicDecoder m_Decoder;
		m_Decoder.MakeBmpFromFile(nomCompletImg,m_pImage);
		largeur = m_pImage->GetWidth();
		hauteur = m_pImage->GetHeight();
		bpp = m_pImage->GetBitsPerPixel();

	}
	catch(PLTextException e)
	{

		//AfxMessageBox("L'image n'a pas pu tre recharge");

	}


}


/********************************************************************/
/* Nom mthode : Sauvegarder
/*                                                                  
/* Fonction : sauvegarde l'image  de nom "nom" sous le format correspondant
/*            a l'extension "type". Limitation :  la version de paintib 
/*            n'autorise que la sauvegarde en 32 bits
/*
/* type : extension du fichier  sauvegarder
/* nom : nom de l'image  sauvegarder
/*
/* Exception : impossible de sauvegarder l'image dans le format dsir
/*             
/*                       
/********************************************************************/


void CImage::Sauvegarder(CString type, CString nom)
{

	//sauvegarde l'image dans le format
	//spcifi par type et sous le nom spcifi par nom
	//attention paintlib n'est pas fini !!!
	//oblig sauvegarder image 32 bits

	type.MakeLower();
	nom.MakeLower();

	CImage *image = new CImage();//largeur,hauteur,32);
	image->Copier32(this);
	
	try
	{
		if(type=="jpg")
		{

			PLJPEGEncoder Encoder;
			Encoder.MakeFileFromBmp(nom,image->m_pImage);

		}
		else if(type=="tif")
		{

			PLTIFFEncoder Encoder;
			Encoder.MakeFileFromBmp(nom,image->m_pImage);

		}
		else if(type=="png")
		{

			PLPNGEncoder Encoder;
			Encoder.MakeFileFromBmp(nom,image->m_pImage);

		}
		else if(type=="bmp")
		{

			PLBmpEncoder Encoder;
			Encoder.MakeFileFromBmp(nom,image->m_pImage);

		}
		else
		{

			AfxMessageBox("Format unvalid",MB_OK);
			ASSERT(false);

		}

	}
	catch(PLTextException e)
	{

			CString message;
			message.Format("Error while loading file: %s,%s",LPCTSTR(e),(LPCTSTR)e.GetCode());
			AfxMessageBox(message);

	}

	delete image;

}


/********************************************************************/
/* Nom mthode : Binariser
/*                                                                  
/* Fonction : binarise une image en 256 niveaux de gris au format BMP 
/*            en fonction du seuil pass en paramtre
/*           
/*
/* seuil : entier reprsentant la valeur de niveau de gris servant
/*         de seuil  l'algo de binarisation
/*         s'il est gal  -1 on applique la inarisation de Niblack qui nous 
/*         propose une valeur puis binarisation classique avec cette valeur
/*         si appel avec un seul param, alors binarise l'instance de l'image
/*         qui appelle cette fonction sinon le rsultat est stock dans
/*         le param im et l'image en cours non affecte
/*
/*                       
/********************************************************************/

void CImage::Binariser(int seuil,CImage *im /*=NULL*/)
{

	int x,y;
	int seuil_bin;
	if (seuil == -1) seuil_bin = Binariser_Niblack(true);
	else seuil_bin = seuil;

	//CProgressBar pg("Binarisation en cours...",hauteur);
	((CMainFrame*)AfxGetMainWnd())->DemarrerProgressBar("Binarisation en cours...",hauteur);
	CWaitCursor wc;

	for(y=0;y<hauteur;y++){

		for(x=0;x<largeur;x++){

			if(getPixel(x,y)<=seuil_bin) 
			{

				if (im == NULL) setPixel(x,y,NOIR);
				else im->setPixel(x,y,NOIR);

			}
			else 
			{	

				if (im == NULL) setPixel(x,y,BLANC);
				else im->setPixel(x,y,BLANC);

			}

		}

		//pg.m_progress.StepIt();
		((CMainFrame*)AfxGetMainWnd())->IncrementerProgressBar();
		
	}

	if (im == NULL) binarisee=true;

}


/********************************************************************/
/* Nom mthode : Inverser
/*                                                                  
/* Fonction : Inverser une image 
/*                       
/********************************************************************/

void CImage::Inverser()
{

	if (!binarisee&&!ndgrisee)
	{

		AfxMessageBox("This function works on greyscale or binarized image");
		return;

	}

	int x,y;
	
	//CProgressBar pg("Inversion en cours...",hauteur);
	((CMainFrame*)AfxGetMainWnd())->DemarrerProgressBar("Inversion en cours...",hauteur);
	CWaitCursor wc;

	for(y=0;y<hauteur;y++)
	{

		for(x=0;x<largeur;x++)
		{

			setPixel(x,y,pow(2,bpp)-1-getPixel(x,y));

		}

		//pg.m_progress.StepIt();	
		((CMainFrame*)AfxGetMainWnd())->IncrementerProgressBar();

	}
	
}

/********************************************************************/
/* Nom mthode : Binariser_Niblack
/*                                                                  
/* Fonction : algo de binarisation de Sauvola
/*            seuillage adaptatif
/*            renvoie une estimation de la valeur de seuillage  utiliser avec
/*            un algo de base
/*            
/********************************************************************/


int CImage::Binariser_Niblack(bool juste_valeur)
{

	unsigned int temp, som;
	int i,j,k,l,fenetre;
	double moy,ecart,moy_ecart,seuil,K,R;

	K=0.4;
	R=128.;
	fenetre=10;
	som=0;
	ecart=0;
	moy_ecart=0;

	unsigned char **m_imbits;
	m_imbits=NULL;
	m_imbits=(unsigned char **)malloc(sizeof(unsigned char **)*hauteur);

	for (i=0;i<hauteur;i++)
	{

		m_imbits[i]=(unsigned char *)malloc(sizeof(unsigned char *)*largeur);

	}

	for(i=0; i<hauteur; i++)
		for(j=0; j<largeur; j++) m_imbits[i][j]=getPixel(j,i);


	int compteur=0;
	double seuil_total=0;
	//CProgressBar pg("Binarisation en cours...",hauteur-fenetre);
	((CMainFrame*)AfxGetMainWnd())->DemarrerProgressBar("Binarisation en cours...",hauteur-fenetre);
	CWaitCursor wc;

	for(i=fenetre/2; i<hauteur-(fenetre/2); i++)
	{

		for(j=(fenetre/2); j<largeur-(fenetre/2); j++)
		{

			som=0;
			ecart=0;
			moy_ecart=0;

			for(k=0; k<fenetre; k++)
			{

				for(l=0;l<fenetre;l++)
				{

					som=getPixel(j+l-fenetre/2,i+k-fenetre/2)+som;
					moy_ecart=(getPixel(j+l-fenetre/2,i+k-fenetre/2))*(getPixel(j+l-fenetre/2,i+k-fenetre/2))+moy_ecart;

				}

			}

			moy=((float)som/(float)(fenetre*fenetre));
			moy_ecart=moy_ecart/(float)(fenetre*fenetre);
			ecart=(((float)(fenetre*fenetre))/((float)(fenetre*fenetre-1)))*(moy_ecart-(moy*moy));
			ecart=(float)sqrt(ecart);
			seuil=(float)(moy*(1.-K*(1.-(ecart/R))));
			
			if ( ecart != 0)
			{

				compteur++;
				seuil_total+=(double)seuil;

				if (((float)getPixel(j,i))<seuil)
				{

					temp=0;

				}
				else
				{

					temp=255;

				}

				m_imbits[i][j]=temp;

			}
			
		}

		//pg.m_progress.StepIt();
		((CMainFrame*)AfxGetMainWnd())->IncrementerProgressBar();

	}

	for(i=0; i<hauteur; i++)
	{

		for(j=0; j<largeur; j++)
		{

			temp=m_imbits[i][j];
			if (!juste_valeur) setPixel(j,i,temp);

		}

	}

	for(k=0; k<hauteur; k++)
	{

		free(m_imbits[k]);

	}

	free(m_imbits);
	binarisee = true;
	return (int)(seuil_total / (double)compteur);

}

/********************************************************************/
/* Nom mthode : Copier
/*                                                                  
/* Fonction : copie l'image pass en paramtre
/*            
/* org : pointeur sur l'image  copier
/*
/* Exception : copie impossible
/*            
/********************************************************************/

void CImage::Copier(CImage *org)
{

	try
	{

		m_pImage->CreateCopy(*(org->m_pImage));
		largeur = org->getLargeur();
		hauteur = org->getHauteur();
		bpp = m_pImage->GetBitsPerPixel();

	}
	catch(PLTextException e){

		CString err;
		err.Format("Error Copy\n%s",e);
		AfxMessageBox(err,MB_OK);

	}

}


/********************************************************************/
/* Nom mthode : Copier32
/*                                                                    
/* Fonction : copie l'image pass en paramtre en forcant une copie 32 bits
/*            
/* org : pointeur sur l'image  copier
/*
/* Exception : copie impossible
/*             
/*                       
/********************************************************************/

void CImage::Copier32(CImage *org)
{

	try
	{

		m_pImage->CreateCopy(*(org->m_pImage),32);
		largeur = org->getLargeur();
		hauteur = org->getHauteur();
		bpp = m_pImage->GetBitsPerPixel();

	}
	catch(PLTextException e){

		CString err;
		err.Format("Error Copy\n%s",e);
		AfxMessageBox(err,MB_OK);

	}

}

/********************************************************************/
/* Nom mthode : getPointeur
/*                                                                    
/* Fonction : retourne le pointeur sur l'image
/*             
/*                       
/********************************************************************/

PLWinBmp * CImage::getPointeur()
{

 	return m_pImage;

}

/********************************************************************/
/* Nom mthode : ConvertirNdg
/*                                                                    
/* Fonction : convertit une image couleur au format BMP en une image au format
/*            au format BMP en une image au format
/*
/* Exception : la conversion n'est pas possible
/*             
/*                       
/********************************************************************/

void CImage::ConvertirNdg()
{

	if(bpp==1 )
	{

		AfxMessageBox("image is allready binarized");
		binarisee=true;
		return;

	}

	if(bpp!=8 )
	{

		try
		{

			PLFilterGrayscale filter;
			filter.ApplyInPlace(m_pImage);
			bpp = m_pImage->GetBitsPerPixel();

		}
		catch(PLTextException e){

			AfxMessageBox(e);

		}

	}

	ndgrisee=true;

}


/********************************************************************/
/* Nom mthode : Eroder
/*                                                                    
/* Fonction : applique l'opration d'rosion sur l'image
/*             
/*                       
/********************************************************************/


void CImage::Eroder()
{

	if (!binarisee)
	{

		AfxMessageBox("Please binarize the image before");
		return;

	}

	int ero_nb=3;
	int **ero_tab=(int**)malloc(ero_nb*sizeof(int*));

	for(int i=0;i<ero_nb;i++){

		ero_tab[i]=(int*)malloc(ero_nb*sizeof(int));
		ero_tab[i][0]=ero_tab[i][1]=ero_tab[i][2]=1;

	}
	

	CImage *tmp = new CImage();
	tmp->Copier(this);
	int x,y;
	bool ok;


	
	//CProgressBar pg("Erosion en cours...",hauteur);
	((CMainFrame*)AfxGetMainWnd())->DemarrerProgressBar("Erosion en cours...",hauteur);
	CWaitCursor wc;

	for(y=1;y<hauteur-1;y++)
	{

		for(x=1;x<largeur-1;x++)
		{

			ok=true;
			int a=0,b=0;

			for(int j=y-ero_nb/2;j<=y+ero_nb/2;j++)
			{

				for(int i=x-ero_nb/2;i<=x+ero_nb/2;i++)
				{

					if(ero_tab[a][b]!=0)

						if(tmp->getPixel(i,j)==BLANC)

							ok=false;
					b++;

				}

				a++;

			}

			if(!ok) setPixel(x,y,BLANC);

		}

		//pg.m_progress.StepIt();
		((CMainFrame*)AfxGetMainWnd())->IncrementerProgressBar();

	}

	delete tmp;

}


/********************************************************************/
/* Nom mthode : Dilater
/*                                                                    
/* Fonction : applique l'opration de dilatation sur l'image
/*             
/*                       
/********************************************************************/

void CImage::Dilater()
{

	if (!binarisee)
	{

		AfxMessageBox("Please binarize the image before");
		return;

	}

	int dil_nb=3;

	/*int **dil_tab=(int**)malloc(dil_nb*sizeof(int*));
	for(int i=0;i<dil_nb;i++)
	{
		dil_tab[i]=(int*)malloc(dil_nb*sizeof(int));
		dil_tab[i][0]=dil_tab[i][1]=dil_tab[i][2]=1;
	}*/


	CImage *tmp = new CImage();
	tmp->Copier(this);

	int x, y, i, j;
	bool ok;
			

	/*non*/ //CProgressBar pg("Dilatation en cours...",hauteur-2);
	//CWaitCursor wc;

	for(y=1;y<hauteur-1;y++){

		for(x=1;x<largeur-1;x++){

			ok=true;
			//int a=0;

			for(j=y-dil_nb/2;j<=y+dil_nb/2;j++){

				//int b = 0;

				for(i=x-dil_nb/2;i<=x+dil_nb/2;i++){

					//if(dil_tab[a][b]!=0)

					if(tmp->getPixel(i,j)==NOIR)

						ok=false;

					//b++;

				}

				//a++;

			}

			if(!ok)

				setPixel(x,y,NOIR);

		}

		/*non*/// pg.m_progress.StepIt();

	}

	/*for(i=0;i<dil_nb;i++)
		free(dil_tab[i]);
	free(dil_tab);*/
	delete tmp;

}

/********************************************************************/
/* Nom mthode : getPixel
/*                                                                  
/* Fonction : renvoie la valeur du pixel de coordones x y
/*            
/* x: abscisse du point dsir
/* y: ordonne du point dsir
/*
/* Sortie : entier reprsentant la valeur du niveau de gris du pixel x,y
/*            
/********************************************************************/

int CImage::getPixel(int x, int y)
{

	PLBYTE **image=m_pImage->GetLineArray();
	if(x<0 || x>=largeur || y<0 || y>=hauteur) return FOND;
	else return image[y][x];

}

/********************************************************************/
/* Nom mthode : setPixel
/*                                                                  
/* Fonction : colorie le pixel de coordones x y avec la valeur val
/*            
/* x: abscisse du point dsir
/* y: ordonne du point dsir
/* val : niveau de gris dsir
/*
/* Sortie : code d'erreur : 0 tout s'est bien pass, -1 sinon
/*            
/********************************************************************/


int CImage::setPixel(int x, int y, int val)
{

	if(x<0 || x>=largeur || y<0 || y>=hauteur) return -1;

	if (bpp==32) 
	{

		PLPixel32 **image=m_pImage->GetLineArray32();
		image[y][x].Set(val,val,val);
		return 0;

	}
	else
	{

		PLBYTE **image=m_pImage->GetLineArray();
		image[y][x]=val;
		return 0;

	}

}


/********************************************************************/
/* Nom mthode : getNbBitsParPixel
/*                                                                  
/* Fonction : renvoie le nombre de bits par pixel
/*
/* Sortie : entier reprsentant le nb de bits par pixel
/*            
/********************************************************************/

int CImage::getNbBitsParPixel()
{

	return bpp;

}


/********************************************************************/
/* Nom mthode : getLargeur
/*                                                                  
/* Fonction : renvoie la largeur de l'image
/*
/* Sortie : entier reprsentant la largeur de l'image
/*            
/********************************************************************/

int CImage::getLargeur()
{

	return largeur;

}

/********************************************************************/
/* Nom mthode : getHauteur
/*                                                                  
/* Fonction : renvoie la hauteur de l'image
/*
/* Sortie : entier reprsentant la hauteur de l'image
/*            
/********************************************************************/

int CImage::getHauteur()
{

	return hauteur;

}


/********************************************************************/
/* Nom mthode : setBinarisee
/*                                                                  
/* Fonction : 
/*            
/********************************************************************/


void CImage::setBinarisee (bool b)
{
	binarisee = b;
}


/********************************************************************/
/* Nom mthode : Afficher
/*                                                                  
/* Fonction : affiche l'image sur le contexte de priphrique "ecran" en 
/*            fonction de la valeur de zoom zoomcoeff
/*            
/* ecran : contexte de priphrique reprsentant la zone de l'cran 
/*         o il est possible de dessiner
/* zoomcoeff : valeur du facteur de zoom
/*            
/********************************************************************/

void CImage::Afficher(CDC *ecran, double zoomcoeff)
{

	int l = (int)(largeur*zoomcoeff);
	int h = (int)(hauteur*zoomcoeff);
	if (m_pImage != NULL)
		m_pImage->StretchDraw(ecran->GetSafeHdc(),0,0,l,h);	

}

/********************************************************************/
/* Nom mthode : EffacerBords
/*                                                                  
/* Fonction : efface le contour exterieur de la page sur un pixel de largeur
/*            necessaire pour l'algorithme de dtection de contours
/*            
/*            
/********************************************************************/


void CImage::EffacerBords()
{

	//CProgressBar pg("Effacement des bords en cours...",4);
	((CMainFrame*)AfxGetMainWnd())->DemarrerProgressBar("Effacement des bords en cours...",4);
	CWaitCursor wc;
	int x,y;

	y=0;
	for(x=0;x<largeur;x++) setPixel(x,y,BLANC);

	//pg.m_progress.StepIt();
	((CMainFrame*)AfxGetMainWnd())->IncrementerProgressBar();

	y=hauteur-1;
	for(x=0;x<largeur;x++) setPixel(x,y,BLANC);

	//pg.m_progress.StepIt();
	((CMainFrame*)AfxGetMainWnd())->IncrementerProgressBar();

	x=0;
	for(y=0;y<hauteur;y++) setPixel(x,y,BLANC);

	//pg.m_progress.StepIt();
	((CMainFrame*)AfxGetMainWnd())->IncrementerProgressBar();

	x=largeur-1;
	for(y=0;y<hauteur;y++) setPixel(x,y,BLANC);

	//pg.m_progress.StepIt();
	((CMainFrame*)AfxGetMainWnd())->IncrementerProgressBar();

}

/********************************************************************/
/* Nom mthode : Ouverture
/*                                                                  
/* Fonction : applique l'opration d'ouverture sur l'image
/*            
/*            
/********************************************************************/


void CImage::Ouverture()
{

	if (!binarisee)
	{

		AfxMessageBox("Please binarize the image before");
		return;

	}

	Eroder();
	Dilater();

}

/********************************************************************/
/* Nom mthode : Fermeture
/*                                                                  
/* Fonction : applique l'opration de fermeture sur l'image
/*            
/*            
/********************************************************************/

void CImage::Fermeture()
{

	if (!binarisee)
	{

		AfxMessageBox("Please binarize the image before");
		return;

	}

	Dilater();
	Eroder();

}

/********************************************************************/
/* Nom mthode : setTailleLxH
/*                                                                  
/* Fonction : 
/*            
/*            
/********************************************************************/

void CImage::setTailleLxH(int L, int H)
{

	largeur = L;
	hauteur = H;
}



/********************************************************************/
/* Nom mthode : Comparer
/*                                                                  
/* Fonction : 
/*            
/*            
/********************************************************************/


bool CImage::Comparer (CImage *img, double seuil)
{
	// cf. algo rapport

	int L,H;
	int x,y;
	int pixel1, pixel2;
	int aire1, aire2, aire_max;
	double distance;

	distance = 0;

	aire1 = img->getLargeur()*img->getHauteur();
	aire2 = largeur*hauteur;
	
	if (aire1 > aire2)

		aire_max = aire1;

	else

		aire_max = aire2;


	if (largeur < img->getLargeur() )

		L = largeur;

	else

		L = img->getLargeur();


	if (hauteur < img->getHauteur() )

		H = hauteur;

	else

		H = img->getHauteur();


	for (x=0; x<L; x++)				// on considere l'image comme ayant l'attribut binarisee  "true" et 8bpp

		for (y=0; y<H; y++)
		{

			pixel1 = img->getPixel (img->getLargeur()/2 - L/2 + x,
									img->getHauteur()/2 - H/2 + y)/255;
			pixel2 = this->getPixel (largeur/2-L/2 + x, hauteur/2-H/2 + y)/255;
			distance += abs(pixel1 - pixel2);
			

		}

	distance += /*MAX**/abs(aire2 - aire1);
	distance = distance / (double)aire_max;

	if (distance > seuil)

		return false;

	return true;

}


/********************************************************************/
/* Nom mthode : Comparer_Lebourg
/*                                                                  
/* Fonction : 
/*            
/*            
/********************************************************************/

bool CImage::Comparer_Lebourg (CImage *img, CImage *img_dilatee, CImage* dilatee, double seuil)
{
	// cf. algo rapport

	bool resultat;
	int Lmin, Hmin, x, y;
	CImage *image1, *image2;

	if (largeur < img->getLargeur ())

		Lmin = largeur;

	else

		Lmin = img->getLargeur ();


	if (hauteur < img->getHauteur ())

		Hmin = hauteur;

	else

		Hmin = img->getHauteur ();


	image1 = new CImage (Lmin, Hmin, 8);
	image2 = new CImage (Lmin, Hmin, 8);
	
	for (x=0; x<Lmin; x++)

		for (y=0; y<Hmin; y++)
		{

			image1->setPixel(x,y,255);
			image2->setPixel(x,y,255);

		}

	for (x=0; x<Lmin; x++)

		for (y=0; y<Hmin; y++)
		{

			if (this->getPixel(largeur/2 - Lmin/2 + x, hauteur/2 - Hmin/2 + y) != 
				img_dilatee->getPixel(img_dilatee->getLargeur()/2 - Lmin/2 + x,
										img_dilatee->getHauteur() / 2 - Hmin/2 + y))

				image1->setPixel(x,y,0);

			if (img->getPixel(img->getLargeur()/2 - Lmin/2 + x,
								img->getHauteur()/2 - Hmin/2 + y) != 
				dilatee->getPixel(dilatee->getLargeur()/2 - Lmin/2 + x,
								dilatee->getHauteur()/2 - Hmin/2 + y))

				image2->setPixel(x,y,0);

		}

	resultat = image1->Comparer (image2, seuil);
	delete image1;
	delete image2;

	return resultat;

}


