/****************************************************************/
// Ligne.cpp: implementation of the CLigne class.
/****************************************************************/

#include "stdafx.h"
#include "myapp.h"
#include "Ligne.h"
#include "MyAppDoc.h"
#include "Atelier.h"
#include <math.h>
#include <direct.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

/****************************************************************/
// Construction/Destruction
/****************************************************************/

CLigne::CLigne()
{
	liste_mots = new CObList(10);
	liste_elements = new CObList(10);
	ordonnee_minimale = 0;
	ordonnee_maximale = 0;
	abscisse_minimale = 0;
	abscisse_maximale = 0;
	y_cdg_moyen = 0;
	espacement_moyen = 0;
	liste_modifiee = false;
	rectL = new CRect(0,0,0,0);
}


CLigne::~CLigne()
{
	POSITION p1;
	if (liste_mots)
	{
		for (p1 = liste_mots->GetHeadPosition();p1 != NULL;)
		{
			CMot *m = (CMot *)liste_mots->GetNext(p1);
			delete m;
		}
		liste_mots->RemoveAll();
		delete liste_mots;
	}
	
	if (liste_elements)
	{
		liste_elements->RemoveAll();
		delete liste_elements;
	}
	delete rectL;
}

CObList* CLigne::get_liste_elements ()
{
	return liste_elements;
}

void CLigne::Calculer_cdg_moyen ()
{
	int cdg_moyen;
	int nb_elements;

	cdg_moyen = 0;
	nb_elements = liste_elements->GetCount ();
	POSITION p1;
	for (p1 = liste_elements->GetHeadPosition();p1 != NULL;)
	{

		CElement *e = (CElement *)liste_elements->GetNext(p1);
		cdg_moyen += e->getCdg ()->y;
	}

	y_cdg_moyen = cdg_moyen/nb_elements;
}

void CLigne::Trouver_ordonnee_minimale ()
{
	int min;
	POSITION p1;

	min = 100000;

	for (p1 = liste_elements->GetHeadPosition();p1 != NULL;)
	{
		int y_haut;
		CElement *e = (CElement *)liste_elements->GetNext(p1);
		y_haut = e->getRect ()->top;
		if (min > y_haut)
			min = y_haut;
	}

	ordonnee_minimale = min;


}

void CLigne::Trouver_ordonnee_maximale ()
{
	int max;
	POSITION p1;

	max = 0;

	for (p1 = liste_elements->GetHeadPosition();p1 != NULL;)
	{
		int y_bas;
		CElement *e = (CElement *)liste_elements->GetNext(p1);
		y_bas = e->getRect ()->bottom;
		if (max < y_bas)
			max = y_bas;
	}

	ordonnee_maximale = max;
}

void CLigne::Calculer_espacement_moyen ()
{
	POSITION p1;
	int espacement = 0;

	for (p1 = liste_elements->GetHeadPosition();p1 != NULL;)
	{
		CElement *e = (CElement *)liste_elements->GetNext(p1);
		if (p1 != NULL)
		{
			CElement *e_suiv = (CElement *)liste_elements->GetAt (p1);
			espacement += (e_suiv->getRect ()->left - e->getRect ()->right);
		}
	}
	espacement_moyen = espacement / liste_elements->GetCount ();
}

bool CLigne::Fusionner_lignes (CLigne* ligne)
{
	// fusionne les lignes dont l'ordonne du centre de gravit de la 1ere ligne est
	// comprise entre l'ordonne min et l'ordonne max de la seconde ligne

	if ((y_cdg_moyen >= ligne->ordonnee_minimale && y_cdg_moyen <= ligne->ordonnee_maximale)
		|| (ligne->get_y_cdg_moyen () >= ordonnee_minimale && ligne->get_y_cdg_moyen () <= ordonnee_maximale))
	{
		POSITION p1;
		for (p1 = ligne->get_liste_elements ()->GetHeadPosition (); p1 != NULL;)
		{
			CElement *e = (CElement *)ligne->get_liste_elements ()->GetNext (p1);
			liste_elements->AddTail (e);
		}
		delete ligne;
		return true;
	}
	else
		return false;
}

/*void CLigne::Rejeter_elements_hors_ligne (CLigne* rejet)
{
	int cdg;
	POSITION p1,p2;

	for (p1 = liste_elements->GetHeadPosition();p1 != NULL;)
	{
		p2 = p1;
		CElement *e = (CElement *)liste_elements->GetNext(p1);
		cdg = e->getCdg ()->y;
		if (cdg < ordonnee_minimale || cdg > ordonnee_maximale)
		{
			rejet->get_liste_elements ()->AddTail (e);
			liste_elements->RemoveAt (p2);
			liste_modifiee = true;
		}
	}
}

void CLigne::Reclasser_rejet (CObList* lignes)
{
	int cdg;
	POSITION p1,p2;

	for (p1 = liste_elements->GetHeadPosition();p1 != NULL;)
	{
		CElement *e = (CElement *)liste_elements->GetNext(p1);
		cdg = e->getCdg ()->y;
		for (p2 = lignes->GetHeadPosition();p2 != NULL;)
		{
			CLigne *l = (CLigne *)lignes->GetNext(p2);
			if (cdg > l->get_ordonnee_minimale () && cdg < l->get_ordonnee_maximale ())
				l->get_liste_elements ()->AddTail (e);
		}
	}
}*/

//  ############## this fi=unction is used to keep things in order

void CLigne::Ordonner_elements_sens_lecture ()
{
	// reordonne les CC de liste_elements selon l'ordre logique de lecture

	int min;
	POSITION p1,p2,pmin;
	CObList *prov = new CObList (10);

	while (!liste_elements->IsEmpty ())
	{
		min = 10000;

		for (p1 = liste_elements->GetHeadPosition (); p1 != NULL;)
		{
			p2 = p1;
			CElement *e = (CElement *)liste_elements->GetNext (p1);
			if (e->getCdg ()->x < min)
			{
				min = e->getCdg ()->x;
				pmin = p2;
			}
		}
		prov->AddTail (liste_elements->GetAt (pmin));
		liste_elements->RemoveAt (pmin);
	}
	liste_elements->AddTail (prov);
	prov->RemoveAll();
	delete prov;
}

void CLigne::Supprimer_accentuation ()
{
	/*****************nom de la fonction****************************/ 
	//Supprimer_accentuation
	/***********************but*************************************/
	//Supprime les CC de la liste "liste_elements" si ce sont des
	//points sur les "i" et "j" ou des accents
	/*********************pr-condition***********************************/
	//La liste doit contenir des CC tries dans le sens de la lecture
	/***************************************************************/
	
/********************************************** Rachit Agrawal 2009 ******************************************/

/*
	POSITION p1,p2,p3,p_back;
	int rmin,lmax,rmax,lmin,common,ltop,rbottom,flag;
	for (p1 = liste_lignes->GetHeadPosition ();p1 != NULL;)
	{
		CLigne *ligne = (CLigne *)(liste_lignes->GetNext(p1));
		for(p2 = ligne->get_liste_elements()->GetHeadPosition(); p2 != NULL;)
		{
			p3 = p2;
			CElement *e = (CElement *)ligne->get_liste_elements()->GetNext(p2);
			flag = 0;
			if (e->GetClassNumber()== 3) // that is accent
			{
				// now let us compare with i-1
				if (p_back != NULL)
				{
					CElement *e_next = (CElement *)ligne->get_liste_elements()->GetAt (p_back); // here e_next = prev
					if(e_next->GetClassNumber()== 1 || e_next->GetClassNumber()== 2)
					{
						// we check i is above i-1
						if(e_next->getRect()->top >= e->getRect()->bottom)
						{
							// now we need to check for 100% overlap
							if(e_next->getRect()->left <= e->getRect()->left &&e_next->getRect()->right >= e->getRect()->right)
							{
								// update p_back and remove p3;
								if(e_next->getRect()->left <= e->getRect()->left)
								{
									lmin = e_next->getRect()->left;
								}
								else
								{
									lmin = e->getRect()->left;
								}
								if(e_next->getRect()->right <= e->getRect()->right)
								{
									rmax = e->getRect()->right;
								}
								else
								{
									rmax = e_next->getRect()->right;
								}
								if(e_next->getRect()->top <= e->getRect()->top)
								{
									ltop = e_next->getRect()->top;
								}
								else
								{
									ltop = e->getRect()->top;
								}
								if(e_next->getRect()->bottom <= e->getRect()->bottom)
								{
									rbottom = e->getRect()->bottom;
								}
								else
								{
									rbottom = e_next->getRect()->bottom;
								}
							
								//	e_next->SetRect()->left = lmin;
								//	e_next->SetRect()->top = ltop;
								//	e_next->SetRect()->right = rmax;
								//	e_next->SetRect()->bottom = rbottom;
								flag = 1;
								ligne->get_liste_elements()->RemoveAt(p3);
							}
							// if they overlap by 50%
							else
							{
								if(e_next->getRect()->left <= e->getRect()->left)
								{
									lmax = e->getRect()->left;
									lmin = e_next->getRect()->left;
								}
								else
								{
									lmin = e->getRect()->left;
									lmax = e_next->getRect()->left;
								}
								if(e_next->getRect()->right <= e->getRect()->right)
								{
									rmin = e_next->getRect()->right;
									rmax = e->getRect()->right;
								}
								else
								{
									rmax = e_next->getRect()->right;
									rmin = e->getRect()->right;
								}
								int count = 10 * ((double)rmin - (double)lmax) / ((double)rmax - (double)lmin);
								if(count>= 5 ) // 50 %overlap ........... this can be changed
								{
									// update p_back and remove p3;
									if(e_next->getRect()->left <= e->getRect()->left)
									{
										lmin = e_next->getRect()->left;
									}
									else
									{
										lmin = e->getRect()->left;
									}
									if(e_next->getRect()->right <= e->getRect()->right)
									{
										rmax = e->getRect()->right;
									}
									else
									{
										rmax = e_next->getRect()->right;
									}
									if(e_next->getRect()->top <= e->getRect()->top)
									{
										ltop = e_next->getRect()->top;
									}
									else
									{
										ltop = e->getRect()->top;
									}
									if(e_next->getRect()->bottom <= e->getRect()->bottom)
									{
										rbottom = e->getRect()->bottom;
									}
									else
									{
										rbottom = e_next->getRect()->bottom;
									}
							
									//	e_next->SetRect()->left = lmin;
									//	e_next->SetRect()->top = ltop;
									//	e_next->SetRect()->right = rmax;
									//	e_next->SetRect()->bottom = rbottom;
									flag = 1;
									ligne->get_liste_elements()->RemoveAt(p3);
								}
							}
						}
					}
				}
				// let us compare with i+1 first
				if (p2 != NULL && flag ==0)
				{
					CElement *e_next = (CElement *)ligne->get_liste_elements()->GetAt (p2);
					if(e_next->GetClassNumber() == 1 || e_next->GetClassNumber() == 2)
					{
						if(e_next->getRect()->top >= e->getRect()->bottom)
						{
							// now we need to check for 100% overlap
							if(e_next->getRect()->left <= e->getRect()->left &&e_next->getRect()->right >= e->getRect()->right)
							{
								// update p2 and remove p3;
								if(e_next->getRect()->left <= e->getRect()->left)
								{
									lmin = e_next->getRect()->left;
								}
								else
								{
									lmin = e->getRect()->left;
								}
								if(e_next->getRect()->right <= e->getRect()->right)
								{
									rmax = e->getRect()->right;
								}
								else
								{
									rmax = e_next->getRect()->right;
								}
								if(e_next->getRect()->top <= e->getRect()->top)
								{
									ltop = e_next->getRect()->top;
								}
								else
								{
									ltop = e->getRect()->top;
								}
								if(e_next->getRect()->bottom <= e->getRect()->bottom)
								{
									rbottom = e->getRect()->bottom;
								}
								else
								{
									rbottom = e_next->getRect()->bottom;
								}
							
								//	e_next->SetRect()->left = lmin;
								//	e_next->SetRect()->top = ltop;
								//	e_next->SetRect()->right = rmax;
								//	e_next->SetRect()->bottom = rbottom;
								flag = 1;
								ligne->get_liste_elements()->RemoveAt(p3);
							}
							// if they overlap by 50%
							else
							{
								if(e_next->getRect()->left <= e->getRect()->left)
								{
									lmax = e->getRect()->left;
									lmin = e_next->getRect()->left;
								}
								else
								{
									lmin = e->getRect()->left;
									lmax = e_next->getRect()->left;
								}
								if(e_next->getRect()->right <= e->getRect()->right)
								{
									rmin = e_next->getRect()->right;
									rmax = e->getRect()->right;
								}
								else
								{
									rmax = e_next->getRect()->right;
									rmin = e->getRect()->right;
								}
								int count = 10 * ((double)rmin - (double)lmax) / ((double)rmax - (double)lmin);
								if(count>= 5 ) // 50 %overlap ........... this can be changed
								{
									// update p2 and remove p3;
									if(e_next->getRect()->left <= e->getRect()->left)
									{
										lmin = e_next->getRect()->left;
									}
									else
									{
										lmin = e->getRect()->left;
									}
									if(e_next->getRect()->right <= e->getRect()->right)
									{
										rmax = e->getRect()->right;
									}
									else
									{
										rmax = e_next->getRect()->right;
									}
									if(e_next->getRect()->top <= e->getRect()->top)
									{
										ltop = e_next->getRect()->top;
									}
									else
									{
										ltop = e->getRect()->top;
									}
									if(e_next->getRect()->bottom <= e->getRect()->bottom)
									{
										rbottom = e->getRect()->bottom;
									}
									else
									{
										rbottom = e_next->getRect()->bottom;
									}
									
									//	e_next->SetRect()->left = lmin;
									//	e_next->SetRect()->top = ltop;
									//	e_next->SetRect()->right = rmax;
									//	e_next->SetRect()->bottom = rbottom;
									flag = 1;
									ligne->get_liste_elements()->RemoveAt(p3);
								}
							}
						}
					}
				}
				if(flag != 1)
				{
					p_back = p3;
				}
			}
		}
	}
	*/			

				// now i hope that merging of acccents with the character is done


/********************************************** Rachit Agrawal 2009 ******************************************/

	POSITION p1, p2;
	int x_e_suiv_gauche, x_e_suiv_droite, x_e_gauche, x_e_droite;

	for (p1 = liste_elements->GetHeadPosition (); p1 != NULL;)
	{
		p2 = p1;
		CElement *e = (CElement *)liste_elements->GetNext (p1);
		if (p1 != NULL)
		{
			CElement *e_suiv = (CElement *)liste_elements->GetAt (p1);
///// ###################### i think here we are storing the co ordinates of 2 conseutive eements

			x_e_suiv_gauche = e_suiv->getRect ()->left;
			x_e_suiv_droite = e_suiv->getRect ()->right;
			x_e_gauche = e->getRect ()->left;
			x_e_droite = e->getRect ()->right;

// ################ i think here we are looking for dot of i n j so either above or below n to remove it from the list

			// si le rect e se trouve au dessus du rect e_suiv

// ############## i think here we compared whether one is on top of other and x co rdinate(starting ir ending) lies within the range ie width of other
// ############## ans finally we are removing it from the list [data loss]
			if (((e->getCdg ()->x >= x_e_suiv_gauche && e->getCdg ()->x <= x_e_suiv_droite) ||
			(x_e_gauche >= x_e_suiv_gauche && x_e_gauche <= x_e_suiv_droite) ||
			(x_e_droite >= x_e_suiv_gauche && x_e_droite <= x_e_suiv_droite))
			&& e->getCdg ()->y < e_suiv->getRect ()->top)
					liste_elements->RemoveAt (p2);
			// si le rect e_suiv se trouve au dessus du rect e
			else if (((e_suiv->getCdg ()->x >= x_e_gauche && e_suiv->getCdg ()->x <= x_e_droite) ||
				(x_e_suiv_gauche >= x_e_gauche && x_e_suiv_gauche <= x_e_droite) ||
				(x_e_suiv_droite >= x_e_gauche && x_e_suiv_droite <= x_e_droite))
				&& e_suiv->getCdg ()->y < e->getRect ()->top)
			{
				liste_elements->RemoveAt (p1);
				liste_elements->GetNext (p2);
				p1 = p2;
			}
		}
	}
	
}

void CLigne::Decomposer_en_mots ()
{
	/*****************nom de la fonction****************************/ 
	//Decomposer_en_mots
	/***********************but*************************************/
	//La liste "liste_mots" est renseigne et chaque objet mot est cr
	//avec sa liste "liste_elements" remplie
	/*********************entre***********************************/
	//epsilon est le nombre de pixel qui, ajout a l'espacement moyen
	//entre deux CC sur une ligne, indique que la CC suivante est la 
	//premiere d'un nouveau mot
	/*********************pr-condition***********************************/
	//La liste doit contenir des CC tries dans le sens de la lecture
	/***************************************************************/

/********************************************* Rachit Agrawal 2009 **********************************************/
/*
	// calculating the xl_mid and xr_mid
	POSITION p1,p2;
	int flag=0,y=0;
	CImage *img = new CImage();
	for (p1 = liste_lignes->GetHeadPosition ();p1 != NULL;)
	{
		CLigne *ligne = (CLigne *)(liste_lignes->GetNext(p1));
		for(p2 = ligne->get_liste_elements()->GetHeadPosition(); p2 != NULL;)
		{
			CElement *e = (CElement *)ligne->get_liste_elements->GetNext(p2);	
			y = (e->getRect()->top + e->getRect()->bottom)/2;
			flag =0;
			for (int i= e->getRect()->left; i< e->getRect()->right; i++)
			{
				if(img->getPixel(i,y)== 0) // pixel == black I hope this should work
				{
					flag = 1;
					e->SetXLM(i);
					break;
				}
			}
			if(flag == 0)
			{
				e->SetXLM((e->getRect()->left + e->getRect()->right)/2); // this is centre we can change it 
			}
			flag =0;
			for (int i= e->getRect()->right; i< e->getRect()->left; i--)
			{
				if(img->getPixel(i,y)== 0) // pixel == black I hope this should work
				{
					flag = 1;
					e->SetXRM(i);
					break;
				}
			}
			if(flag == 0)
			{
				e->SetXRM((e->getRect()->left + e->getRect()->right)/2); // this is centre we can change it 
			}
		}
	}

	// now we need to calculate the average
	// we have 2 options either we can calculate per page basis or per line basis
	// i am writing code taking per page basis.
	POSITION p1,p2,p3;
	int total = 0, count =1;
	for (p1 = liste_lignes->GetHeadPosition ();p1 != NULL;)
	{
		total = 0;
		count = 1;
		CLigne *ligne = (CLigne *)(liste_lignes->GetNext(p1));
		for(p2 = ligne->get_liste_elements()->GetHeadPosition(); p2 != NULL;)
		{
			CElement *e = (CElement *)ligne->get_liste_elements->GetNext(p2);
			if(p2 == NULL)
			{
				break;
			}
			CElement *e_next = (CElement *)ligne->get_liste_elements->GetAt(p2);
			if((e_next->getRect()->left - e->getRect()->right) >0)
			{
				total += e_next->getRect()->left - e->getRect()->right;
				count++;
			}
		}
		double average = 0.0;
		average = (double)total /(double)count;
		int threshold = (int) 1.5 * average;

		// now we have calculated the threshold value
		// now we need to analyse the gap between the 2 consecutive chars

		int wordcount = 1;
		int gap1 = 0, gap2 = 0;
		for(p2 = ligne->get_liste_elements()->GetHeadPosition(); p2 != NULL;)
		{
			CElement *e = (CElement *)ligne->get_liste_elements->GetNext(p2);
			if(p2 == NULL)
			{
				e->SetWordNumber(wordcount);
				break;
			}
			CElement *e_next = (CElement *)ligne->get_liste_elements->GetAt(p2);
			// apply rules
			gap1 = e_next->getrect()->left - e->getrect()->right ;
			gap2 = e_next->GetXLM() - e->GetXRM() ;
			
			if(e_next->GetClassNumber() == 4) // for punctuation
			{
				e->SetWordNumber(wordcount);
				wordcount++;
			}

			else if(e->GetClassNumber() == 4) // for punctuation
			{
				e->SetWordNumber(wordcount);
				wordcount++;
			}

			else
			{
				if(gap1 >= threshold && gap2 >= threshold)
				{
					e->SetWordNumber(wordcount);
					wordcount++;
				}
				else if(gap1 <= threshold && gap2 >= 1.2*threshold) // we can change this threshold
				{
					e->SetWordNumber(wordcount);
					wordcount++;
				}
				else
				{
					e->SetWordNumber(wordcount);
				}
			}
		}
	}
	// now we can try building the list of words
	POSITION p3;
	for (p1 = liste_lignes->GetHeadPosition ();p1 != NULL;)
	{
		CLigne *ligne = (CLigne *)(liste_lignes->GetNext(p1));
		for(p2 = ligne->get_liste_elements()->GetHeadPosition(); p2 != NULL;)
		{
			CElement *e = (CElement *)ligne->get_liste_elements->GetNext(p2);
			CMot* mot = new CMot ();
			mot->get_liste_elements ()->AddTail (e);
			liste_mots->AddTail (mot);
			
			CElement *e_prev;
			int flag =0;
			e_prev = e;

			while(p2!= NULL && flag == 0)
			{
				CElement *e = (CElement *)ligne->get_liste_elements->GetAt(p2);
				if(e->GetWordNumber() != e_prev->GetWordNumber())
				{
					flag = 1;
				}
				else
				{
					mot->get_liste_elements ()->AddTail (e);
					e_prev = e;
					(CElement *)ligne->get_liste_elements->GetNext(p2);
				}
			}
		}
	}

	// i think now we are done with line-number , class-number, char-number and word-number :D:D:D:D
	// liste_lignes contains the link list of line 
	// liste_mots contains the link list of words
	// liste_elements contains the link list of characters

*/

/********************************************* Rachit Agrawal 2009 **********************************************/


	POSITION p1,p2;
	int espacement;
	CElement* e_prec;
	bool meme_mot;
	double borne;

	borne = espacement_moyen * 1.5;         // ################ 1.5 times the average space

	while (!liste_elements->IsEmpty ())
	{
		CElement* tete = (CElement *)(liste_elements->RemoveHead ());
		CMot* mot = new CMot ();
		mot->get_liste_elements ()->AddTail (tete);
		liste_mots->AddTail (mot);
		
		e_prec = tete;
		meme_mot = true;

		while (meme_mot && !liste_elements->IsEmpty ())
		{
			p1 = liste_elements->GetHeadPosition ();
			p2 = p1;
			CElement *e = (CElement *)liste_elements->GetNext (p1);
			espacement = e->getRect ()->left - e_prec->getRect ()->right;
			if (espacement > borne)	// ################  it think we are checking if the space is less than 1.5 decompose or something?? doubt??
				meme_mot = false;
			else
			{
				mot->get_liste_elements ()->AddTail (e);
				liste_elements->RemoveAt (p2);
				e_prec = e;
			}
		}
	}
}

int CLigne::get_y_cdg_moyen ()
{
	return y_cdg_moyen;
}

int CLigne::get_ordonnee_minimale ()
{
	return ordonnee_minimale;
}

int CLigne::get_ordonnee_maximale ()
{
	return ordonnee_maximale;
}

bool CLigne::get_liste_modifiee ()
{
	return liste_modifiee;
}

CObList* CLigne::get_liste_mots ()
{
	return liste_mots;
}




/*************************************************************************************************/
/* PFE 2006
/*************************************************************************************************/




/********************************************************************/
/* Nom mthode : CalculerLmoy
/*                                                                  
/* Fonction : permet de calculer la largeur moyenne de tous les elements
/*            situs sur une ligne, avec point de dpart et nombre de CC
/*            concernes
/********************************************************************/

int CLigne::CalculerLmoy(POSITION depart, int nbC)
{
	POSITION p3;
	int x1,x2;
	int largeurmoy=0;
	int largeurcourante=0;
	int largeurmoyenne=0;
	int nbElementTraite=0;
	
	for (p3 = depart;p3 != NULL && nbElementTraite != nbC;) // pour chaque element de la ligne
	{
		CElement *e = (CElement *)liste_elements->GetNext(p3); // on met dans e l'lment
		rect = e->getRect();
		x1 = rect->TopLeft().x;	// rcupre la coord x en haut  gauche
		x2 = rect->BottomRight().x;	// rcupre la coord x en bas  droite

		largeurcourante = abs(x2 - x1);
		largeurmoy = largeurmoy + largeurcourante ;
		largeurcourante=0; // remise  zro
		nbElementTraite++;
	}

	largeurmoyenne = largeurmoy / nbC;

	return(largeurmoyenne);
}

/********************************************************************/
/* Nom mthode : CalculerHmoy
/*                                                                  
/* Fonction : permet de calculer la hauteur moyenne de tous les elements
/*            situs sur une ligne avec point de dpart et nombre de CC
/*            concernes
/********************************************************************/
int CLigne::CalculerHmoy(POSITION depart, int nbC)
{

	POSITION p3;
	int y1,y2;
	int hauteurmoy=0;
	int largeurcourante=0;
	int largeurmoyenne=0;
	int nbElementTraite=0;
	
	for (p3 = depart;p3 != NULL && nbElementTraite != nbC;) // pour chaque element de la ligne
	{
		CElement *e = (CElement *)liste_elements->GetNext(p3); // on met dans e l'lment
		rect = e->getRect();
		y1 = rect->TopLeft().y;	// rcupre la coord y en haut  gauche 
		y2 = rect->BottomRight().y;	// rcupre la coord y en bas  droite

		largeurcourante = abs(y2 - y1);
		hauteurmoy = hauteurmoy + largeurcourante ;
		largeurcourante=0; // remise  zro
		nbElementTraite++;
	}

	largeurmoyenne = hauteurmoy / nbC;

	return(largeurmoyenne);
}



/********************************************************************/
/* Nom mthode : CalculerCdgMoyen
/*                                                                  
/* Fonction : permet de calculer le cdg moyen de tous les elements
/*            situs sur une ligne avec point de dpart et nombre de CC
/*            concernes
/********************************************************************/
int CLigne::CalculerCdgMoyen(POSITION depart, int nbC)
{
	int cdg_moyen=0;
	int nb_elements;
	int resultat=0;
	int nbElementTraite=0;
	POSITION p1;

	nb_elements = nbC;

	for (p1 = depart ;p1 != NULL && nbElementTraite != nbC ;)
	{
		CElement *e = (CElement *)liste_elements->GetNext(p1);
		cdg_moyen = cdg_moyen + e->getCdg ()->y;
		nbElementTraite++;
	}

	resultat = cdg_moyen/nb_elements;

	return(resultat);
}



/********************************************************************/
/* Nom mthode : CalculerEcartTypeLargeur
/*                                                                  
/* Fonction : permet de calculer l'cart type au niveau de la largeur
/*            avec point de dpart et nombre de CC concernes
/*            
/********************************************************************/
double CLigne::CalculerEcartTypeLargeur(int moyenne,POSITION depart, int nbC)
{

	POSITION p3;
	int x1,x2;
	double resultat=0;
	double resultat2=0;
	int largeurcourante=0;
	double nbElement=0;
	int nbElementTraite=0;
	
	for (p3 = depart;p3 != NULL && nbElementTraite != nbC;) // pour chaque element de la ligne
	{
		CElement *e = (CElement *)liste_elements->GetNext(p3); // on met dans e l'lment
		rect = e->getRect();
		x1 = rect->TopLeft().x;	// rcupre la coord x en haut  gauche
		x2 = rect->BottomRight().x;	// rcupre la coord x en bas  droite

		largeurcourante = abs(x2 - x1);
		resultat = resultat + pow((largeurcourante - moyenne),2); // somme avec carr
		largeurcourante=0; // remise  zro
		nbElementTraite++;
	}
	
	nbElement=nbC;
	resultat2 = sqrt ((1/nbElement)*resultat); // racine

	return(resultat2);
}


/********************************************************************/
/* Nom mthode : CalculerEcartTypeHauteur
/*                                                                  
/* Fonction : permet de calculer l'cart type au niveau de la hauteur
/*            avec point de dpart et nombre de CC concernes
/*            
/********************************************************************/
double CLigne::CalculerEcartTypeHauteur(int moyenne,POSITION depart, int nbC)
{

	POSITION p3;
	int y1,y2;
	double resultat=0;
	double resultat2=0;
	int hauteurcourante=0;
	double nbElement=0;
	int nbElementTraite=0;
	
	for (p3 = depart;p3 != NULL && nbElementTraite != nbC;) // pour chaque element de la ligne
	{
		CElement *e = (CElement *)liste_elements->GetNext(p3); // on met dans e l'lment
		rect = e->getRect();
		y1 = rect->TopLeft().y;	// rcupre la coord y en haut  gauche 
		y2 = rect->BottomRight().y;	// rcupre la coord y en bas  droite

		hauteurcourante = abs(y2 - y1);
		resultat = resultat + pow((hauteurcourante - moyenne),2); // somme avec carr
		hauteurcourante=0; // remise  zro
		nbElementTraite++;
	}
	
	nbElement=nbC;
	resultat2 = sqrt ((1/nbElement)*resultat); // racine

	return(resultat2);

}


/********************************************************************/
/* Nom mthode : CalculerEcartTypeCdg
/*                                                                  
/* Fonction : permet de calculer l'cart type au niveau du Y (centre de gravit)
/*            avec point de dpart et nombre de CC concernes
/*            
/********************************************************************/
double CLigne::CalculerEcartTypeCdg(int moyenne,POSITION depart, int nbC)
{
	POSITION p3;
	int cdgcourant=0;
	double resultat=0;
	double resultat2=0;
	double nbElement=0;
	int nbElementTraite=0;

	for (p3 = depart;p3 != NULL && nbElementTraite != nbC;)
	{
		CElement *e = (CElement *)liste_elements->GetNext(p3);
		cdgcourant = e->getCdg()->y;
		resultat = resultat + pow((cdgcourant - moyenne),2); // somme
		cdgcourant = 0; // remise  zro
		nbElementTraite++;
	}

	nbElement=nbC;
	resultat2 = sqrt ((1/nbElement)*resultat); // racine

	return(resultat2);
}

/********************************************************************/
/* Nom mthode : Creation_Histogramme
/*                                                                  
/* Fonction : permet de crer l'histogramme des effectifs de la ligne
/*			  permet de trouver le pic max de l'histogramme et de retourner
/*            la coordonne associ           
/*            Avec point de dpart et nombre de CC concernes
/********************************************************************/
int CLigne::Creation_Histogramme_Y_Haut(int hauteurImageCourante,int ordonneeMin,int ordonneeMax,POSITION depart, int nbC)
{
	POSITION p3;
	CRect *rectangle;
	int yHautCourant=0;
	int picMaxYHaut=0;
	int effectif=0;
	int *tabHistoYHaut=NULL;
	tabHistoYHaut=(int*)malloc(hauteurImageCourante*sizeof(int));
	int nbElementTraite=0;

	// initialise le tableau avec des 0
	for(int j=0;j<hauteurImageCourante;j++)
	{
		tabHistoYHaut[j]=0;
	}
	
	// remplissage du tableau histogramme
	for (p3 = depart;p3 != NULL && nbElementTraite != nbC;)
	{
		CElement *e = (CElement *)liste_elements->GetNext(p3);
		rectangle = e->getRect();
		yHautCourant = rectangle->TopLeft().y;
		tabHistoYHaut[yHautCourant] = tabHistoYHaut[yHautCourant]+1;
		nbElementTraite++;
	}


	// recherche du pic maximum present dans l'histogramme
	picMaxYHaut = ordonneeMin;

	for(int i=ordonneeMin;i<=ordonneeMax;i++)
	{
		if(tabHistoYHaut[i]>effectif)
		{
			picMaxYHaut = i;
			effectif = tabHistoYHaut[i];
		}
	}



	free(tabHistoYHaut);
	return(picMaxYHaut);
}


/********************************************************************/
/* Nom mthode : Creation_Histogramme
/*                                                                  
/* Fonction : permet de crer l'histogramme des effectifs de la ligne
/*			  permet de trouver le pic max de l'histogramme et de retourner
/*            la coordonne associ
/*            Avec point de dpart et nombre de CC concernes
/********************************************************************/
int CLigne::Creation_Histogramme_Y_Bas(int hauteurImageCourante,int ordonneeMin,int ordonneeMax,POSITION depart, int nbC)
{
	POSITION p3;
	CRect *rectangle;
	int picMaxYBas=0;
	int yBasCourant=0;
	int effectif=0;
	int *tabHistoYBas=NULL;
	tabHistoYBas=(int*)malloc(hauteurImageCourante*sizeof(int));
	int nbElementTraite=0;

	// initialise le tableau avec des 0
	for(int j=0;j<hauteurImageCourante;j++)
	{
		tabHistoYBas[j]=0;
	}

	// remplissage du tableau histogramme
	for (p3 = depart;p3 != NULL && nbElementTraite != nbC;)
	{
		CElement *e = (CElement *)liste_elements->GetNext(p3);
		rectangle = e->getRect();
		yBasCourant = rectangle->BottomRight().y;
		tabHistoYBas[yBasCourant] = tabHistoYBas[yBasCourant]+1;
		nbElementTraite++;
	}
	
	// recherche du pic maximum present dans l'histogramme
	picMaxYBas = tabHistoYBas[ordonneeMin];

	for(int i=ordonneeMin;i<=ordonneeMax;i++)
	{
		if(tabHistoYBas[i]>effectif)
		{
			picMaxYBas = i;
			effectif = tabHistoYBas[i];
		}
	}

	free(tabHistoYBas);
	return(picMaxYBas);
}



/********************************************************************/
/* Nom mthode : DetectionAnomalie
/*                                                                  
/* Fonction : permet de classer les CC dans diffrentes classes selon
/*            si elles correspondent  des anomalies ou non
/*			  Classement effectue sur une ligne entire, pas de gestion
/*			  de la longueur de la ligne
/*            
/********************************************************************/
void CLigne::DetectionAnomalie(CImage *image, CDocument *pDoc, CString chemin_ligne, int numBloc, int numLigne, int hauteurImageCourante)
{
	int longueur = 0; // longueur de la zone virtuelle
	int x1 = 0,x2 = 0;	// coordonnes
	int y1 = 0,y2 = 0;	// coordonnes
	int x1S = 0,x2S = 0;	// coordonnes
	int y1S = 0,y2S = 0;	// coordonnes
	int x1P = 0,x2P = 0;	// coordonnes
	int y1P = 0,y2P = 0;	// coordonnes
	
	int	distSuivant = 0;	// distance entre element courant et suivant
	int	distPrecedent = 0;	// distance entre element courant et prcdent

	int cdgcourant = 0;	// centre de gravit de l'lment courant
	int hauteurcourante = 0;	// hauteur de l'lment courant
	int largeurcourante = 0;	// largeur de l'lment courant
	
	bool precedentExiste;	// boolen : true = il existe un lment prcdent l'lment courant
	bool suivantExiste;	// boolen : true = il existe un lment suivant l'lment courant
	
	int numElement = 0;	// numro de l'lment en cours (donner dans l'ordre de traitement

	int cdg_moyen = 0;
	int largeur_moyenne = 0;
	int hauteur_moyenne = 0;
	double ecart_largeur = 0;
	double ecart_hauteur = 0;
	double ecart_cdg = 0;
	int ordonnee_Max = 0;
	int ordonnee_Min = 0;
	int pic_Max_Y_Haut = 0;
	int pic_Max_Y_Bas = 0;
	int element_traiter = 0;
	int classeElement = 0;

	double Binf_Y = 0;	// bornes de tests, Y = centre de gravit
	double Bsup_Y = 0;	// L = largeur
	double Binf_L = 0;	// H = hauteur
	double Bsup_L = 0;
	double Binf_H = 0;
	double Bsup_H = 0;

	int y1a = 0;	// Bornes definissant des lignes virtuelle sur lesquel
	int y2a = 0;	// on ce base pour classer les lments
	int y2b = 0;
	int y1b = 0;
	int y1ab = 0;
	int y2ab = 0;
	int y12a = 0; 

	bool domaineClasse1 = false;	// boolen : true =  appartient  la classe
	bool domaineClasse2 = false;	// boolen : true =  appartient  la classe
	bool domaineClasse3 = false;	// boolen : true =  appartient  la classe
	bool domaineClasse4 = false;	// boolen : true =  appartient  la classe
	bool domaineClasse5 = false;	// boolen : true =  appartient  la classe


	POSITION pEncours, pPrecedent, pSuivant, pDepartTraitement ,p1;
	CObject * temp = NULL, * temp2 = NULL;
	CElement *elementTemp = NULL, *elementS = NULL, *elementP = NULL;
	CRect *rectangle, *rectangleP, *rectangleS;

// #############" will contain the output path for the treated path

	CString chemin_nom_image_sortie; // contiendra le chemin de sortie pour l'lment trait

// #############" get the file name being processed

	CString nom_fichier=((CMyAppDoc*)pDoc)->m_nom; // rcupration du nom du fichier en cours de traitement

// #############" get the file extension being processed

	CString extension=((CMyAppDoc*)pDoc)->m_extension; // rcupration de l'extension du fichier en cours de traitement

// ############### create directories of classes

	// Cration des rpertoires des classes
	CString chemin_classe1;
	chemin_classe1.Format ("%s\\Classe1", chemin_ligne);
	_mkdir (chemin_classe1);

	CString chemin_classe2;
	chemin_classe2.Format ("%s\\Classe2", chemin_ligne);
	_mkdir (chemin_classe2);	

	CString chemin_classe3;
	chemin_classe3.Format ("%s\\Classe3", chemin_ligne);
	_mkdir (chemin_classe3);

	CString chemin_classe4;
	chemin_classe4.Format ("%s\\Classe4", chemin_ligne);
	_mkdir (chemin_classe4);
	
	CString chemin_classe5;
	chemin_classe5.Format ("%s\\Classe5", chemin_ligne);
	_mkdir (chemin_classe5);

	CString chemin_classeA;
	chemin_classeA.Format ("%s\\ClasseA", chemin_ligne);
	_mkdir (chemin_classeA);
	// fin cration des rpertoires des classes

/////////////////////////////////////////////////////////////////////////////////////////////

	// On fait une passe sur la ligne complte afin de trouver toutes les CC qui
	// sont des anomalies d au fait qu'elles ce recouvrent entre elles
	pSuivant = liste_elements->GetHeadPosition();
	(CElement *) liste_elements->GetNext(pSuivant);
	pPrecedent = liste_elements->GetHeadPosition();
	precedentExiste = false;

	for( pEncours = liste_elements->GetHeadPosition(); pEncours != NULL;)
	{
		// rcuperation des parametre element courant
		CElement *element_encours = (CElement *)liste_elements->GetNext(pEncours);
		rectangle = element_encours->getRect();
		x1 = rectangle->TopLeft().x;	// rcupre la coord x en haut  gauche
		x2 = rectangle->BottomRight().x;	// rcupre la coord x en bas  droite
		y1 = rectangle->TopLeft().y;	// rcupre la coord y en haut  gauche 
		y2 = rectangle->BottomRight().y;	// rcupre la coord y en bas  droite

		// lment suivant  l'lment courant
		if( pSuivant != NULL )
		{
			if ( (temp = liste_elements->GetNext(pSuivant)) != NULL )
			{
				suivantExiste = true ;
				elementS = (CElement *)temp;
				rectangleS = elementS->getRect();
				x1S = rectangleS->TopLeft().x;	// rcupre la coord x en haut  gauche
				x2S = rectangleS->BottomRight().x;	// rcupre la coord x en bas  droite
				y1S = rectangleS->TopLeft().y;	// rcupre la coord y en haut  gauche 
				y2S = rectangleS->BottomRight().y;	// rcupre la coord y en bas  droite
				distSuivant = abs(x1S-x2); // calcul distance avec lment suivant
			}
			else
			{
				suivantExiste = false;
			}
		}
		else
		{
			suivantExiste = false;
		}

		// lment prcdent  l'lment courant
		if( precedentExiste == true )	// on passe ici que si il existe un prcdent
		{
			if ( (temp2 = liste_elements->GetNext(pPrecedent)) != NULL )
			{
				elementP = (CElement *)temp2;
				rectangleP = elementP->getRect();
				x1P = rectangleP->TopLeft().x;	// rcupre la coord x en haut  gauche
				x2P = rectangleP->BottomRight().x;	// rcupre la coord x en bas  droite
				y1P = rectangleP->TopLeft().y;	// rcupre la coord y en haut  gauche 
				y2P = rectangleP->BottomRight().y;	// rcupre la coord y en bas  droite
				distPrecedent = abs(x1-x2P); // calcul distance avec lment suivant
			}
		}

		// test si l'element suivant existe
		if( suivantExiste == true )
		{

// ############## here we are testing for the anamolies class doubt???????????????????????

			// Test si il y a recouvrement de l'element courant avec  element suivant
			if( ((x2 >= x1S && x1S >= x1) && (y2 >= y1S && y1S >= y1)) ||	
				((x2 >= x2S && x2S >= x1) && (y2 >= y1S && y1S >= y1)) ||
				((x2 >= x1S && x1S >= x1) && (y2 >= y2S && y2S >= y1)) ||
				((x2 >= x2S && x2S >= x1) && (y2 >= y2S && y2S >= y1)) ||
						
				((x2S >= x1 && x1 >= x1S) && (y2S >= y1 && y1 >= y1S)) ||
				((x2S >= x2 && x2 >= x1S) && (y2S >= y1 && y1 >= y1S)) ||
				((x2S >= x1 && x1 >= x1S) && (y2S >= y2 && y2 >= y1S)) ||
				((x2S >= x2 && x2 >= x1S) && (y2S >= y2 && y2 >= y1S)) 	)
			{
				// on marque l'element courant et l'element suivant en anomalie
				elementS->SetClasse(6);
				element_encours->SetClasse(6);
			} 

		}

		// test si l'element precedent existe
		if( precedentExiste == true )
		{
			// Test si il y a recouvrement de l'element courant avec  element suivant
			if( ((x2 >= x1P && x1P >= x1) && (y2 >= y1P && y1P >= y1)) ||	
				((x2 >= x2P && x2P >= x1) && (y2 >= y1P && y1P >= y1)) ||
				((x2 >= x1P && x1P >= x1) && (y2 >= y2P && y2P >= y1)) ||
				((x2 >= x2P && x2P >= x1) && (y2 >= y2P && y2P >= y1)) ||
						
				((x2P >= x1 && x1 >= x1P) && (y2P >= y1 && y1 >= y1P)) ||
				((x2P >= x2 && x2 >= x1P) && (y2P >= y1 && y1 >= y1P)) ||
				((x2P >= x1 && x1 >= x1P) && (y2P >= y2 && y2 >= y1P)) ||
				((x2P >= x2 && x2 >= x1P) && (y2P >= y2 && y2 >= y1P)) 	)
			{
				// on marque l'element courant et l'element prcedent en anomalie
				elementP->SetClasse(6);
				element_encours->SetClasse(6);
			} 

		}

		precedentExiste = true; // apres une itration il existe un precedent

	}

	p1 = liste_elements->GetHeadPosition();

	// On calcul les diffrentes donnes dont on a besoin
	cdg_moyen = CalculerCdgMoyen(p1,liste_elements->GetCount());
	largeur_moyenne = CalculerLmoy(p1,liste_elements->GetCount());
	hauteur_moyenne = CalculerHmoy(p1,liste_elements->GetCount());
	ecart_largeur = CalculerEcartTypeLargeur(largeur_moyenne,p1,liste_elements->GetCount());
	ecart_hauteur = CalculerEcartTypeHauteur(hauteur_moyenne,p1,liste_elements->GetCount());
	ecart_cdg = CalculerEcartTypeCdg(cdg_moyen,p1,liste_elements->GetCount());
	ordonnee_Max = Trouver_ordonnee_maximale_nbCC(p1,liste_elements->GetCount());
	ordonnee_Min = Trouver_ordonnee_minimale_nbCC(p1,liste_elements->GetCount());
	pic_Max_Y_Haut = Creation_Histogramme_Y_Haut(hauteurImageCourante,ordonnee_Min,ordonnee_Max,p1,liste_elements->GetCount());
	pic_Max_Y_Bas = Creation_Histogramme_Y_Bas(hauteurImageCourante,ordonnee_Min,ordonnee_Max,p1,liste_elements->GetCount());

	// ############# now actually what are we trying to do doubt????????????? ecart cdg cdg moyen ?????
	
	
	if(cdg_moyen-ecart_cdg > 0) // bornes de tests, Y = centre de gravit
	{
		Binf_Y = (cdg_moyen-ecart_cdg);
	}else
	{
		Binf_Y = 0;
	}

	if(cdg_moyen+ecart_cdg > 0) // L = largeur
	{
		Bsup_Y = (cdg_moyen+ecart_cdg);
	}else
	{
		Bsup_Y = 0;
	}

	if(largeur_moyenne-ecart_largeur > 0) // H = hauteur
	{
		Binf_L = (largeur_moyenne-ecart_largeur);
	}else
	{
		Binf_L = 0;
	}

	if(largeur_moyenne+ecart_largeur > 0) 
	{
		Bsup_L = (largeur_moyenne+ecart_largeur);
	}else
	{
		Bsup_L = 0;
	}

	if(hauteur_moyenne-ecart_hauteur > 0)
	{
		Binf_H = (hauteur_moyenne-ecart_hauteur);
	}else
	{
		Binf_H = 0;
	}

	if(hauteur_moyenne+ecart_hauteur > 0) 
	{
		Bsup_H = (hauteur_moyenne+ecart_hauteur);
	}else
	{
		Bsup_H = 0;
	}

	y1a = pic_Max_Y_Haut;	// Bornes definissant des lignes virtuelle sur lesquel
	y2a = pic_Max_Y_Bas;	// on ce base pour classer les lments
	y2b = ordonnee_Max;
	y1b = ordonnee_Min;
	y1ab = ( (y1a - y1b)/2 ) + y1b;
	y2ab = ( (y2b - y2a)/2 ) + y2a;
	y12a = ( (y2a - y1a)/2 ) + y1a; 

	/* fichier de sortie rsultats pour test
		FILE *fp1;
		CString fichier;
		fichier.Format ("%s\\Resultat_%s.txt", ((CMyAppApp*)(AfxGetApp()))->m_resultat,nom_fichier);
		fp1 = fopen (fichier, "a");
		fprintf (fp1, "\n*********************************************\n");
		fprintf (fp1, "Y Moy : %d\t\tLargeur Moy : %d\t\t\tHauteur Moy : %d\n",cdg_moyen,largeur_moyenne,hauteur_moyenne);
		fprintf (fp1, "Y Ecart : %f\t\tLargeur Ecart : %f\t\tHauteur Ecart : %f\n\n",ecart_cdg,ecart_largeur,ecart_hauteur);
		fclose (fp1);
	// fin fichier de sortie rsultats	*/

// ################### now we are trying to make the output file for test results for differnet classses

	for(pDepartTraitement = liste_elements->GetHeadPosition(); pDepartTraitement != NULL ;)
	{
		// On test si l'element est dj affect  une classe
		// Si oui on fait rien, sinon on le classe
		CElement *elementTraiter = (CElement *) liste_elements->GetNext(pDepartTraitement);
		classeElement = elementTraiter->GetClasse();
		domaineClasse1 = false;
		domaineClasse2 = false;
		domaineClasse3 = false;
		domaineClasse4 = false;
		domaineClasse5 = false;


		if( classeElement == 0 )
		{	// la classe n'est pas encore dfinit on rcupre les paramtres de l'lment 
			// et on le classe
			largeurcourante = elementTraiter->getRect()->Width();
			hauteurcourante = elementTraiter->getRect()->Height();
			cdgcourant = elementTraiter->getCdg()->y; 
			x1 = elementTraiter->getRect()->TopLeft().x;	// rcupre la coord x en haut  gauche
			x2 = elementTraiter->getRect()->BottomRight().x;	// rcupre la coord x en bas  droite
			y1 = elementTraiter->getRect()->TopLeft().y;	// rcupre la coord y en haut  gauche 
			y2 = elementTraiter->getRect()->BottomRight().y;	// rcupre la coord y en bas  droite
			
			/*// fichier de sortie rsultats pour test
			FILE *fp1;
			CString fichier;
			fichier.Format ("%s\\Resultat_%s.txt", ((CMyAppApp*)(AfxGetApp()))->m_resultat, nom_fichier);
			fp1 = fopen (fichier, "a"); // ouverture et fermeture du flux pour crer le fichier
			fprintf (fp1, "Element %d ->\t\tX1 = %d\t\tY1 = %d\t\tX2 = %d\t\tY2 = %d\t\tcdgY = %d\t\tLargeur = %d\t\tHauteur = %d\t\tBinf_Y = %f\t\tBsup_Y = %f\t\tBinf_L = %f\t\tBsup_L = %f\t\tBinf_H = %f\t\tBsup_H = %f\t\tDistance Prcdent = %d\t\tDistance Suivant = %d\n",element_traiter,x1,y1,x2,y2,cdgcourant,largeurcourante,hauteurcourante,Binf_Y,Bsup_Y,Binf_L,Bsup_L,Binf_H,Bsup_H);
			fclose (fp1);
			// fin fichier de sortie rsultats */

			// classe 1: |
			if( (cdgcourant>=Binf_Y && cdgcourant<=Bsup_Y)
				&& (hauteurcourante>=Bsup_H)
				&& (largeurcourante>=(Binf_L-((ecart_largeur)/2)) && largeurcourante<=(Bsup_L+((ecart_largeur)/2)))
				&& (y1>=y1b && y1<=y1ab)
				&& (y2>=y2ab && y2<=y2b) )
			{
				elementTraiter->SetClasse(1); // Mise  jour de la classe de l'lment
				domaineClasse1 = true;
			} 
			
			// classe 2: l,f,d,b
			if( (largeurcourante>=(Binf_L-((ecart_largeur)/2)) && largeurcourante<=(Bsup_L+((ecart_largeur)/2)))
				&& (hauteurcourante>=Bsup_H)
				&& (cdgcourant<=(cdg_moyen-(ecart_cdg/3))) // rectification de la borne inf de Y pour cette classe
				&& (y1>=y1b && y1<=y1ab)
				&& (y2>=y12a && y2<=y2b) )
			{
				elementTraiter->SetClasse(2); // Mise  jour de la classe de l'lment
				domaineClasse2 = true;
			}
			
			// classe 3: p,q,g
			if( (cdgcourant>=(cdg_moyen-(ecart_cdg/3))) // rectification de la borne sup de Y pour cette classe
				&& (largeurcourante>=(Binf_L-ecart_largeur-((ecart_largeur)/2)) && largeurcourante<=(Bsup_L+ecart_largeur+((ecart_largeur)/2)))
				&& (hauteurcourante>=Bsup_H)
				&& (y1>=y1b && y1<=y12a)
				&& (y2>=y2ab && y2<=y2b) )
			{
				elementTraiter->SetClasse(3); // Mise  jour de la classe de l'lment
				domaineClasse3 = true;
			} 
			
			// classe 4: m,n,a,e,o
			if( (largeurcourante>=(Binf_L-((ecart_largeur)/2)) && largeurcourante<=(Bsup_L+((ecart_largeur)/2)))
				&& (cdgcourant>=(Binf_Y-((ecart_cdg)/2)) && cdgcourant<=(Bsup_Y+((ecart_cdg)/2)))
				&& (hauteurcourante>=Binf_H && hauteurcourante<=Bsup_H)
				&& (y1>=y1b && y1<=y12a)
				&& (y2>=y12a && y2<=y2b) )
			{
				elementTraiter->SetClasse(4); // Mise  jour de la classe de l'lment
				domaineClasse4 = true;
			} 
			
			// classe 5: A,B,E
			if( (largeurcourante>=Bsup_L)
				&& (hauteurcourante>=Bsup_H)
				&& (y1>=y1b && y1<=y1ab)
				&& (y2>=y12a && y2<=y2b) )
			{
				elementTraiter->SetClasse(5); // Mise  jour de la classe de l'lment
				domaineClasse5 = true;
			} 				
			// classe 6 : anomalies
			if( domaineClasse1 != true && domaineClasse2 != true 
				&& domaineClasse3 != true && domaineClasse4 != true
				&& domaineClasse5 != true )
			{
				elementTraiter->SetClasse(6); // Mise  jour de la classe de l'lment
			} 

		} // fin si pas de classe dfini

	} // fin for pour chaque element

	// ########### all elements of the line are listed on posters in the correct directory


	// Tous les elements de la ligne sont class, on les affiches dans le bon rpertoire	
	for( pEncours = liste_elements->GetHeadPosition(); pEncours != NULL;)
	{
		
		numElement++;
		CElement *element = (CElement *) liste_elements->GetNext(pEncours);
		classeElement = element->GetClasse();		
		largeurcourante = element->getRect()->Width();
		hauteurcourante = element->getRect()->Height();		
		rectangle = element->getRect();
		x1 = rectangle->TopLeft().x;	// rcupre la coord x en haut  gauche
		x2 = rectangle->BottomRight().x;	// rcupre la coord x en bas  droite
		y1 = rectangle->TopLeft().y;	// rcupre la coord y en haut  gauche 
		y2 = rectangle->BottomRight().y;	// rcupre la coord y en bas  droite

		// ############ new image is created that will contain the original image
		// ############ copy the current image
		// ############ crreate the thumbnail
		// ############ apply the filter


		CImage *image_courante=new CImage(); // cration d'une nouvelle CImage qui va contenir l'image originale
		image_courante->Copier(image); // recopie de l'image courante

		// Cration de l'imagette
		CImage *image_tmp =new CImage(); // cration CImage temporaire qui contiendra la sortie
		image_tmp->setTailleLxH(largeurcourante,hauteurcourante); // mise  la bonne taille
		PLFilterCrop filtre(x1,y1,x2,y2); // application d'un filtre
		try
		{
			filtre.Apply(image_courante->getPointeur(),image_tmp->getPointeur()); // recopie dans image_tmp la partie qui nous interesse de image_courante
		}
		catch(PLTextException e)
		{
			CString message;
			message.Format("Filter error application ");
			AfxMessageBox(message);
			continue;
		}
		// Fin cration imagette

		chemin_nom_image_sortie.Empty(); // vide le chemin de sortie

		// Cration du chemin selon la classe trouve prcdement
		if(classeElement==1)
		{	// classe 1
			chemin_nom_image_sortie.Format ("%s\\E%d_%s_%d.%s",chemin_classe1, numElement, nom_fichier, numElement, extension ); // chemin complet de sortie
		}
		if(classeElement==2)
		{	// classe 2
			chemin_nom_image_sortie.Format ("%s\\E%d_%s_%d.%s",chemin_classe2, numElement, nom_fichier, numElement, extension ); // chemin complet de sortie
		}
		if(classeElement==3)
		{	// classe 3
			chemin_nom_image_sortie.Format ("%s\\E%d_%s_%d.%s",chemin_classe3, numElement, nom_fichier, numElement, extension ); // chemin complet de sortie
		}
		if(classeElement==4)
		{	// classe 4
			chemin_nom_image_sortie.Format ("%s\\E%d_%s_%d.%s",chemin_classe4, numElement, nom_fichier, numElement, extension ); // chemin complet de sortie
		}
		if(classeElement==5)
		{	// classe 5
			chemin_nom_image_sortie.Format ("%s\\E%d_%s_%d.%s",chemin_classe5, numElement, nom_fichier, numElement, extension ); // chemin complet de sortie
		}
		if(classeElement==6)
		{	// anomalie
			chemin_nom_image_sortie.Format ("%s\\A_E%d_%s_%d.%s",chemin_classeA, numElement, nom_fichier, numElement, extension ); // chemin complet de sortie
		}
		// Fin de la cration du chemin selon la classe

		// Enregistrement imagette
		try
		{
			image_tmp->Sauvegarder(extension,chemin_nom_image_sortie); // sauvegarde image dans le repertoire
			delete image_tmp;
		}
		catch(PLTextException e)
		{
			CString message;
			message.Format("Error while saving file: %s",LPCTSTR(e),(LPCTSTR)e.GetCode());
			AfxMessageBox(message);
		}	
		delete image_courante;
// ####### end recording image

		// Fin enregistrement imagette
		

	}



}



/********************************************************************/
/* Nom mthode : DetectionAnomalie2
/*                                                                  
/* Fonction : permet de classer les CC dans diffrentes classes selon
/*            si elles correspondent  des anomalies ou non
/*			  Classement effectue sur une ligne en utilisant des 
/*			  morceaux afin de contrer le problme d'inclinaison
/*            
/********************************************************************/
void CLigne::DetectionAnomalie2(CImage *image, CDocument *pDoc, CString chemin_ligne, int numBloc, int numLigne, int longueur_zone_virtuelle, int hauteurImageCourante)
{
	int longueur = 0; // longueur de la zone virtuelle
	int x1 = 0,x2 = 0;	// coordonnes
	int y1 = 0,y2 = 0;	// coordonnes
	int x1S = 0,x2S = 0;	// coordonnes
	int y1S = 0,y2S = 0;	// coordonnes
	int x1P = 0,x2P = 0;	// coordonnes
	int y1P = 0,y2P = 0;	// coordonnes
	
	int	distSuivant = 0;	// distance entre element courant et suivant
	int	distPrecedent = 0;	// distance entre element courant et prcdent

	int cdgcourant = 0;	// centre de gravit de l'lment courant
	int hauteurcourante = 0;	// hauteur de l'lment courant
	int largeurcourante = 0;	// largeur de l'lment courant
	
	bool precedentExiste;	// boolen : true = il existe un lment prcdent l'lment courant
	bool suivantExiste;	// boolen : true = il existe un lment suivant l'lment courant
	
	int numElement = 0;	// numro de l'lment en cours (donner dans l'ordre de traitement

	int cdg_moyen = 0;
	int largeur_moyenne = 0;
	int hauteur_moyenne = 0;
	double ecart_largeur = 0;
	double ecart_hauteur = 0;
	double ecart_cdg = 0;
	int ordonnee_Max = 0;
	int ordonnee_Min = 0;
	int pic_Max_Y_Haut = 0;
	int pic_Max_Y_Bas = 0;
	int element_traiter = 0;
	int classeElement = 0;

	double Binf_Y = 0;	// bornes de tests, Y = centre de gravit
	double Bsup_Y = 0;	// L = largeur
	double Binf_L = 0;	// H = hauteur
	double Bsup_L = 0;
	double Binf_H = 0;
	double Bsup_H = 0;

	int y1a = 0;	// Bornes definissant des lignes virtuelle sur lesquel
	int y2a = 0;	// on ce base pour classer les lments
	int y2b = 0;
	int y1b = 0;
	int y1ab = 0;
	int y2ab = 0;
	int y12a = 0; 

	bool domaineClasse1 = false;	// boolen : true =  appartient  la classe
	bool domaineClasse2 = false;	// boolen : true =  appartient  la classe
	bool domaineClasse3 = false;	// boolen : true =  appartient  la classe
	bool domaineClasse4 = false;	// boolen : true =  appartient  la classe
	bool domaineClasse5 = false;	// boolen : true =  appartient  la classe


	POSITION pEncours, pPrecedent, pSuivant, pDepart, pFin, pDepartTraitement;
	CObject * temp = NULL, * temp2 = NULL;
	CElement *elementTemp = NULL, *elementS = NULL, *elementP = NULL;
	CRect *rectangle, *rectangleP, *rectangleS;

	CString chemin_nom_image_sortie; // contiendra le chemin de sortie pour l'lment trait

	CString nom_fichier=((CMyAppDoc*)pDoc)->m_nom; // rcupration du nom du fichier en cours de traitement
	CString extension=((CMyAppDoc*)pDoc)->m_extension; // rcupration de l'extension du fichier en cours de traitement

	// Cration des rpertoires des classes
	CString chemin_classe1;
	chemin_classe1.Format ("%s\\Classe1", chemin_ligne);
	_mkdir (chemin_classe1);

	CString chemin_classe2;
	chemin_classe2.Format ("%s\\Classe2", chemin_ligne);
	_mkdir (chemin_classe2);	

	CString chemin_classe3;
	chemin_classe3.Format ("%s\\Classe3", chemin_ligne);
	_mkdir (chemin_classe3);

	CString chemin_classe4;
	chemin_classe4.Format ("%s\\Classe4", chemin_ligne);
	_mkdir (chemin_classe4);
	
	CString chemin_classe5;
	chemin_classe5.Format ("%s\\Classe5", chemin_ligne);
	_mkdir (chemin_classe5);

	CString chemin_classeA;
	chemin_classeA.Format ("%s\\ClasseA", chemin_ligne);
	_mkdir (chemin_classeA);
	// fin cration des rpertoires des classes

/////////////////////////////////////////////////////////////////////////////////////////////

	// On fait une passe sur la ligne complte afin de trouver toutes les CC qui
	// sont des anomalies d au fait qu'elles ce recouvrent entre elles
	pSuivant = liste_elements->GetHeadPosition();
	(CElement *) liste_elements->GetNext(pSuivant);
	pPrecedent = liste_elements->GetHeadPosition();
	precedentExiste = false;

	for( pEncours = liste_elements->GetHeadPosition(); pEncours != NULL;)
	{
		// rcuperation des parametre element courant
		CElement *element_encours = (CElement *)liste_elements->GetNext(pEncours);
		rectangle = element_encours->getRect();
		x1 = rectangle->TopLeft().x;	// rcupre la coord x en haut  gauche
		x2 = rectangle->BottomRight().x;	// rcupre la coord x en bas  droite
		y1 = rectangle->TopLeft().y;	// rcupre la coord y en haut  gauche 
		y2 = rectangle->BottomRight().y;	// rcupre la coord y en bas  droite

		// lment suivant  l'lment courant
		if( pSuivant != NULL )
		{
			if ( (temp = liste_elements->GetNext(pSuivant)) != NULL )
			{
				suivantExiste = true ;
				elementS = (CElement *)temp;
				rectangleS = elementS->getRect();
				x1S = rectangleS->TopLeft().x;	// rcupre la coord x en haut  gauche
				x2S = rectangleS->BottomRight().x;	// rcupre la coord x en bas  droite
				y1S = rectangleS->TopLeft().y;	// rcupre la coord y en haut  gauche 
				y2S = rectangleS->BottomRight().y;	// rcupre la coord y en bas  droite
				distSuivant = abs(x1S-x2); // calcul distance avec lment suivant
			}
			else
			{
				suivantExiste = false;
			}
		}
		else
		{
			suivantExiste = false;
		}

		// lment prcdent  l'lment courant
		if( precedentExiste == true )	// on passe ici que si il existe un prcdent
		{
			if ( (temp2 = liste_elements->GetNext(pPrecedent)) != NULL )
			{
				elementP = (CElement *)temp2;
				rectangleP = elementP->getRect();
				x1P = rectangleP->TopLeft().x;	// rcupre la coord x en haut  gauche
				x2P = rectangleP->BottomRight().x;	// rcupre la coord x en bas  droite
				y1P = rectangleP->TopLeft().y;	// rcupre la coord y en haut  gauche 
				y2P = rectangleP->BottomRight().y;	// rcupre la coord y en bas  droite
				distPrecedent = abs(x1-x2P); // calcul distance avec lment suivant
			}
		}
		
		// test si l'element suivant existe
		if( suivantExiste == true )
		{
			// Test si il y a recouvrement de l'element courant avec  element suivant
			if( ((x2 >= x1S && x1S >= x1) && (y2 >= y1S && y1S >= y1)) ||	
				((x2 >= x2S && x2S >= x1) && (y2 >= y1S && y1S >= y1)) ||
				((x2 >= x1S && x1S >= x1) && (y2 >= y2S && y2S >= y1)) ||
				((x2 >= x2S && x2S >= x1) && (y2 >= y2S && y2S >= y1)) ||
						
				((x2S >= x1 && x1 >= x1S) && (y2S >= y1 && y1 >= y1S)) ||
				((x2S >= x2 && x2 >= x1S) && (y2S >= y1 && y1 >= y1S)) ||
				((x2S >= x1 && x1 >= x1S) && (y2S >= y2 && y2 >= y1S)) ||
				((x2S >= x2 && x2 >= x1S) && (y2S >= y2 && y2 >= y1S)) 	)
			{
				// on marque l'element courant et l'element suivant en anomalie
				elementS->SetClasse(6);
				element_encours->SetClasse(6);
			} 

		}

		// test si l'element precedent existe
		if( precedentExiste == true )
		{
			// Test si il y a recouvrement de l'element courant avec  element suivant
			if( ((x2 >= x1P && x1P >= x1) && (y2 >= y1P && y1P >= y1)) ||	
				((x2 >= x2P && x2P >= x1) && (y2 >= y1P && y1P >= y1)) ||
				((x2 >= x1P && x1P >= x1) && (y2 >= y2P && y2P >= y1)) ||
				((x2 >= x2P && x2P >= x1) && (y2 >= y2P && y2P >= y1)) ||
						
				((x2P >= x1 && x1 >= x1P) && (y2P >= y1 && y1 >= y1P)) ||
				((x2P >= x2 && x2 >= x1P) && (y2P >= y1 && y1 >= y1P)) ||
				((x2P >= x1 && x1 >= x1P) && (y2P >= y2 && y2 >= y1P)) ||
				((x2P >= x2 && x2 >= x1P) && (y2P >= y2 && y2 >= y1P)) 	)
			{
				// on marque l'element courant et l'element prcedent en anomalie
				elementP->SetClasse(6);
				element_encours->SetClasse(6);
			} 

		}

		precedentExiste = true; // apres une itration il existe un precedent

	}
	


	// On repasse sur la ligne mais maintenant on tient compte de la longueur
	// et on classe les elements selon les paramtre calcul avant
	
	// On cherche l'adresse  laquelle on va s'arrter pour la zone virtuelle
	// On s'arrte  l'adresse de fin de liste - la longueur de la zone virtuelle
	pFin = liste_elements->GetTailPosition();
	for(int i=0; (i != (longueur_zone_virtuelle-2)  && pFin != NULL) ;i++)
	{
		(CElement *) liste_elements->GetPrev(pFin); 
	}

	pDepart = liste_elements->GetHeadPosition();
	for(pDepart; (pDepart != pFin && pDepart != NULL);)
	{
		// On calcul les diffrentes donnes dont on a besoin entre Depart et Depart + longueur_zone_virtuelle
		// qui correspond  la zone que l'on traite
		cdg_moyen = CalculerCdgMoyen(pDepart,longueur_zone_virtuelle);
		largeur_moyenne = CalculerLmoy(pDepart,longueur_zone_virtuelle);
		hauteur_moyenne = CalculerHmoy(pDepart,longueur_zone_virtuelle);
		ecart_largeur = CalculerEcartTypeLargeur(largeur_moyenne,pDepart,longueur_zone_virtuelle);
		ecart_hauteur = CalculerEcartTypeHauteur(hauteur_moyenne,pDepart,longueur_zone_virtuelle);
		ecart_cdg = CalculerEcartTypeCdg(cdg_moyen,pDepart,longueur_zone_virtuelle);
		ordonnee_Max = Trouver_ordonnee_maximale_nbCC(pDepart,longueur_zone_virtuelle);
		ordonnee_Min = Trouver_ordonnee_minimale_nbCC(pDepart,longueur_zone_virtuelle);
		pic_Max_Y_Haut = Creation_Histogramme_Y_Haut(hauteurImageCourante,ordonnee_Min,ordonnee_Max,pDepart,longueur_zone_virtuelle);
		pic_Max_Y_Bas = Creation_Histogramme_Y_Bas(hauteurImageCourante,ordonnee_Min,ordonnee_Max,pDepart,longueur_zone_virtuelle);

		if(cdg_moyen-ecart_cdg > 0) // bornes de tests, Y = centre de gravit
		{
			Binf_Y = (cdg_moyen-ecart_cdg);
		}else
		{
			Binf_Y = 0;
		}

		if(cdg_moyen+ecart_cdg > 0) // L = largeur
		{
			Bsup_Y = (cdg_moyen+ecart_cdg);
		}else
		{
			Bsup_Y = 0;
		}

		if(largeur_moyenne-ecart_largeur > 0) // H = hauteur
		{
			Binf_L = (largeur_moyenne-ecart_largeur);
		}else
		{
			Binf_L = 0;
		}

		if(largeur_moyenne+ecart_largeur > 0) 
		{
			Bsup_L = (largeur_moyenne+ecart_largeur);
		}else
		{
			Bsup_L = 0;
		}

		if(hauteur_moyenne-ecart_hauteur > 0)
		{
			Binf_H = (hauteur_moyenne-ecart_hauteur);
		}else
		{
			Binf_H = 0;
		}

		if(hauteur_moyenne+ecart_hauteur > 0) 
		{
			Bsup_H = (hauteur_moyenne+ecart_hauteur);
		}else
		{
			Bsup_H = 0;
		}

		y1a = pic_Max_Y_Haut;	// Bornes definissant des lignes virtuelle sur lesquel
		y2a = pic_Max_Y_Bas;	// on ce base pour classer les lments
		y2b = ordonnee_Max;
		y1b = ordonnee_Min;
		y1ab = ( (y1a - y1b)/2 ) + y1b;
		y2ab = ( (y2b - y2a)/2 ) + y2a;
		y12a = ( (y2a - y1a)/2 ) + y1a; 

	/* fichier de sortie rsultats pour test
		FILE *fp1;
		CString fichier;
		fichier.Format ("%s\\Resultat_%s.txt", ((CMyAppApp*)(AfxGetApp()))->m_resultat,nom_fichier);
		fp1 = fopen (fichier, "a");
		fprintf (fp1, "\n*********************************************\n");
		fprintf (fp1, "Y Moy : %d\t\tLargeur Moy : %d\t\t\tHauteur Moy : %d\n",cdg_moyen,largeur_moyenne,hauteur_moyenne);
		fprintf (fp1, "Y Ecart : %f\t\tLargeur Ecart : %f\t\tHauteur Ecart : %f\n\n",ecart_cdg,ecart_largeur,ecart_hauteur);
		fclose (fp1);
	// fin fichier de sortie rsultats	*/


		// On classe les elements situe entre Depart et Depart + longueur_zone_virtuelle
		pDepartTraitement = pDepart; // on utilise pDepartTraitement pour ne pas modifier pDepart
									 // lors du parcours des elements
		element_traiter = 0;
		for(pDepartTraitement; element_traiter != longueur_zone_virtuelle && pDepart != NULL ;)
		{
			// On test si l'element est dj affect  une classe
			// Si oui on fait rien, sinon on le classe
			CElement *elementTraiter = (CElement *) liste_elements->GetNext(pDepartTraitement);
			classeElement = elementTraiter->GetClasse();
			element_traiter ++;
			domaineClasse1 = false;
			domaineClasse2 = false;
			domaineClasse3 = false;
			domaineClasse4 = false;
			domaineClasse5 = false;

			if( classeElement == 0 )
			{	// la classe n'est pas encore dfinit on rcupre les paramtres de l'lment 
				// et on le classe
				largeurcourante = elementTraiter->getRect()->Width();
				hauteurcourante = elementTraiter->getRect()->Height();
				cdgcourant = elementTraiter->getCdg()->y; 
				x1 = elementTraiter->getRect()->TopLeft().x;	// rcupre la coord x en haut  gauche
				x2 = elementTraiter->getRect()->BottomRight().x;	// rcupre la coord x en bas  droite
				y1 = elementTraiter->getRect()->TopLeft().y;	// rcupre la coord y en haut  gauche 
				y2 = elementTraiter->getRect()->BottomRight().y;	// rcupre la coord y en bas  droite

			/*// fichier de sortie rsultats pour test
				FILE *fp1;
				CString fichier;
				fichier.Format ("%s\\Resultat_%s.txt", ((CMyAppApp*)(AfxGetApp()))->m_resultat, nom_fichier);
				fp1 = fopen (fichier, "a"); // ouverture et fermeture du flux pour crer le fichier
				fprintf (fp1, "Element %d ->\t\tX1 = %d\t\tY1 = %d\t\tX2 = %d\t\tY2 = %d\t\tcdgY = %d\t\tLargeur = %d\t\tHauteur = %d\t\tBinf_Y = %f\t\tBsup_Y = %f\t\tBinf_L = %f\t\tBsup_L = %f\t\tBinf_H = %f\t\tBsup_H = %f\t\tDistance Prcdent = %d\t\tDistance Suivant = %d\n",element_traiter,x1,y1,x2,y2,cdgcourant,largeurcourante,hauteurcourante,Binf_Y,Bsup_Y,Binf_L,Bsup_L,Binf_H,Bsup_H);
				fclose (fp1);
			// fin fichier de sortie rsultats*/
	
				// classe 1: |
				if( (cdgcourant>=Binf_Y && cdgcourant<=Bsup_Y)
					&& (hauteurcourante>=Bsup_H)
					&& (largeurcourante>=(Binf_L-((ecart_largeur)/2)) && largeurcourante<=(Bsup_L+((ecart_largeur)/2)))
					&& (y1>=y1b && y1<=y1ab)
					&& (y2>=y2ab && y2<=y2b) )
				{
					elementTraiter->SetClasse(1); // Mise  jour de la classe de l'lment
					domaineClasse1 = true;
				} 
				
				// classe 2: l,f,d,b
				if( (largeurcourante>=(Binf_L-((ecart_largeur)/2)) && largeurcourante<=(Bsup_L+((ecart_largeur)/2)))
					&& (hauteurcourante>=Bsup_H)
					&& (cdgcourant<=(cdg_moyen-(ecart_cdg/3))) // rectification de la borne inf de Y pour cette classe
					&& (y1>=y1b && y1<=y1ab)
					&& (y2>=y12a && y2<=y2b) )
				{
					elementTraiter->SetClasse(2); // Mise  jour de la classe de l'lment
					domaineClasse2 = true;
				}
				
				// classe 3: p,q,g
				if( (cdgcourant>=(cdg_moyen-(ecart_cdg/3))) // rectification de la borne sup de Y pour cette classe
					&& (largeurcourante>=(Binf_L-ecart_largeur-((ecart_largeur)/2)) && largeurcourante<=(Bsup_L+ecart_largeur+((ecart_largeur)/2)))
					&& (hauteurcourante>=Bsup_H)
					&& (y1>=y1b && y1<=y12a)
					&& (y2>=y2ab && y2<=y2b) )
				{
					elementTraiter->SetClasse(3); // Mise  jour de la classe de l'lment
					domaineClasse3 = true;
				} 
				
				// classe 4: m,n,a,e,o
				if( (largeurcourante>=(Binf_L-((ecart_largeur)/2)) && largeurcourante<=(Bsup_L+((ecart_largeur)/2)))
					&& (cdgcourant>=(Binf_Y-((ecart_cdg)/2)) && cdgcourant<=(Bsup_Y+((ecart_cdg)/2)))
					&& (hauteurcourante>=Binf_H && hauteurcourante<=Bsup_H)
					&& (y1>=y1b && y1<=y12a)
					&& (y2>=y12a && y2<=y2b) )
				{
					elementTraiter->SetClasse(4); // Mise  jour de la classe de l'lment
					domaineClasse4 = true;
				} 
				
				// classe 5: A,B,E
				if( (largeurcourante>=Bsup_L)
					&& (hauteurcourante>=Bsup_H)
					&& (y1>=y1b && y1<=y1ab)
					&& (y2>=y12a && y2<=y2b) )
				{
					elementTraiter->SetClasse(5); // Mise  jour de la classe de l'lment
					domaineClasse5 = true;
				} 				

				// classe 6 : anomalies
				if( domaineClasse1 != true && domaineClasse2 != true 
					&& domaineClasse3 != true && domaineClasse4 != true
					&& domaineClasse5 != true )
				{
					elementTraiter->SetClasse(6); // Mise  jour de la classe de l'lment
				} 

			} // fin si pas de classe dfini

		} // fin for pour chaque element

		(CElement *) liste_elements->GetNext(pDepart);

	} // fin for pour la zone virtuelle


	// Tous les elements de la ligne sont class, on les affiches dans le bon rpertoire	
	for( pEncours = liste_elements->GetHeadPosition(); pEncours != NULL;)
	{
		
		numElement++;
		CElement *element = (CElement *) liste_elements->GetNext(pEncours);
		classeElement = element->GetClasse();		
		largeurcourante = element->getRect()->Width();
		hauteurcourante = element->getRect()->Height();		
		rectangle = element->getRect();
		x1 = rectangle->TopLeft().x;	// rcupre la coord x en haut  gauche
		x2 = rectangle->BottomRight().x;	// rcupre la coord x en bas  droite
		y1 = rectangle->TopLeft().y;	// rcupre la coord y en haut  gauche 
		y2 = rectangle->BottomRight().y;	// rcupre la coord y en bas  droite

		CImage *image_courante=new CImage(); // cration d'une nouvelle CImage qui va contenir l'image originale
		image_courante->Copier(image); // recopie de l'image courante

		// Cration de l'imagette
		CImage *image_tmp =new CImage(); // cration CImage temporaire qui contiendra la sortie
		image_tmp->setTailleLxH(largeurcourante,hauteurcourante); // mise  la bonne taille
		PLFilterCrop filtre(x1,y1,x2,y2); // application d'un filtre
		try
		{
			filtre.Apply(image_courante->getPointeur(),image_tmp->getPointeur()); // recopie dans image_tmp la partie qui nous interesse de image_courante
		}
		catch(PLTextException e)
		{
			CString message;
			message.Format("filter application error");
			AfxMessageBox(message);
			continue;
		}
		// Fin cration imagette

		chemin_nom_image_sortie.Empty(); // vide le chemin de sortie

		// Cration du chemin selon la classe trouve prcdement
		if(classeElement==1)
		{	// classe 1
			chemin_nom_image_sortie.Format ("%s\\E%d_%s_%d.%s",chemin_classe1, numElement, nom_fichier, numElement, extension ); // chemin complet de sortie
		}
		if(classeElement==2)
		{	// classe 2
			chemin_nom_image_sortie.Format ("%s\\E%d_%s_%d.%s",chemin_classe2, numElement, nom_fichier, numElement, extension ); // chemin complet de sortie
		}
		if(classeElement==3)
		{	// classe 3
			chemin_nom_image_sortie.Format ("%s\\E%d_%s_%d.%s",chemin_classe3, numElement, nom_fichier, numElement, extension ); // chemin complet de sortie
		}
		if(classeElement==4)
		{	// classe 4
			chemin_nom_image_sortie.Format ("%s\\E%d_%s_%d.%s",chemin_classe4, numElement, nom_fichier, numElement, extension ); // chemin complet de sortie
		}
		if(classeElement==5)
		{	// classe 5
			chemin_nom_image_sortie.Format ("%s\\E%d_%s_%d.%s",chemin_classe5, numElement, nom_fichier, numElement, extension ); // chemin complet de sortie
		}
		if(classeElement==6)
		{	// anomalie
			chemin_nom_image_sortie.Format ("%s\\A_E%d_%s_%d.%s",chemin_classeA, numElement, nom_fichier, numElement, extension ); // chemin complet de sortie
		}
		// Fin de la cration du chemin selon la classe

		// Enregistrement imagette
		try
		{
			image_tmp->Sauvegarder(extension,chemin_nom_image_sortie); // sauvegarde image dans le repertoire
			delete image_tmp;
		}
		catch(PLTextException e)
		{
			CString message;
			message.Format("Error while saving file: %s",LPCTSTR(e),(LPCTSTR)e.GetCode());
			AfxMessageBox(message);
		}	
		delete image_courante;
		// Fin enregistrement imagette
		

	}

}



/********************************************************************/
/* Nom mthode : Trouver_abscisse_maximale
/*                                                                  
/* Fonction : permet de trouver l'abscisse maximale sur la ligne
/*            
/*            
/********************************************************************/
void CLigne::Trouver_abscisse_maximale ()
{
	int max;	
	int x_droite;
	POSITION p1;

	max = 0;

	for (p1 = liste_elements->GetHeadPosition();p1 != NULL;)
	{
		CElement *e = (CElement *)liste_elements->GetNext(p1);
		x_droite = e->getRect ()->right;

		if (max < x_droite)
			max = x_droite;
	}

	abscisse_maximale = max;
}



/********************************************************************/
/* Nom mthode : Trouver_abscisse_minimale
/*                                                                  
/* Fonction : permet de trouver l'abscisse minimale sur la ligne
/*            
/*            
/********************************************************************/
void CLigne::Trouver_abscisse_minimale ()
{
	int min;
	int x_gauche;
	POSITION p1;

	min = 1000000;

	for (p1 = liste_elements->GetHeadPosition();p1 != NULL;)
	{
		CElement *e = (CElement *)liste_elements->GetNext(p1);
		x_gauche = e->getRect()->left;

		if (min > x_gauche)
			min = x_gauche;
	}

	abscisse_minimale = min;
}



/********************************************************************/
/* Nom mthode : Trouver_abscisse_maximale_nbCC
/*                                                                  
/* Fonction : permet de trouver l'abscisse maximale sur la ligne
/*            
/*            
/********************************************************************/
int CLigne::Trouver_abscisse_maximale_nbCC(POSITION depart, int nbC)
{
	int max;	
	int x_droite;
	int resultat=0;
	POSITION p1;
	int nbElementTraite=0;
	max = 0;

	for (p1 = depart;p1 != NULL && nbElementTraite != nbC;)
	{
		CElement *e = (CElement *)liste_elements->GetNext(p1);
		x_droite = e->getRect ()->right;

		if (max < x_droite)
			max = x_droite;

		nbElementTraite++;
	}

	resultat = max;
	return(resultat);
}



/********************************************************************/
/* Nom mthode : Trouver_abscisse_minimale_nbCC
/*                                                                  
/* Fonction : permet de trouver l'abscisse minimale sur la ligne
/*            
/*            
/********************************************************************/
int CLigne::Trouver_abscisse_minimale_nbCC(POSITION depart, int nbC)
{
	int min;
	int x_gauche;
	int resultat=0;
	POSITION p1;
	int nbElementTraite=0;
	min = 1000000;

	for (p1 = depart;p1 != NULL && nbElementTraite != nbC;)
	{
		CElement *e = (CElement *)liste_elements->GetNext(p1);
		x_gauche = e->getRect()->left;

		if (min > x_gauche)
			min = x_gauche;

		nbElementTraite++;
	}

	resultat = min;
	return(resultat);
}



/********************************************************************/
/* Nom mthode : Trouver_ordonnee_minimale_nbCC
/*                                                                  
/* Fonction : permet de trouver l'ordonnee maximale sur la ligne
/*            
/*            
/********************************************************************/
int CLigne::Trouver_ordonnee_minimale_nbCC(POSITION depart, int nbC)
{
	int min;
	POSITION p1;
	int resultat=0;
	int nbElementTraite=0;
	min = 100000;

	for (p1 = depart;p1 != NULL && nbElementTraite != nbC;)
	{
		int y_haut;
		CElement *e = (CElement *)liste_elements->GetNext(p1);
		y_haut = e->getRect ()->top;
		if (min > y_haut)
			min = y_haut;

		nbElementTraite++;
	}

	resultat = min;
	return(resultat);

}



/********************************************************************/
/* Nom mthode : Trouver_ordonnee_maximale_nbCC
/*                                                                  
/* Fonction : permet de trouver l'ordonnee maximale sur la ligne
/*            
/*            
/********************************************************************/
int CLigne::Trouver_ordonnee_maximale_nbCC(POSITION depart, int nbC)
{
	int max;
	POSITION p1;
	int resultat=0;
	int nbElementTraite=0;
	max = 0;

	for (p1 = depart;p1 != NULL && nbElementTraite != nbC;)
	{
		int y_bas;
		CElement *e = (CElement *)liste_elements->GetNext(p1);
		y_bas = e->getRect ()->bottom;
		if (max < y_bas)
			max = y_bas;

		nbElementTraite++;
	}

	resultat = max;
	return(resultat);
}

/********************************************************************/
/* Nom mthode : get_abscisse_minimale
/*                                                                  
/* Fonction : permet de rcuprer l'abscisse minimale sur la ligne
/*            
/*            
/********************************************************************/
int CLigne::get_abscisse_minimale ()
{
	return abscisse_minimale;
}



/********************************************************************/
/* Nom mthode : get_abscisse_maximale
/*                                                                  
/* Fonction : permet de rcuprer l'abscisse maximale sur la ligne
/*            
/*            
/********************************************************************/
int CLigne::get_abscisse_maximale ()
{
	return abscisse_maximale;
}



/********************************************************************/
/* Nom mthode : getRect
/*                                                                  
/* Fonction : retourne le rectangle englobant de la ligne
/*            
/*            
/********************************************************************/
CRect * CLigne::getRect()
{
	
	return rectL;

}



/********************************************************************/
/* Nom mthode : MiseAJourRectangle
/*                                                                  
/* Fonction : permet de mettre a jour la taille du rectangle englobant
/*            la ligne, il faut lancer Trouver_abscisse_minimale,
/*            Trouver_abscisse_maximale, Trouver_ordonnee_minimale
/*			  Trouver_ordonnee_maximale avant de l'utiliser
/********************************************************************/
void CLigne::MiseAJourRectangle()
{

	rectL->SetRect(abscisse_minimale,ordonnee_minimale,abscisse_maximale,ordonnee_maximale);

}



/********************************************************************/
/* Nom mthode : MiseAJourRectangle
/*                                                                  
/* Fonction : permet de mettre a jour la taille du rectangle englobant
/*            la ligne, il faut lancer Trouver_abscisse_minimale,
/*            Trouver_abscisse_maximale, Trouver_ordonnee_minimale
/*			  Trouver_ordonnee_maximale avant de l'utiliser
/*
/* Sortie : la liste n'a qu'un element -> largeur de l'element
/*			la liste  plusieurs elements -> distance moyenne entre les cdg
/*
/********************************************************************/
int CLigne::CalculerMoyenneEcartEntreCdg()
{

	POSITION p1,p2,p3;
	int resultat = 0;
	int cdg_e1_x = 0;
	int cdg_e1_y = 0;
	int cdg_e2_x = 0;
	int cdg_e2_y = 0;
	int distance = 0;
	int distanceFinale = 0;

	if( liste_elements->GetCount() != 1 )
	{
		p3 = liste_elements->GetHeadPosition();
		CElement *e = (CElement *)liste_elements->GetNext(p3);

		for (p2 = liste_elements->GetHeadPosition();p3 != NULL;) // pour chaque element de la ligne
		{
			CElement *e1 = (CElement *)liste_elements->GetNext(p2); // on met dans e1 l'lment
			CElement *e2 = (CElement *)liste_elements->GetNext(p3); // on met dans e2 l'lment
			
			cdg_e1_x = e1->getCdg()->x;
			cdg_e1_y = e1->getCdg()->y;
			cdg_e2_x = e2->getCdg()->x;
			cdg_e2_y = e2->getCdg()->y;

			distance = sqrt( (double)((cdg_e2_x - cdg_e1_x)^2) + (double)((cdg_e2_y - cdg_e1_y)^2) );

			distanceFinale = distanceFinale + distance;

			distance = 0;

		}

		resultat = distanceFinale / ( liste_elements->GetCount() ) - 1;
		
	}
	else
	{
		p1 = liste_elements->GetHeadPosition(); // on recupre un pointeur sur le seul element de la liste
		CElement *e1 = (CElement *)liste_elements->GetNext(p1); // on met dans e1 l'lment
		resultat = e1->getRect()->Width();	// on recupre la largeur
	}

	return(resultat);
}



/********************************************************************/
/* Nom mthode : CalculerEcartTypeDistanceCdg
/*                                                                  
/* Fonction : permet de calculer l'cart type au niveau de la distance
/*            sparant 2 cdg 
/*            
/* Sortie : si il y a qu'un element dans la liste -> resultat = 1
/*			si il y a plusieurs elements -> resultat = ecart type distance cdg
/*
/********************************************************************/
double CLigne::CalculerEcartTypeDistanceCdg(int moyenne)
{
	POSITION p2,p3;
	int cdg_e1_x = 0;
	int cdg_e1_y = 0;
	int cdg_e2_x = 0;
	int cdg_e2_y = 0;
	int distance = 0;
	double resultat=0;
	double resultat2=0;
	double nbElement=0;

	if( liste_elements->GetCount() != 1 )
	{
		p3 = liste_elements->GetHeadPosition();
		CElement *e = (CElement *)liste_elements->GetNext(p3);

		for (p2 = liste_elements->GetHeadPosition();p3 != NULL;) // pour chaque element de la ligne
		{
			CElement *e1 = (CElement *)liste_elements->GetNext(p2); // on met dans e1 l'lment
			CElement *e2 = (CElement *)liste_elements->GetNext(p3); // on met dans e2 l'lment
				
			cdg_e1_x = e1->getCdg()->x;
			cdg_e1_y = e1->getCdg()->y;
			cdg_e2_x = e2->getCdg()->x;
			cdg_e2_y = e2->getCdg()->y;

			distance = sqrt( (double)((cdg_e2_x - cdg_e1_x)^2) + (double)((cdg_e2_y - cdg_e1_y)^2) );

			resultat = resultat + pow((distance - moyenne),2); // somme
			distance = 0; // remise  zro
		}
		nbElement = ( liste_elements->GetCount() - 1 );
		resultat2 = sqrt ( ( 1/nbElement ) * resultat ); // racine
	}
	else
	{
		resultat2 = 2;
	}
	return(resultat2);
}



/********************************************************************/
/* Nom mthode : MettreAJourParametreLigne
/*                                                                  
/* Fonction : permet de mettre  jour les paramtres de la ligne
/*            
/* Sortie : le rectangle englobant de la ligne est mise  jour
/*			
/********************************************************************/
void CLigne::MettreAJourParametreLigne()
{

	Calculer_cdg_moyen ();
	Trouver_ordonnee_maximale ();
	Trouver_ordonnee_minimale ();
	Trouver_abscisse_maximale ();
	Trouver_abscisse_minimale ();
	MiseAJourRectangle();
}



/********************************************************************/
/* Nom mthode : get_ordonnee_minimale_virtuel
/*                                                                  
/* Fonction : permet de trouver l'ordonne minimale virtuelle de la ligne
/*			  cette ordonne est recherch parmie les X dernire composante
/*            de la ligne
/*            
/* Entre : le nombre de composantes prise en compte nbC
/* Sortie : ordonne minimale
/*			
/********************************************************************/
int CLigne::get_ordonnee_minimale_virtuel(int nbC)
{

	int resultat = 0;
	int min;
	int y_haut;
	POSITION p1;

	if( liste_elements->GetCount() >= nbC )
	{
		min = 1000000;

		for (p1 = liste_elements->GetTailPosition() ; nbC != 0 ;)
		{
			CElement *e = (CElement *)liste_elements->GetPrev(p1);
			y_haut = e->getRect()->top;

			if (min > y_haut)
				min = y_haut;
			nbC--;
		}

		resultat = min;


	}

	return(resultat);
}



/********************************************************************/
/* Nom mthode : get_ordonnee_maximale_virtuel
/*                                                                  
/* Fonction : permet de trouver l'ordonne maximale virtuelle de la ligne
/*			  cette ordonne est recherch parmie les X dernire composante
/*            de la ligne
/*            
/* Entre : le nombre de composantes prise en compte nbC
/* Sortie : ordonne maximale
/*			
/********************************************************************/
int CLigne::get_ordonnee_maximale_virtuel(int nbC)
{
	int resultat = 0;
	int max;
	int y_bas;
	POSITION p1;

	if( liste_elements->GetCount() >= nbC )
	{
		max = 0;

		for (p1 = liste_elements->GetTailPosition() ; nbC != 0 ;)
		{
			CElement *e = (CElement *)liste_elements->GetPrev(p1);
			y_bas = e->getRect()->bottom;

			if (max < y_bas)
				max = y_bas;
			nbC--;
		}

		resultat = max;


	}

	return(resultat);
}



/********************************************************************/
/* Nom mthode : get_abscisse_minimale_virtuel
/*                                                                  
/* Fonction : permet de trouver l'abscisse minimale virtuelle de la ligne
/*			  cette ordonne est recherch parmie les X dernire composante
/*            de la ligne
/*            
/* Entre : le nombre de composantes prise en compte nbC
/* Sortie : abscisse minimale
/*			
/********************************************************************/
int CLigne::get_abscisse_minimale_virtuel(int nbC)
{
	int resultat = 0;
	int min;
	int x_gauche;
	POSITION p1;

	if( liste_elements->GetCount() >= nbC )
	{
		min = 1000000;

		for (p1 = liste_elements->GetTailPosition() ; nbC != 0 ;)
		{
			CElement *e = (CElement *)liste_elements->GetPrev(p1);
			x_gauche = e->getRect()->left;

			if (min > x_gauche)
				min = x_gauche;
			nbC--;
		}

		resultat = min;


	}

	return(resultat);
}



/********************************************************************/
/* Nom mthode : get_abscisse_maximale_virtuel
/*                                                                  
/* Fonction : permet de trouver l'abscisse maximale virtuelle de la ligne
/*			  cette ordonne est recherch parmie les X dernire composante
/*            de la ligne
/*            
/* Entre : le nombre de composantes prise en compte nbC
/* Sortie : abscisse maximale
/*			
/********************************************************************/
int CLigne::get_abscisse_maximale_virtuel(int nbC)
{
	int resultat = 0;
	int max;
	int x_droite;
	POSITION p1;

	if( liste_elements->GetCount() >= nbC )
	{
		max = 0;

		for (p1 = liste_elements->GetTailPosition() ; nbC != 0 ;)
		{
			CElement *e = (CElement *)liste_elements->GetPrev(p1);
			x_droite = e->getRect()->right;

			if (max < x_droite)
				max = x_droite;
			nbC--;
		}

		resultat = max;


	}

	return(resultat);
}



/********************************************************************/
/* Nom mthode : Calculer_cdg_moyen_nbCC
/*                                                                  
/* Fonction : permet de calculer le cdg moyen sur un nombre d'lement 
/*			  spcifi en paramtre, le calcul commence par la fin de 
/*            la liste des lments composant la ligne
/*            
/* Entre : le nombre de composantes prise en compte nbC
/* Sortie : cdg moyen
/*			
/********************************************************************/
int CLigne::Calculer_cdg_moyen_nbCC (int nbC)
{
	POSITION p1;
	int cdg_moyen = 0;
	int resultat = 0;
	int nb_element = nbC;

	if(liste_elements->GetCount() >= nbC)
	{
		for (p1 = liste_elements->GetTailPosition(); ( nbC !=0 && p1 !=NULL );)
		{
			CElement *e = (CElement *)liste_elements->GetPrev(p1);
			cdg_moyen = cdg_moyen + ( e->getCdg()->y );
			
			nbC--;
		}

		resultat = cdg_moyen/( nb_element );
	}
	else
	{
		nbC = liste_elements->GetCount();

		for (p1 = liste_elements->GetTailPosition(); ( nbC !=0 && p1 !=NULL );)
		{
			CElement *e = (CElement *)liste_elements->GetPrev(p1);
			cdg_moyen = cdg_moyen + ( e->getCdg()->y );
			
			nbC--;
		}

		resultat = cdg_moyen/( liste_elements->GetCount () );

	}
	return( resultat );
}



/********************************************************************/
/* Nom mthode : Calculer_ecart_type_cdg_nbCC
/*                                                                  
/* Fonction : permet de calculer l'cart type au niveau du centre de
/*            gravit en prenant en compte seulement quelques composantes
/*            connexes, le calcul commence par la fin de la liste des CC
/*            de la ligne
/* Entre : nbC = nombre de composantes  traiter
/* Sortie : ecart type
/*            
/********************************************************************/
double CLigne::Calculer_ecart_type_cdg_nbCC(int moyenne, int nbC)
{
	POSITION p1;
	int cdgcourant = 0;
	double resultat = 0;
	double resultat2 = 0;
	double nbElement = nbC;


	if(liste_elements->GetCount() >= nbC)
	{
		for (p1 = liste_elements->GetTailPosition(); ( nbC !=0 && p1 !=NULL );)
		{
			CElement *e = (CElement *)liste_elements->GetPrev(p1);
			cdgcourant = e->getCdg()->y;
			resultat = resultat + pow((cdgcourant - moyenne),2); // somme
			cdgcourant = 0; // remise  zro
			
			nbC--;
		}

		resultat2 = sqrt ((1/nbElement)*resultat); // racine

	}
	else
	{
		for (p1 = liste_elements->GetTailPosition(); ( nbC !=0 && p1 !=NULL );)
		{
			CElement *e = (CElement *)liste_elements->GetPrev(p1);
			cdgcourant = e->getCdg()->y;
			resultat = resultat + pow((cdgcourant - moyenne),2); // somme
			cdgcourant = 0; // remise  zro
			
			nbC--;
		}

		nbElement=(liste_elements->GetCount());
		resultat2 = sqrt ((1/nbElement)*resultat); // racine

	}
	return(resultat2);
}

