
#pragma once

#include <afxtempl.h>


/**
@brief Classe effectuant les calculs mathmatiques qui permettent ensuite de dterminer
quelle(s) caractristique(s) sont les plus appropries pour appliquer les rgles
correspondantes dans un "scnario automatique"
@author Clment Cureau
@file ParamCarac.h
@version 1.0
@date 08-03-2007
*/

class CParamCarac : public CObject 
{
private:
	/**
	contient le nom de la caratristique (ex. : RapportLSurH, ...)
	*/
	CString nomCarac;
	/**
	contient le numero de la caratristique (ex. : 1,2, ...) avec les correspondances
	dans constante.h
	*/
	int numCarac;
	/**
	contient la liste des valeurs de la caractristique
	*/
	CList<double> *listeValeurs;
	/**
	contient la borne infrieure de l'intervalle
	(soit dtermine statiquement, soit dtermine en fonction des autres zones
	appartenant  la mme classe)
	*/
	double borneInf;
	/**
	contient la borne suprieure de l'intervalle
	(soit dtermine statiquement, soit dtermine en fonction des autres zones
	appartenant  la mme classe)
	*/
	double borneSup;
	/**
	contient la valeur minimum de toute la population
	*/
	double mini;
	/**
	contient la valeur maximum de toute la population
	*/
	double maxi;
	/**
	contient la variance de toute la population
	*/
	double variance;
	/**
	contient l'cart-type de toute la population
	*/
	double ecartType;
	/**
	contient la valeur moyenne arithmtique de toute la population
	*/
	double moyenne;
	/**
	contient la valeur mdiane de toute la population
	*/
	double mediane;

	/**
	contient la valeur minimum norme (ramene entre 0 et 100) de toute la population,
	donc ici 0
	*/
	double miniNorm;
	/**
	contient la valeur maximum norme (ramene entre 0 et 100) de toute la population,
	donc ici 100
	*/
	double maxiNorm;


public:
	/**
	@brief Accesseur de la variable nomCarac
	@param nom de la caractristique  mettre  jour (CString)
	@return 
	@note
	*/
	void setNomCarac(CString valeurDeLaCaracteristique){nomCarac = valeurDeLaCaracteristique;}
	/**
	@brief Accesseur de la variable numCarac
	@param numero de la caractristique  mettre  jour (int)
	@return 
	@note les correspondances sont disponibles dans constante.h
	*/
	void setNumCarac(int val){numCarac = val;}
	/**
	@brief Accesseur de la variable ListeValeurs
	@param nom de la caractristique  mettre  jour (CList<double>*)
	@return 
	@note recopie la liste ici
	*/
	void setListeValeurs(CList<double> *liste)
	{
		POSITION posListe = liste->GetHeadPosition();
		while(posListe)
		{
			listeValeurs->AddTail(liste->GetNext(posListe));
		}
	}
	/**
	@brief Ajout d'une valeur  la variable ListeValeurs
	@param valeur a ajouter (double)
	@return 
	@note
	*/
	void addValeur(double valeur){listeValeurs->AddTail(valeur);}
	/**
	@brief Accesseur de la variable borneInf
	@param valeur de la borne infrieure  mettre  jour (double)
	@return 
	@note
	*/
	void setBorneInf(double val){borneInf = val;}
	/**
	@brief Accesseur de la variable borneSup
	@param valeur de la borne suprieure  mettre  jour (double)
	@return 
	@note
	*/
	void setBorneSup(double val){borneSup = val;}
	/**
	@brief Accesseur de la variable mini
	@param valeur du minimum  mettre  jour (double)
	@return 
	@note
	*/
	void setMini(double val){mini = val;}
	/**
	@brief Accesseur de la variable maxi
	@param valeur du maximum  mettre  jour (double)
	@return 
	@note
	*/
	void setMaxi(double val){maxi = val;}
	/**
	@brief Accesseur de la variable variance
	@param valeur de la variance  mettre  jour (double)
	@return 
	@note
	*/
	void setVariance(double val){variance = val;}
	/**
	@brief Accesseur de la variable variance
	@param valeur de la variance  mettre  jour (double)
	@return 
	@note
	*/
	void setEcartType(double val){ecartType = val;}
	/**
	@brief Accesseur de la variable moyenne
	@param valeur de la moyenne  mettre  jour (double)
	@return 
	@note
	*/
	void setMoyenne(double val){moyenne = val;}
	/**
	@brief Accesseur de la variable mediane
	@param valeur de la mdiane  mettre  jour (double)
	@return 
	@note
	*/
	void setMediane(double val){mediane = val;}
	/**
	@brief Accesseur de la variable miniNorm
	@param valeur du minimum norm (ramen entre 0 et 100)  mettre  jour (double)
	@return 
	@note
	*/
	void setMiniNorm(double val){miniNorm = val;}
	/**
	@brief Accesseur de la variable maxiNorm
	@param valeur du maximum norm (ramen entre 0 et 100)  mettre  jour (double)
	@return 
	@note
	*/
	void setMaxiNorm(double val){maxiNorm = val;}

	CString getNomCarac(){return nomCarac;}
	int getNumCarac(){return numCarac;}
	CList<double>* getListeValeurs(){return listeValeurs;}
	double getBorneInf(){return borneInf;}
	double getBorneSup(){return borneSup;}
	double getMini(){return mini;}
	double getMaxi(){return maxi;}
	double getVariance(){return variance;}
	double getEcartType(){return ecartType;}
	double getMoyenne(){return moyenne;}
	double getMediane(){return mediane;}
	double getMiniNorm(){return miniNorm;}
	double getMaxiNorm(){return maxiNorm;}

	void DeterminerBornes(int numeroCarac);
	int CalculerClasseMajeure();

	void ViderParam();

	void ToutCalculer();
	double CalculerMini(CList<double> *listeACalculer);
	double CalculerMaxi(CList<double> *listeACalculer);
	double CalculerVariance(CList<double> *listeACalculer, double moyCalculee);
	double CalculerEcartType(double varianceCalculee);
	double CalculerMoyenne(CList<double> *listeACalculer);
	double CalculerMediane(CList<double> *listeACalculer);
	CList<double>* NormaliserValeurs(CList<double> *listeACalculer, double miniCalcule, double maxiCalcule);
	double CalculNormalisation(double valeurANormaliser);

	void QuickSort(double tabATrier[], int minimum, int maximum);
	int Partitionner(double tabATrier[], int minimum, int maximum);

	CParamCarac(CString nomACreer, int numeroCarac);
	~CParamCarac(void);
};
