
#pragma once

#include <afxtempl.h>
#include "TypeClasse.h"


/**
@brief Classe effectuant les calculs mathmatiques qui permettent ensuite de dterminer
quelle(s) caractristique(s) sont les plus appropries pour appliquer les rgles
correspondantes dans un "scnario automatique"
@author Clment Cureau
@file ParamCarac.h
@version 1.0
@date 08-03-2007
*/

class CParamClasse : public CObject 
{
private:
	/**
	contient le nom de la caratristique (ex. : RapportLSurH, ...)
	*/
	CString nomCarac;
	/**
	contient le numero de la caratristique (ex. : 1,2, ...) avec les correspondances
	dans constante.h
	*/
	int numCarac;
	/**
	contient la liste des valeurs de la caractristique (liste des numeros correspondant
	a chaque classe contenue dans la selection)
	*/
	CList<int> *listeValeurs;
	/**
	contient la liste des classes de la caractristique (liste des noms correspondant
	a chaque classe contenue dans la selection)
	*/
	CList<CString,CString> *listeClasses;
	/**
	contient le nombre de classes contenues dans la slection
	*/
	int nbClasses;
	/**
	contient la classe qui est retenue
	*/
	CTypeClasse *classeChoisie;
	/**
	contient le pourcentage de la classe choisie
	*/
	double pourcentageClasse;
	


public:
	/**
	@brief Accesseur de la variable nomCarac
	@param nom de la caractristique  mettre  jour (CString)
	@return 
	@note
	*/
	void setNomCarac(CString valeurDeLaCaracteristique){nomCarac = valeurDeLaCaracteristique;}
	/**
	@brief Accesseur de la variable numCarac
	@param numero de la caractristique  mettre  jour (int)
	@return 
	@note les correspondances sont disponibles dans constante.h
	*/
	void setNumCarac(int val){numCarac = val;}
	/**
	@brief Accesseur de la variable ListeValeurs
	@param nom de la caractristique  mettre  jour (CList<double>*)
	@return 
	@note recopie la liste ici
	*/
	void setListeValeurs(CList<int> *liste)
	{
		POSITION posListe = liste->GetHeadPosition();
		while(posListe)
		{
			listeValeurs->AddTail(liste->GetNext(posListe));
		}
	}
	/**
	@brief Ajout d'une valeur  la variable ListeValeurs
	@param valeur a ajouter (double)
	@return 
	@note
	*/
	void addValeur(int valeur){listeValeurs->AddTail(valeur);}
	/**
	@brief Accesseur de la variable borneInf
	@param valeur de la borne infrieure  mettre  jour (double)
	@return 
	@note
	*/
	void setNbClasses(int val){nbClasses = val;}
	
	void ViderParam();

	CString getNomCarac(){return nomCarac;}
	int getNumCarac(){return numCarac;}
	CList<int>* getListeValeurs(){return listeValeurs;}
	CList<CString,CString>* getListeClasses(){return listeClasses;}
	int getNbClasses(){return nbClasses;}
	CTypeClasse* getClasseChoisie(){return classeChoisie;}
	double getPourcentageClasse(){return pourcentageClasse;}
	
	void ToutCalculer(CObList* listeTypesClasse);
	int CalculerClasseMajeure();
	double CalculerPourcentage();
	void RemplirListeClasses(CObList* listeTypesClasse);
	CTypeClasse* TrouverClasse(int numClasse);


	CParamClasse(CString nomACreer, int numeroCarac);
	~CParamClasse(void);
};
