/****************************************************************/
// RegleCC.cpp: implementation of the CRegleCC class.
//
// Permet la gestion des rgle de classification selon les
// les caractristique des zones
//
/******************************************************************/


/*****************/
// INCLUSION
/*****************/

#include "stdafx.h"
#include "myapp.h"
#include "RegleCC.h"
#include "myappdoc.h"
#include "myappview.h"
#include "constante.h"



/*****************/
// PRECOMPILATION
/*****************/



#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


/*****************/
// CONSTRUCTEUR
/*****************/


CRegleCC::CRegleCC()
{
	balise = "RCC";
}

CRegleCC::CRegleCC(CString org, CString dest, CString dest_sinon, double pHLmin,double pHLmax,int pNBCCmin,int pNBCCmax,double pHHmin,double pHHmax,int pLmin,int pLmax,int pHmin,int pHmax, double pDPBmin, double pDPBmax, int pDNdgmin, int pDNdgmax)
{

	balise = "RCC";
	type_org = org;
	type_dest = dest;
	type_dest_sinon = dest_sinon;

	HLmin = pHLmin;
	HLmax = pHLmax;
	NBCCmin = pNBCCmin;
	NBCCmax = pNBCCmax;
	HHmin = pHHmin;
	HHmax = pHHmax;
	Lmin = pLmin;
	Lmax = pLmax;
	Hmin = pHmin;
	Hmax = pHmax;
	DPBmin = pDPBmin;
	DPBmax = pDPBmax;
	DNdgmin = pDNdgmin;
	DNdgmax = pDNdgmax;
}


/*****************/
// DESTRUCTEUR
/*****************/

CRegleCC::~CRegleCC()
{

}


/*****************/
// METHODES
/*****************/


/********************************************************************/
/* Nom mthode : Executer                                     
/*                                                                  
/* Fonction : redfinition de la mthode dj prsente dans la classe
/*            mre CRegle pour le traitement de cette rgle                                                    
/* 
/*                                                           
/********************************************************************/

void CRegleCC::Executer(CDocument *pDoc)
{
	//charger le doc:
	((CMyAppApp*)AfxGetApp())->ChargerDoc((CMyAppDoc*)pDoc);


	Classification_Caracteristiques(pDoc);	

}



/********************************************************************/
/* Nom mthode : Serialize                                     
/*                                                                  
/* Fonction : redfinition de la mthode dj prsente dans la classe
/*            mre CRegle pour le traitement de cette rgle                                                    
/* 
/*                                                           
/********************************************************************/

void CRegleCC::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{

		ar<<type_org<<type_dest<<type_dest_sinon<<HLmin<<HLmax<<NBCCmin<<NBCCmax<<HHmin<<HHmax<<Lmin<<Lmax<<Hmin<<Hmax<<DPBmin<<DPBmax<<DNdgmin<<DNdgmax;

	}
	else
	{

		ar>>type_org>>type_dest>>type_dest_sinon>>HLmin>>HLmax>>NBCCmin>>NBCCmax>>HHmin>>HHmax>>Lmin>>Lmax>>Hmin>>Hmax>>DPBmin>>DPBmax>>DNdgmin>>DNdgmax;

	}

}


/********************************************************************/
/* Nom mthode : Classification_Caracteristiques                                     
/*                                                                  
/* Fonction : 
/*                                                            
/* 
/*                                                           
/********************************************************************/

void CRegleCC::Classification_Caracteristiques(CDocument *pDoc)
{
	//recuperer le pointeur sur l'application :
	CMyAppApp* pAppTemp = (CMyAppApp*)AfxGetApp();
	
	CAtelier *pAtelier = ((CMyAppDoc *)pDoc)->atelier;
	CObList *liste = (CObList *)pAtelier->liste_de_zones->liste;
	CObList *liste_types_classe;

	int typeLabo = 0;
	typeLabo = pAppTemp->projet->QuelLaboEnCours();

	switch (typeLabo)
	{
	case LABOTRAIT :
		liste_types_classe = pAppTemp->projet->getLaboTrait()->getListeTypesClasse();
		break;
	case LABOCLASSIF :
		liste_types_classe = pAppTemp->projet->getLaboClassif()->getListeTypesClasse();
		break;
	case LABOPROJET :
		liste_types_classe = pAppTemp->projet->getLaboProj()->getListeTypesClasse();
		break;
	default :
		AfxMessageBox("Error while loading types.");
		break;
	}

	CZone *z;
	POSITION pos_zone = liste->GetHeadPosition();

	while (pos_zone != NULL)
	{
		z = (CZone *)liste->GetNext(pos_zone);

		if (z->getClasse()->getNomClasse() == type_org)
		{

			double HL = z->getCaracCC()->getRapportLsurH();
			int NBCC = z->getCaracCC()->getNbElements();
			double HH = z->getCaracCC()->getRapportHsurHmoyCC();
			int L = z->getCaracCC()->getLargeurZone();
			int H = z->getCaracCC()->getHauteurZone();
			double DPB = z->getCaracCC()->getDensitePixBlanc();
			int DNdg = z->getCaracCC()->getDensiteNdgSurNbPix();

			bool cond1=true,cond2=true,cond3=true,cond4=true,cond5=true,cond6=true,cond7=true;

			if (HLmin != -1 && HLmax != -1) if (HL < HLmin || HL > HLmax) cond1=false;

			if (NBCCmin != -1 && NBCCmax != -1) if (NBCC < NBCCmin || NBCC > NBCCmax) cond2=false;

			if (HHmin != -1 && HHmax != -1) if (HH < HHmin || HH > HHmax) cond3=false;

			if (Lmin != -1 && Lmax != -1) if (L < Lmin || L > Lmax) cond4=false;

			if (Hmin != -1 && Hmax != -1) if (H < Hmin || H > Hmax) cond5=false;

			if (DPBmin != -1 && DPBmax != -1) if (DPB < DPBmin || DPB > DPBmax) cond6=false;

			if (DNdgmin != -1 && DNdgmax != -1) if (DNdg < DNdgmin || DNdg > DNdgmax) cond6=false;


			if (cond1&&cond2&&cond3&&cond4&&cond5&&cond6&&cond7)
			{
				//la zone remplit les conditions requises pour
				//passer du type "type_org"  "type_dest"
				z->setClasse(type_dest,liste_types_classe);

			}
			else
			{

				z->setClasse(type_dest_sinon,liste_types_classe);

			}
		}
	}

}



/********************************************************************/
/* Nom mthode : getDescription                                     
/*                                                                  
/* Fonction : redfinition de la mthode dj prsente dans la classe
/*            mre CRegle pour le traitement de cette rgle                                                    
/* 
/*                                                           
/********************************************************************/


CString CRegleCC::getDescription()
{
	CString desc,sinon;
	CString c1="",c2="",c3="",c4="",c5="",c6="",c7="";

	if (HLmin != -1 && HLmax != -1)		c1.Format(" si le rapport largeur/hauteur est compris entre %.2f et %.2f",HLmin,HLmax);

	if (NBCCmin != -1 && NBCCmax != -1) c2.Format(" si le nombre d'lments est compris entre %d et %d",NBCCmin,NBCCmax);

	if (HHmin != -1 && HHmax != -1)		c3.Format(" si le rapport hauteur moyenne/hauteur est compris entre %.2f et %.2f",HHmin,HHmax);

	if (Lmin != -1 && Lmax != -1)		c4.Format(" si la largeur est comprise entre %d et %d",Lmin,Lmax);

	if (Hmin != -1 && Hmax != -1)		c5.Format(" si la hauteur est comprise entre %d et %d",Hmin,Hmax);

	if (DPBmin != -1 && DPBmax != -1)	c6.Format(" si la densit de pixels blancs est comprise entre %.2f et %.2f",DPBmin,DPBmax);

	if (DNdgmin != -1 && DNdgmax != -1)	c7.Format(" si la moyenne des niveaux de gris est comprise entre %d et %d",DNdgmin,DNdgmax);

	sinon.Format(" sinon il devient de type %s",type_dest_sinon);
	desc.Format("le type %s devient de type %s" ,type_org,type_dest);

	return desc+c1+c2+c3+c4+c5+c6+c7+sinon;

}
