/****************************************************************/
// RegleCV.cpp: implementation of the CRegleCV class.
//
// Permet la gestion des rgles de classification selon les
// relations de voisinage
//
/*******************************************************/

/*****************/
// INCLUSION
/*****************/

#include "stdafx.h"
#include "myapp.h"
#include "RegleCV.h"
#include "myappdoc.h"
#include "MainFrm.h"
#include "constante.h"


/*****************/
// PRECOMPILATION
/*****************/


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


/*****************/
// CONSTRUCTEUR
/*****************/


CRegleCV::CRegleCV()
{
	balise = "RCV";
	liste_g = new CList<CString,CString>();
	liste_d = new CList<CString,CString>();
	liste_h = new CList<CString,CString>();
	liste_b = new CList<CString,CString>();
	liste_i = new CList<CString,CString>();
}



CRegleCV::CRegleCV(CString type_org, CList<CString,CString> *vg, CList<CString,CString> *vd, CList<CString,CString> *vh, CList<CString,CString> *vb, CList<CString,CString> *vi, CString type_dest, CString type_sinon)
{
	balise = "RCV";
	org = type_org;
	POSITION pos;

	liste_g = new CList<CString,CString>();
	pos = vg->GetHeadPosition();

	while (pos != NULL)
	{

		CString l = (CString )vg->GetNext(pos);
		liste_g->AddTail(l);

	}


	liste_d = new CList<CString,CString>();
	pos = vd->GetHeadPosition();

	while (pos != NULL)
	{

		CString l = (CString )vd->GetNext(pos);
		liste_d->AddTail(l);

	}


	liste_h = new CList<CString,CString>();
	pos = vh->GetHeadPosition();

	while (pos != NULL)
	{

		CString l = (CString )vh->GetNext(pos);
		liste_h->AddTail(l);

	}


	liste_b = new CList<CString,CString>();
	pos = vb->GetHeadPosition();

	while (pos != NULL)
	{

		CString l = (CString )vb->GetNext(pos);
		liste_b->AddTail(l);

	}


	liste_i = new CList<CString,CString>();
	pos = vi->GetHeadPosition();

	while (pos != NULL)
	{

		CString l = (CString )vi->GetNext(pos);
		liste_i->AddTail(l);

	}
	
	
	dest_si = type_dest;
	dest_sinon = type_sinon;

}



/*****************/
// DESTRUCTEUR
/*****************/

CRegleCV::~CRegleCV()
{

	delete liste_g;
	delete liste_d;
	delete liste_h;
	delete liste_b;
	delete liste_i;

}



/*****************/
// METHODES
/*****************/


/********************************************************************/
/* Nom mthode : Serialize                                     
/*                                                                  
/* Fonction : redfinition de la mthode dj prsente dans la classe
/*            mre CRegle pour le traitement de cette rgle                                                    
/* 
/*                                                           
/********************************************************************/

void CRegleCV::Serialize(CArchive& ar)
{
	
	if (ar.IsStoring())
	{

		POSITION pos;

		ar<<dest_si<<dest_sinon<<org;

		ar<<liste_b->GetCount();
		pos = liste_b->GetHeadPosition();
		while (pos != NULL)
			ar<<(CString)liste_b->GetNext(pos);

		ar<<liste_h->GetCount();
		pos = liste_h->GetHeadPosition();
		while (pos != NULL)
			ar<<(CString)liste_h->GetNext(pos);

		ar<<liste_d->GetCount();
		pos = liste_d->GetHeadPosition();
		while (pos != NULL)
			ar<<(CString)liste_d->GetNext(pos);

		ar<<liste_g->GetCount();
		pos = liste_g->GetHeadPosition();
		while (pos != NULL)
			ar<<(CString)liste_g->GetNext(pos);

		ar<<liste_i->GetCount();
		pos = liste_i->GetHeadPosition();
		while (pos != NULL)
			ar<<(CString)liste_i->GetNext(pos);

	}
	else
	{

		int i,nb;
		ar>>dest_si>>dest_sinon>>org;

		ar>>nb;
		for (i=0;i<nb;i++)
		{
			CString g;
			ar>>g;
			liste_b->AddTail(g);
		}

		ar>>nb;
		for (i=0;i<nb;i++)
		{
			CString g;
			ar>>g;
			liste_h->AddTail(g);
		}

		ar>>nb;
		for (i=0;i<nb;i++)
		{
			CString g;
			ar>>g;
			liste_d->AddTail(g);
		}

		ar>>nb;
		for (i=0;i<nb;i++)
		{
			CString g;
			ar>>g;
			liste_g->AddTail(g);
		}

		ar>>nb;
		for (i=0;i<nb;i++)
		{
			CString g;
			ar>>g;
			liste_i->AddTail(g);
		}

		
		
	}
}



/********************************************************************/
/* Nom mthode : Executer                                     
/*                                                                  
/* Fonction : redfinition de la mthode dj prsente dans la classe
/*            mre CRegle pour le traitement de cette rgle                                                    
/* 
/*                                                           
/********************************************************************/

void CRegleCV::Executer(CDocument *pDoc)
{
	//charger le doc:
	((CMyAppApp*)AfxGetApp())->ChargerDoc((CMyAppDoc*)pDoc);

	//appel de la fonction de classification selon le voisinage
	Cherche_Voisins(pDoc);
	Classification_Voisinage(pDoc, org,liste_g,liste_d,liste_h,liste_b,liste_i,dest_si,dest_sinon);
	
}



/********************************************************************/
/* Nom mthode : Cherche_Voisins                                     
/*                                                                  
/* Fonction : 
/*                                                                
/* 
/*                                                           
/********************************************************************/

void CRegleCV::Cherche_Voisins(CDocument *pDoc)
{
	CAtelier *pAtelier = ((CMyAppDoc *)pDoc)->atelier;
	CImage *Image = pAtelier->getImageCourante();
	CObList *liste = (CObList *)pAtelier->liste_de_zones->liste;

	int largeur = Image->getLargeur();
	int hauteur = Image->getHauteur();

	CPoint P1,P2;
	double delta_x,delta_y,distance;
	int xTL,yTL,xBR,yBR; 

	int min_G,min_D,min_H,min_B;
	double max_I;
	CZone *p_G,*p_D,*p_H,*p_B,*p_I;

	CRect *RG;
	CRect *RD;
	CRect *RH;
	CRect *RB;
	
	POSITION p1;
	POSITION p2;

	//CProgressBar pg("Dtection du voisinage...",liste->GetCount());
	((CMainFrame*)AfxGetMainWnd())->DemarrerProgressBar("Dtection du voisinage...",liste->GetCount());
	CWaitCursor wc;

	for (p1 = liste->GetHeadPosition();p1 != NULL;)
	{

		min_G = 20000;min_D = 20000;min_H = 20000;min_B = 20000;max_I=0;
		p_G=NULL;p_D=NULL;p_H=NULL;p_B=NULL;p_I=NULL;

		CZone *z = (CZone *)liste->GetNext(p1);
		P1 = *(z->getCdg());
		xTL = z->getRect()->TopLeft().x;
		yTL = z->getRect()->TopLeft().y;
		xBR = z->getRect()->BottomRight().x;
		yBR = z->getRect()->BottomRight().y; 

		RD = new CRect(P1.x,yTL,(largeur-P1.x),yBR);
		RG = new CRect(0,yTL,P1.x,yBR);
		RH = new CRect(xTL,0,xBR,P1.y);
		RB = new CRect(xTL,P1.y,xBR,(hauteur-P1.y));

		for (p2 = liste->GetHeadPosition(); p2 != NULL;)
		{

			CZone *zz = (CZone *)liste->GetNext(p2);

			if (z==zz)continue;

			P2 = *(zz->getCdg());
					
			delta_x = (P2.x - P1.x)*(P2.x - P1.x);
			delta_y = (P2.y - P1.y)*(P2.y - P1.y);
			distance = sqrt(delta_x + delta_y);

			CRect *rr=new CRect();


			//si le centre de gravite de la zone consideree est
			//et le rectangle droit
			//et si la distance qui le spare de l'autre cdg 
			//de la zone qui cherche ses voisins
			//est infrieure  la derniere distance mini trouvee
			if (rr->IntersectRect(RD,zz->getRect()) && distance < min_D)
			{

				for (int j=zz->getRect()->TopLeft().y;j<=zz->getRect()->BottomRight().y;j++)
				{

					CPoint p(zz->getRect()->TopLeft().x,j);

					if (RD->PtInRect(p))
					{

						//alors le pointeur sur le plus proche voisin de droite est ...
						p_D = zz;
						min_D = distance;
						break;

					}					
				}

				
			}

			//si le centre de gravite de la zone consideree est
			//et le rectangle gauche et ...
			if (rr->IntersectRect(RG,zz->getRect()) && distance < min_G)
			{

				for (int j=zz->getRect()->TopLeft().y;j<=zz->getRect()->BottomRight().y;j++)
				{

					CPoint p(zz->getRect()->BottomRight().x,j);
					if (RG->PtInRect(p))
					{

						//alors le pointeur sur le plus proche voisin de gauche est ...
						p_G = zz;
						min_G = distance;

					}					
				}
			}
			//si le centre de gravite de la zone consideree est
			//et le rectangle haut
			if (rr->IntersectRect(RH,zz->getRect()) && distance < min_H)
			{

				for (int i=zz->getRect()->TopLeft().x;i<=zz->getRect()->BottomRight().x;i++)
				{

					CPoint p(i,zz->getRect()->BottomRight().y);
					if (RH->PtInRect(p))
					{

						//alors le pointeur sur le plus proche voisin du haut est ...
						p_H = zz;
						min_H = distance;

					}					
				}
			}
			//si le centre de gravite de la zone consideree est
			//et le rectangle bas
			if (rr->IntersectRect(RB,zz->getRect()) && distance < min_B)
			{

				for (int i=zz->getRect()->TopLeft().x;i<=zz->getRect()->BottomRight().x;i++)
				{

					CPoint p(i,zz->getRect()->TopLeft().y);
					if (RB->PtInRect(p))
					{

						//alors le pointeur sur le plus proche voisin du bas est ...
						p_B = zz;
						min_B = distance;

					}					
				}
				
			}

			//si la zone est incluse dans une autre
			if (rr->IntersectRect(z->getRect(),zz->getRect()))
			{
				
				double rapport =  (double)(rr->Height()*rr->Width()) / (double)(z->getRect()->Height()*z->getRect()->Width());
				if (rapport > max_I)
				{
					p_I = zz;
					max_I = rapport;
				}
			}

			delete rr;
		}

		z->p_voisin_gauche = p_G;
		z->p_voisin_droite = p_D;
		z->p_voisin_haut = p_H;
		z->p_voisin_bas = p_B;
		z->p_inclus_dans = p_I;
		z->getCaracCG()->setDegreInclusion(max_I);

		//pg.m_progress.StepIt();
		((CMainFrame*)AfxGetMainWnd())->IncrementerProgressBar();

	}	
}



/********************************************************************/
/* Nom mthode : Classification_Voisinage                                     
/*                                                                  
/* Fonction : 
/*                                                                
/* 
/*                                                           
/********************************************************************/

void CRegleCV::Classification_Voisinage(CDocument *pDoc, CString type_org, CList<CString,CString> *vg, CList<CString,CString> *vd, CList<CString,CString> *vh, CList<CString,CString> *vb, CList<CString,CString> *vi, CString type_dest, CString type_sinon)
{
	//recuperer le pointeur sur l'application :
	CMyAppApp* pAppTemp = (CMyAppApp*)AfxGetApp();
	
	//si la zone qui est classe de type "type_org"
	//possde comme voisine de gauche une zone classe dans un des types de vg et
	//possde comme voisine de droite une zone classe dans un des types de vd et
	//...
	//alors elle devient de type "type_dest"

	CAtelier *pAtelier = ((CMyAppDoc *)pDoc)->atelier;
	CObList *liste = (CObList *)pAtelier->liste_de_zones->liste;
	CObList *liste_types_classe;

	int typeLabo = 0;
	typeLabo = pAppTemp->projet->QuelLaboEnCours();

	switch (typeLabo)
	{
	case LABOTRAIT :
		liste_types_classe = pAppTemp->projet->getLaboTrait()->getListeTypesClasse();
		break;
	case LABOCLASSIF :
		liste_types_classe = pAppTemp->projet->getLaboClassif()->getListeTypesClasse();
		break;
	case LABOPROJET :
		liste_types_classe = pAppTemp->projet->getLaboProj()->getListeTypesClasse();
		break;
	default :
		AfxMessageBox("Error while loading types.");
		break;
	}

	CString nomg,nomd,nomh,nomb,nomi; 
	CZone *tg,*td,*th,*tb,*ti,*z;

	int compteur;
	bool gauche,droite,haut,bas,inclus;

	do
	{
		compteur=0;
		POSITION pos_zone = liste->GetHeadPosition();

		while (pos_zone != NULL)
		{

			z = (CZone *)liste->GetNext(pos_zone);
			gauche=true;droite=true;haut=true;bas=true;inclus=true;

			if (z->getClasse()->getNomClasse() == type_org)
			{

				tg = (CZone *)z->p_voisin_gauche;
				td = (CZone *)z->p_voisin_droite;
				th = (CZone *)z->p_voisin_haut;
				tb = (CZone *)z->p_voisin_bas;
				ti = (CZone *)z->p_inclus_dans;

				
				if (tg!=NULL) nomg = tg->getClasse()->getNomClasse(); else nomg="RIEN";

				if (td!=NULL) nomd = td->getClasse()->getNomClasse(); else nomd="RIEN";

				if (th!=NULL) nomh = th->getClasse()->getNomClasse(); else nomh="RIEN";

				if (tb!=NULL) nomb = tb->getClasse()->getNomClasse(); else nomb="RIEN";

				if (ti!=NULL) nomi = ti->getClasse()->getNomClasse(); else nomi="RIEN";
				

				if (vg->GetCount()!=0) if (vg->Find(nomg)==NULL) gauche = false;

				if (vd->GetCount()!=0) if (vd->Find(nomd)==NULL) droite = false;

				if (vh->GetCount()!=0) if (vh->Find(nomh)==NULL) haut = false;

				if (vb->GetCount()!=0) if (vb->Find(nomb)==NULL) bas = false;

				if (vi->GetCount()!=0) if (vi->Find(nomi)==NULL) inclus = false;


				if (gauche&&droite&&haut&&bas&&inclus)
				{
					//la zone remplit les conditions requises pour
					//passer du type "type_org"  "type_dest"
					z->setClasse(type_dest,liste_types_classe);

					if (type_org != type_dest) compteur++;

				}
				else
				{
					//la zone ne remplit pas les conditions requises pour
					//passer du type "type_org"  "type_dest" donc elle
					//devient type_sinon
					z->setClasse(type_sinon,liste_types_classe);

					if (type_org != type_sinon) compteur++;

				}

			}
		}

	}

	while (compteur!=0);

}



/********************************************************************/
/* Nom mthode : getDescription                                     
/*                                                                  
/* Fonction : redfinition de la mthode dj prsente dans la classe
/*            mre CRegle pour le traitement de cette rgle                                                    
/* 
/*                                                           
/********************************************************************/

CString CRegleCV::getDescription()
{

	POSITION pos;
	CString g,d,h,b,i;

	pos = liste_g->GetHeadPosition();
	while (pos != NULL)
		g += (CString )liste_g->GetNext(pos)+" ";

	pos = liste_d->GetHeadPosition();
	while (pos != NULL)
		d += (CString )liste_d->GetNext(pos)+" ";

	pos = liste_h->GetHeadPosition();
	while (pos != NULL)
		h += (CString )liste_h->GetNext(pos)+" ";

	pos = liste_b->GetHeadPosition();
	while (pos != NULL)
		b += (CString )liste_b->GetNext(pos)+" ";

	pos = liste_i->GetHeadPosition();
	while (pos != NULL)
		i += (CString )liste_i->GetNext(pos)+" ";
	

	CString desc;

	desc.Format("le type %s devient de type %s si son voisin de gauche est dans {%s} et son voisin de droite est dans {%s} et son voisin du haut est dans {%s} et son voisin du bas est dans {%s} et est inclus dans {%s}	"
		,org,dest_si,g,d,h,b,i);

	return desc;

}
