/****************************************************************/
// RegleContour.cpp: implementation of the CRegleContour class.
//
// Permet de scnariser les traitements de dtection de composantes connexes
// et l'tude de leur taille
//
/*************************************************************************/


/*****************/
// INCLUSION
/*****************/

#include "stdafx.h"
#include "myapp.h"
#include "RegleContour.h"
#include "myappdoc.h"
#include "TypeClasse.h"
#include "constante.h"



/*****************/
// PRECOMPILATION
/*****************/


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


/*****************/
// CONSTRUCTEUR
/*****************/


CRegleContour::CRegleContour()
{
	balise = "RCO";
}

CRegleContour::CRegleContour(int PTE_CC_DX,int PTE_CC_DY,int GDE_CC_DX,int GDE_CC_DY)
{
	balise = "RCO";
	pte_dx = PTE_CC_DX;
	pte_dy = PTE_CC_DY;
	gde_dx = GDE_CC_DX;
	gde_dy = GDE_CC_DY;

}


/*****************/
// DESTRUCTEUR
/*****************/

CRegleContour::~CRegleContour()
{

}



/*****************/
// METHODES
/*****************/


/********************************************************************/
/* Nom mthode : Executer                                     
/*                                                                  
/* Fonction : redfinition de la mthode dj prsente dans la classe
/*            mre CRegle pour le traitement de cette rgle                                                    
/* 
/*                                                           
/********************************************************************/

void CRegleContour::Executer(CDocument *pDoc)
{
	//charger le doc:
	((CMyAppApp*)AfxGetApp())->ChargerDoc((CMyAppDoc*)pDoc);

	CAtelier *pAtelier = ((CMyAppDoc *)pDoc)->atelier;
	//recuperer le pointeur sur l'application :
	CMyAppApp* pAppTemp = (CMyAppApp*)AfxGetApp();

	CObList *liste_types_classe;

	int typeLabo = 0;
	typeLabo = pAppTemp->projet->QuelLaboEnCours();

	switch (typeLabo)
	{
	case LABOTRAIT :
		liste_types_classe = pAppTemp->projet->getLaboTrait()->getListeTypesClasse();
		break;
	case LABOCLASSIF :
		liste_types_classe = pAppTemp->projet->getLaboClassif()->getListeTypesClasse();
		break;
	case LABOPROJET :
		liste_types_classe = pAppTemp->projet->getLaboProj()->getListeTypesClasse();
		break;
	default :
		AfxMessageBox("Error while loading types.");
		break;
	}
	
	int indiceLastClasse = 0;
	int indiceNewClasse = 0;
	if (!liste_types_classe->IsEmpty())
	{
		CTypeClasse *typeTemp = (CTypeClasse*)liste_types_classe->GetTail();
		indiceLastClasse = typeTemp->getNumClasse();
		indiceNewClasse = indiceLastClasse+1;
	}

	bool isExiste_texte, isExiste_image, isExiste_bruit;
	
	CTypeClasse *texte=new CTypeClasse("TEXTE",RGB(0,0,255),indiceNewClasse);
	CTypeClasse *image=new CTypeClasse("IMAGE",RGB(0,255,255),indiceNewClasse+1);
	CTypeClasse *bruit=new CTypeClasse("BRUIT",RGB(0,0,0),indiceNewClasse+2);
	
	//verifie si les types n'existent pas dj
	switch (typeLabo)
	{
	case LABOTRAIT :
		if (pAppTemp->projet->getLaboTrait()->findElemListeTypesClasse("TEXTE"))
			isExiste_texte = true;
		else
			isExiste_texte = false;

		if (pAppTemp->projet->getLaboTrait()->findElemListeTypesClasse("IMAGE"))
			isExiste_image = true;
		else
			isExiste_image = false;

		if (pAppTemp->projet->getLaboTrait()->findElemListeTypesClasse("BRUIT"))
			isExiste_bruit = true;
		else
			isExiste_bruit = false;
		break;
	case LABOCLASSIF :
		if (pAppTemp->projet->getLaboClassif()->findElemListeTypesClasse("TEXTE"))
			isExiste_texte = true;
		else
			isExiste_texte = false;

		if (pAppTemp->projet->getLaboClassif()->findElemListeTypesClasse("IMAGE"))
			isExiste_image = true;
		else
			isExiste_image = false;

		if (pAppTemp->projet->getLaboClassif()->findElemListeTypesClasse("BRUIT"))
			isExiste_bruit = true;
		else
			isExiste_bruit = false;
		break;
	case LABOPROJET :
		if (pAppTemp->projet->getLaboProj()->findElemListeTypesClasse("TEXTE"))
			isExiste_texte = true;
		else
			isExiste_texte = false;

		if (pAppTemp->projet->getLaboProj()->findElemListeTypesClasse("IMAGE"))
			isExiste_image = true;
		else
			isExiste_image = false;

		if (pAppTemp->projet->getLaboProj()->findElemListeTypesClasse("BRUIT"))
			isExiste_bruit = true;
		else
			isExiste_bruit = false;
		break;
	}

	if (!isExiste_texte)
		liste_types_classe->AddTail(texte);
	if (!isExiste_image)
		liste_types_classe->AddTail(image);
	if (!isExiste_bruit)
		liste_types_classe->AddTail(bruit);

	
	pAtelier->DetectionContours(pte_dx,pte_dy,gde_dx,gde_dy);	
	
}


/********************************************************************/
/* Nom mthode : Serialize                                     
/*                                                                  
/* Fonction : redfinition de la mthode dj prsente dans la classe
/*            mre CRegle pour le traitement de cette rgle                                                    
/* 
/*                                                           
/********************************************************************/

void CRegleContour::Serialize(CArchive& ar)
{

	if (ar.IsStoring())
	{
		ar<<pte_dx<<pte_dy<<gde_dx<<gde_dy;
	}
	else
	{
		ar>>pte_dx>>pte_dy>>gde_dx>>gde_dy;
	}

}


/********************************************************************/
/* Nom mthode : getDescription                                     
/*                                                                  
/* Fonction : redfinition de la mthode dj prsente dans la classe
/*            mre CRegle pour le traitement de cette rgle                                                    
/* 
/*                                                           
/********************************************************************/

CString CRegleContour::getDescription()
{

	CString desc;
	desc.Format("dtection des contours et etiquettage en fonction de leur taille %d %d %d %d (image courante modifie)",pte_dx,pte_dy,gde_dx,gde_dy);
	return desc; 

}