/****************************************************************/
// Scenario.cpp: implementation of the CScenario class.
//
// Cette classe est prvue pour grer une liste d'objet de type Cregle.
// Un objet scenario contient donc une liste d'objets "rgles".
// Cette classe permet la gestion d'un scnario et permet notamment
// d'ajouter une rgle, d'effacer une ragle, de dplacer une rgle au sein
// du scnario, de sauvegarder et de charger le scnario
/*********************************************************************/


/***********************/
// INCLUSION
/***********************/

#include "stdafx.h"
#include "myapp.h"
#include "Scenario.h"
#include "mainfrm.h"

#include <afxrich.h>


/***********************/
// PRECOMPILATION
/***********************/

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


/***********************/
// CONSTRUCTEUR
/***********************/

CScenario::CScenario()
{
	regles = new CObList(10);
	modif =false;
}


/***********************/
// DESTRUCTEUR
/***********************/

CScenario::~CScenario()
{
	regles->RemoveAll();
	delete regles;
}



/***********************/
// METHODES
/***********************/


/********************************************************************/
/* Nom mthode : Ajouter                                     
/*                                                                  
/* Fonction : permet d'ajouter une rgle dans la liste du scnario
/*                                                                 
/* r : objet de type CRegle  ajouter
/*                                                           
/********************************************************************/
void CScenario::Ajouter(CRegle *r)
{

	modif =true;
	regles->AddTail(r);
	r->setNumero(regles->GetCount());
	Afficher();
	
}

/********************************************************************/
/* Nom mthode : AjouterScenario                                     
/*                                                                  
/* Fonction : permet d'ajouter les regles d'un scenario (scenarIn) au scenario
/*                                                                 
/*                                                           
/********************************************************************/
void CScenario::AjouterScenario(CScenario *scenarIn)
{
	POSITION pos = scenarIn->getListeRegles()->GetHeadPosition();

	while (pos != NULL)
	{
		CRegle *a = new CRegle();
		a = (CRegle *)scenarIn->getListeRegles()->GetNext(pos);
		Ajouter(a);
	}
	Afficher();
}



/********************************************************************/
/* Nom mthode : Sauvegarder                                     
/*                                                                  
/* Fonction : permet la sauvegarde de la liste sous la forme d'un scnario
/*                                                                 
/*                                                           
/********************************************************************/

bool CScenario::Sauvegarder(CString nomCompletFichier)
{
	CFile f;
	char buf[512];

	if( !f.Open( nomCompletFichier, CFile::modeCreate | CFile::modeWrite ) ) 
	{

	   return 1;

	}

	FILE *fprim;
	fprim = fopen(nomCompletFichier+".txt","w");

	CArchive ar( &f, CArchive::store, 512, buf );
	
	ar<<regles->GetCount();
	POSITION pos = regles->GetHeadPosition();

	while (pos != NULL)
	{

		CRegle *a = (CRegle *)regles->GetNext(pos);
		ar<<a->getBalise();
		fprintf(fprim,"%s\n",a->getDescription());
		
		a->Serialize(ar);
		
	}


	ar.Close();

	fclose(fprim);

	modif = false;

	return true;

}



/********************************************************************/
/* Nom mthode : Ouvrir                                     
/*                                                                  
/* Fonction : permet d'ouvrir un scnario existant et de lancer son chargement
/*                                                                 
/*                                                           
/********************************************************************/

int CScenario::Ouvrir(bool ecrase)
{
	static char BASED_CODE Filtre[] = "(scenario)|*.sce||";

 	CFileDialog a(TRUE,".sce",NULL,OFN_EXPLORER,Filtre,NULL);
//	a.m_ofn.lpstrInitialDir =  ((CMyAppApp*)(AfxGetApp()))->m_scenario;

	if (a.DoModal()==IDCANCEL)
	{

		return 0;

	}
	else
	{

		modif =false;
		return Charger(a.GetPathName(),ecrase);	

	}

}



/********************************************************************/
/* Nom mthode : Charger                                     
/*                                                                  
/* Fonction : permet de charger un scnario pralablement cr
/*                                                                 
/*                                                           
/********************************************************************/

int CScenario::Charger(CString nom, bool ecrase)
{
	CFile f;

	if( !f.Open( nom, CFile::modeRead ) ) 
	{

		AfxMessageBox(nom +"is unreadable",MB_OK);
		return 0;

	}


	if (ecrase) EffacerTout();	

	char buf[512];
	CArchive ar( &f, CArchive::load, 512, buf );
	int n;
	ar>>n;

	for (int i=0;i<n;i++)
	{

		CString bal;
		ar>>bal;
		CRegle *a;

		if (bal == "TYPE")
		{

			a = new CRegleType();

		}

		if (bal == "RCG")
		{

			a = new CRegleCG();

		}

		if (bal == "RCV")
		{

			a = new CRegleCV();

		}

		if (bal == "RCC")
		{

			a = new CRegleCC();

		}

		if (bal == "FUS")
		{

			a = new CRegleFusion();

		}

		if (bal == "RST")
		{

			a = new CRegleSupType();

		}

		if (bal == "RBI")
		{

			a = new CRegleBin();

		}

		if (bal == "RMO")
		{

			a = new CRegleMorph();

		}

		if (bal == "RCO")
		{

			a = new CRegleContour();

		}

		if (bal == "RIN")
		{

			a = new CRegleIntersection();

		}

		if (bal == "RPL")
		{

			a = new CReglePlage();

		}

		if (bal == "RPO")
		{

			a = new CReglePost();

		}

		if (bal == "RSR")
		{

			a = new CRegleSR();

		}

		if (bal == "TAO")
		{

			a = new CRegleTAO();

		}

		if (bal == "RAF")
		{

			a = new CRegleAnalyseFine();

		}

		if (bal == "RCAR")
		{

			a = new CRegleCarac();

		}

		if (bal == "RCT")
		{

			a = new CRegleChangeType();

		}

		a->Serialize(ar);
		Ajouter(a);

	}

	ar.Close();
	modif =false;
	return 1;

}



/********************************************************************/
/* Nom mthode : Executer                                     
/*                                                                  
/* Fonction : permet d'excuter les diffrentes action prsente 
/*            dans le scnario                                                     
/*                                                           
/********************************************************************/

int CScenario::Executer(CDocument *pDoc)
{
	//charger le doc:
	//((CMyAppApp*)AfxGetApp())->ChargerDoc((CMyAppDoc*)pDoc);
		
	POSITION pos = regles->GetHeadPosition();

	while (pos != NULL)
	{

		CRegle *r = (CRegle *)regles->GetNext(pos);
		r->Executer((CMyAppDoc*)pDoc);
	}

	//recharger le doc qui tait slectionn avant l'application du scnario
	((CMyAppApp*)AfxGetApp())->ChargerDocSelectionne();

	return 0;
}

/********************************************************************/
/* Nom mthode : regleDejaPresente(CString balise)                                     
/*                                                                  
/* Fonction : permet de savoir une regle est deja presente (analyse fine textuelle par exemple)
/*                                                             
/*                                                           
/********************************************************************/
bool CScenario::regleDejaPresente(CString balise)
{
	CString bal;
	POSITION pos = regles->GetHeadPosition();

	while (pos != NULL)
	{

		CRegle *r = (CRegle *)regles->GetNext(pos);
		bal = r->getBalise();
		if (bal == balise) return true;
	}

	return false;
}


/********************************************************************/
/* Nom mthode : getNbRegles                                     
/*                                                                  
/* Fonction : permet de savoir le nombre de rgle prsent dans un scnario
/*                                                             
/*                                                           
/********************************************************************/

int CScenario::getNbRegles()
{

	return regles->GetCount();

}


/********************************************************************/
/* Nom mthode : lire_libelle                                     
/*                                                                  
/* Fonction : permet de lire le libell exacte d'une action (rgle) 
/*            dans le fichier scnario                                                
/*                                                           
/********************************************************************/

CString CScenario::lire_libelle(FILE *f)
{

	CString libelle;

	fscanf(f,"\t<LIBELLE>\t%s\t<\\LIBELLE>\n",libelle);

	char temp[300];
	bool fin=false;

	while (!fin)
	{

		fscanf(f,"%s",temp);
		if (strcmp(temp,"<\\LIBELLE>")!=1) fin = true;
		else	libelle+=temp;

	}

	return libelle;

}



/********************************************************************/
/* Nom mthode : lire_numero                                     
/*                                                                  
/* Fonction : permet de connaitre la position de la rgle dans le scnario
/*                                                        
/*                                                           
/********************************************************************/

int CScenario::lire_numero(FILE *f)
{

	int numero;

	fscanf(f,"\t<NUMERO>\t%d\t<\\NUMERO>\n",&numero);
	
	return numero;

}


/********************************************************************/
/* Nom mthode : Echanger                                     
/*                                                                  
/* Fonction : permet de changer une rgle de place au sein du scnario
/*                                                        
/*                                                           
/********************************************************************/

bool CScenario::Echanger(int pos1, int pos2)
{
	bool bien_passe;
	POSITION posun;
	POSITION posdeux;

	if( ( posun = regles->FindIndex( pos1 )) != NULL  && ( posdeux = regles->FindIndex( pos2 )) != NULL )
	{

		CRegle *r1 = (CRegle *)regles->GetAt(posun);
		CRegle *r2 = (CRegle *)regles->GetAt(posdeux);
		
		regles->SetAt(posun,r2);
		regles->SetAt(posdeux,r1);

		bien_passe = true;

	}
	else
	{

		Beep(500,100);
		bien_passe = false;

	}

	modif = true;
	return bien_passe;

}


/********************************************************************/
/* Nom mthode : Afficher                                     
/*                                                                  
/* Fonction : permet l'affichage des rgles (du scnario) dans le bas
/*            du logiciel dans une ListBox                                            
/*                                                           
/********************************************************************/

void CScenario::Afficher()
{
	CListBox *pl=(CListBox *)((CMainFrame*)AfxGetMainWnd())->m_wndDlgTab.getM_Journal()->GetDlgItem(IDC_LOG);
	pl->ResetContent();
	POSITION pos = regles->GetHeadPosition();
	int max=0;

	while (pos != NULL)
	{

		CRegle *r = (CRegle *)regles->GetNext(pos);
		pl->AddString(r->getDescription());

		if (r->getDescription().GetLength() > max) max=r->getDescription().GetLength();

	}
    
	pl->SetHorizontalExtent(max*10);
	pl->SetCurSel(regles->GetCount()-1);

}



/********************************************************************/
/* Nom mthode : Effacer                                     
/*                                                                  
/* Fonction : permet d'effacer une rgle selectionne dans la ListBox
/*                                                     
/*                                                           
/********************************************************************/

bool CScenario::Effacer(int index)
{

	bool bien_passe;
	POSITION pos;

	if( ( pos = regles->FindIndex( index )) != NULL )
	{

		regles->RemoveAt(pos);
		bien_passe=true;

	}
	else
	{

		bien_passe=false;

	}

	modif = true;
	return bien_passe;

}


/********************************************************************/
/* Nom mthode : EffacerTout                                     
/*                                                                  
/* Fonction : permet d'effacer tout le contenu de la ListBox, c'est  dire
/*            d'effacer toutes les rgles                                       
/*                                                           
/********************************************************************/

bool CScenario::EffacerTout()
{

	modif = true;
	if (regles) regles->RemoveAll();
	return true;

}


