/****************************************************************/
// Zone.cpp: implementation of the CZone class.
//
// Classe destine  regrouper un ensemble de CElements.
// Permet de manipuler la liste des CElements contenus dans cette zone
// Elle permet galement d'obtenir certaine statistiques sur les lements
// contenus (hauteur moyenne, largeur moyenne, ...)
// Elle possde un pointeur sur les zones voisines (gauche, droite, haut, bas)
//
/*******************************************************************/


/***********************/
//  INCLUSION
/***********************/

#include "stdafx.h"
#include "MyApp.h"
#include "Zone.h"
#include "MyAppDoc.h"
#include "constante.h"

/***********************/
//  PRECOMPILATION
/***********************/

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


/***********************/
//  DEFINITIONS
/***********************/


#define BLANC 255
#define NOIR 0




/***********************/
//  CONSTRUCTEURS
/***********************/

/***************************************************************/ 
//constructeur de la classe CZone
/***************************************************************/
/*
CZone::CZone(CDocument* pDocContenant)
{
	m_DocContenant = pDocContenant;

	liste_elements	= new CObList;
	densite = 0;
	couche = 0;
	
}*/


/***************************************************************/ 
//constructeur de la classe CZone avec paramtre numro de zone
/***************************************************************/

CZone::CZone(int z, CDocument* pDocContenant)
{
	m_DocContenant = pDocContenant;

	p_voisin_gauche = NULL;
	p_voisin_droite = NULL;
	p_voisin_haut = NULL;
	p_voisin_bas = NULL;
	p_inclus_dans = NULL;

	liste_elements	= new CObList;
	zone = z;
	cdg = new CPoint;
	rect = NULL;	//new CRect;
	classe=NULL;
	densite = 0;	
	couche = 0;

	caracCC = new CCaracCC();
	caracCG = new CCaracCG();
	caracCV = new CCaracCV();
}




/***********************/
//  DESSTRUCTEURS
/***********************/


/***************************************************************/ 
//destructeur de la classe CZone
/***************************************************************/

CZone::~CZone()
{

	POSITION pos_elem = liste_elements->GetHeadPosition();

    while( pos_elem != NULL )
	{
		CElement *e = (CElement *)liste_elements->GetNext(pos_elem);
		delete e;
    }	

	liste_elements->RemoveAll();
	delete liste_elements;
	delete cdg;
	delete rect;
	//delete concepts;

	delete caracCC;
	delete caracCG;
	delete caracCV;
}

/********************************************************************/
/* Nom mthode : Ajouter_Element                                    */
/*                                                                  */
/* Fonction : ajoute un objet de type CElement  la liste de        */    
/*            CElement contenue dans la classe CZone et met  jour  */
/*            le rectangle englobant de la zone ainsi que son       */
/*            centre de gravit                                     */
/*                                                                  */
/* e : objet de type CElement                                       */
/*                                                                  */
/********************************************************************/


void CZone::Ajouter_Element(CElement *e)
{

	liste_elements->AddTail(e);

	if (rect==NULL)
	{
	
		rect = new CRect(e->getRect());

	}
	else
	{
		
		rect->UnionRect(rect,e->getRect());

	}

	*cdg = rect->CenterPoint();
	Calcul_Hauteur_Moyenne();
	Calcul_Largeur_Moyenne();
	
}


/********************************************************************/
/* Nom mthode : get_liste_elements                                 */
/*                                                                  */
/* Fonction : retourne un pointeur sur la liste des elements        */    
/*            contenus dans la zone                                 */
/*                                                                  */
/********************************************************************/
CObList * CZone::get_liste_elements()
{

	return liste_elements; 

}


/********************************************************************/
/* Nom mthode : calculerCaracteristiques                           */
/*                                                                  */
/* Fonction : calcule toutes les carac                              */
/*                                                                  */
/********************************************************************/
void CZone::calculerCaracteristiques()
{
	caracCC->CalculerToutesLesCaracCC(this);
	caracCG->CalculerToutesLesCaracCG(this);
	caracCV->CalculerToutesLesCaracCV(this);
}

/********************************************************************/
/* Nom mthode : Afficher_Elements                                  */
/*                                                                  */
/* Fonction : dessine les elements de la liste sur le contexte      */    
/*            de priphrique ecran en prenant en compte la         */
/*            valeur de zoom                                        */
/*                                                                  */
/* ecran : contexte de priphrique reprsentant la zone de l'cran */
/*         o il est possible de dessiner                           */
/* zoomcoeff : valeur du facteur de zoom                            */
/*                                                                  */
/********************************************************************/

void CZone::Afficher_Elements(CDC *ecran, double zoomcoeff)
{

	//position de la tete de liste des elements de la zone 
	POSITION pos_elem = liste_elements->GetHeadPosition();

    while( pos_elem != NULL )
	{
		
		//pointeur sur element
		CElement *e = (CElement *)liste_elements->GetNext(pos_elem);
		e->Afficher(ecran,zoomcoeff);

    }	

	/*Colorier(dest,BLANC);
	
	//position de la tete de liste des elements de la zone 
	POSITION pos_elem = liste_elements->GetHeadPosition();
    while( pos_elem != NULL )
	{
		//pointeur sur element
		CElement *e = (CElement *)liste_elements->GetNext(pos_elem);
		e->AfficheElement(dest);
    }	

	Entourer(dest,NOIR);*/

}


/********************************************************************/
/* Nom mthode : Afficher_Zones                                     */
/*                                                                  */
/* Fonction : dessine la zone sur le contexte de priphrique       */    
/*            ecran en prenant en compte la valeur de zoom et la    */
/*            couleur passe en paramtre                           */
/*                                                                  */
/* ecran : contexte de priphrique reprsentant la zone de l'cran */
/*         o il est possible de dessiner                           */
/* zoomcoeff : valeur du facteur de zoom                            */
/* color : couleur utilise pour le dessin                          */
/*                                                                  */
/********************************************************************/


void CZone::Afficher_Zones(CDC *ecran, double zoomcoeff,UINT color)
{

	int x1,y1,xf,yf;
	x1=rect->TopLeft().x;
	y1=rect->TopLeft().y;

	xf=rect->BottomRight().x;
	yf=rect->BottomRight().y;

	x1=floor((double)x1*zoomcoeff);
	y1=floor((double)y1*zoomcoeff);
	xf=ceil((double)xf*zoomcoeff);
	yf=ceil((double)yf*zoomcoeff);

	CRect r(x1,y1,xf,yf);
	ecran->Draw3dRect(r,color,color);
	
}


/********************************************************************/
/* Nom mthode : SurlignerZone                                      */
/*                                                                  */
/* Fonction : met en surbrillance la zone slectionne              */
/*                                                                  */
/* ecran : contexte de priphrique reprsentant la zone de l'cran */
/*         o il est possible de dessiner                           */
/* zoomcoeff : valeur du facteur de zoom                            */
/* color : couleur utilise pour le dessin                          */
/*                                                                  */
/********************************************************************/


void CZone::SurlignerZone(CDC *ecran, double zoomcoeff,UINT color, CPoint* offset)
{

	int x1,y1,xf,yf;
	x1=rect->TopLeft().x;
	y1=rect->TopLeft().y;

	xf=rect->BottomRight().x;
	yf=rect->BottomRight().y;

	x1=floor((double)((x1*zoomcoeff)-(offset->x)));
	y1=floor((double)((y1*zoomcoeff)-(offset->y)));
	xf=ceil((double)((xf*zoomcoeff)-(offset->x)));
	yf=ceil((double)((yf*zoomcoeff)-(offset->y)));

	CRect r(x1,y1,xf,yf);
	//r.OffsetRect(offset->x,offset->y);
	CBrush *brush=new CBrush(HS_BDIAGONAL,color);
	//CBrush *brush=new CBrush(color);
	//ecran->DrawEdge(r,EDGE_RAISED,BF_RECT);
	ecran->FillRect(r,brush);
	
}



/********************************************************************/
/* Nom mthode : getCdg                                             */
/*                                                                  */
/* Fonction : retourne le centre de gravit de la zone              */    
/*                                                                  */
/********************************************************************/

CPoint * CZone::getCdg()
{

	return cdg;

}



/********************************************************************/
/* Nom mthode : Colorier
/*                                                                  
/* Fonction : colorie le contenu de la zone avec la valeur de                 
/*            niveau de gris c               
/* 
/* dest : image sur laquele on colorie       
/* c : valeur de ndg utilise pour colorier       
/*                         
/********************************************************************/


void CZone::Colorier(CImage *dest, int c)
{
	
	if (c==INCOLOR) return;
	int x,y;
	for(y=rect->top;y<=rect->bottom;y++)
		for(x=rect->left;x<=rect->right;x++)		
			dest->setPixel(x,y,c);
	
}



/********************************************************************/
/* Nom mthode : Entourer
/*                                                                  
/* Fonction : dessine le rectangle englobant de la zone avec la valeur              
/*            de niveau de gris c               
/* 
/* dest : image sur laquele on colorie       
/* c : valeur de ndg utilise pour colorier       
/*                         
/********************************************************************/

void CZone::Entourer(CImage *dest, int c)
{
	int x,y;

	x=rect->left;
	for(y=rect->top;y<=rect->bottom;y++) dest->setPixel(x,y,c);

	x=rect->right;
	for(y=rect->top;y<=rect->bottom;y++) dest->setPixel(x,y,c);
	
	y=rect->top;
	for(x=rect->left;x<=rect->right;x++) dest->setPixel(x,y,c);
	
	y=rect->bottom;
	for(x=rect->left;x<=rect->right;x++) dest->setPixel(x,y,c);

}


/********************************************************************/
/* Nom mthode : get_hauteur_moyenne
/*                                                                  
/* Fonction : retourne la hauteur moyenne des elements inclus dans la zone            
/*                         
/********************************************************************/


int CZone::get_hauteur_moyenne()
{

	return hauteur_moyenne;

}


/********************************************************************/
/* Nom mthode : get_largeur_moyenne
/*                                                                  
/* Fonction : retourne la largeur moyenne des elements inclus dans la zone            
/*                         
/********************************************************************/


int CZone::get_largeur_moyenne()
{

	return largeur_moyenne;

}


/********************************************************************/
/* Nom mthode : getRect
/*                                                                  
/* Fonction : retourne le rectangle englobant la zone           
/*                         
/********************************************************************/


CRect * CZone::getRect()
{

	return rect;

}

/********************************************************************/
/* Nom mthode : setClasse
/*                                                                  
/* Fonction : on prcise la classse de la zone     
/*  
/* nom : nom de la classe
/* iste_types_classe : liste des diffrents types de classe
/*                       
/********************************************************************/


void CZone::setClasse(CString nom,CObList *liste_types_classe)
{

	POSITION pt;

	for (pt = liste_types_classe->GetHeadPosition();pt != NULL;)
	{
		CTypeClasse *t = (CTypeClasse *)liste_types_classe->GetNext(pt);

		if (t->getNomClasse() == nom)
		{
		
			classe = t;
			break;

		}
	}
	
}

/********************************************************************/
/* Nom mthode : getClasse
/*                                                                  
/* Fonction : retourne un pointeur sur le type de classe de la zone        
/*                         
/********************************************************************/


CTypeClasse * CZone::getClasse()
{
	classe->AssertValid();
	return classe;

}

/********************************************************************/
/* Nom mthode : getZone
/*                                                                  
/* Fonction : retourne le numro de la zone        
/*                         
/********************************************************************/


int CZone::getZone()
{

	return zone;

}

/********************************************************************/
/* Nom mthode : getZone
/*                                                                  
/* Fonction : prcise le numro de la zone avec la valeur z       
/*                         
/********************************************************************/


void CZone::setZone(int z)
{

	zone=z;

}

/********************************************************************/
/* Nom mthode : Calcul_Hauteur_Moyenne
/*                                                                  
/* Fonction : on calcul la hauteur moyenne et on met  jour l'attribut     
/*                         
/********************************************************************/

void CZone::Calcul_Hauteur_Moyenne()
{

	int temp=0;

	POSITION pos_elem = liste_elements->GetHeadPosition();

    while( pos_elem != NULL )
	{

		//pointeur sur element
		CElement *e = (CElement *)liste_elements->GetNext(pos_elem);
		temp += e->getRect()->Height();

    }	

	hauteur_moyenne = temp / liste_elements->GetCount();

}


/********************************************************************/
/* Nom mthode : Calcul_Largeur_Moyenne
/*                                                                  
/* Fonction : on calcul la largeur moyenne et on met  jour l'attribut     
/*                         
/********************************************************************/

void CZone::Calcul_Largeur_Moyenne()
{

	int temp=0;

	POSITION pos_elem = liste_elements->GetHeadPosition();

    while( pos_elem != NULL )
	{

		//pointeur sur element
		CElement *e = (CElement *)liste_elements->GetNext(pos_elem);
		temp += e->getRect()->Width();

    }	

	largeur_moyenne = temp / liste_elements->GetCount();

}



/********************************************************************/
/* Nom mthode : getNbElements
/*                                                                  
/* Fonction : retourne le nb d'elements contenus dans la zone   
/*             
/********************************************************************/


int CZone::getNbElements()
{

	return liste_elements->GetCount();

}


/********************************************************************/
/* Nom mthode : getDensite
/*                                                                  
/* Fonction : 
/*             
/********************************************************************/


double CZone::getDensite()
{

	return densite;

}


/********************************************************************/
/* Nom mthode : CalculDensite
/*                                                                  
/* Fonction : 
/*             
/********************************************************************/

double CZone::CalculDensite(CImage *im)
{
	double aire = rect->Height() * rect->Width();
	int x,y;
	int compteur=0;

	for (y=rect->top;y<rect->bottom;y++)
	{

		for (x=rect->left;x<rect->right;x++)
		{

			if (im->getPixel(x,y) == BLANC) compteur++;

		}

	}

	densite = ((double)compteur / aire);
	return densite;

}
